//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : meIDS.cs
//Description : Constants, types and function prototypes of the ME-iDS API 
//              for accessing the API in C#
//Note:         32bit library: meIDSmain.dll
//              64bit library: meIDSmain64.dll
//==============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace meIDS_CS
{
    /// <summary>
    /// Defines all constants, types, structures and methods to access the 
    /// ME-iDS driver system.
    /// </summary>
    public sealed class meIDS
    {
        #region Constants

        //==================================================================
        //  General
        //==================================================================

        public const int ME_VALUE_NOT_USED                          = 0x0;

        public const int ME_VALUE_DEFAULT                           = 0x0;

        public const int ME_NO_FLAGS                                = 0x0;
        
        public const int ME_VALUE_INVALID                           = -1;

        //==================================================================
        //  Defines common to access functions
        //==================================================================

        public const int ME_LOCK_RELEASE                            = 0x00010001;
        public const int ME_LOCK_SET                                = 0x00010002;
        public const int ME_LOCK_CHECK                              = 0x00010003;

        //==================================================================
        //  Defines meOpen function
        //==================================================================

        public const int ME_OPEN_NO_FLAGS                           = 0x0;

        //==================================================================
        //  Defines meClose function
        //==================================================================

        public const int ME_CLOSE_NO_FLAGS                          = 0x0;

        //==================================================================
        //  Defines meLockDriver function
        //==================================================================

        public const int ME_LOCK_DRIVER_NO_FLAGS                    = 0x0;

        //==================================================================
        //  Defines meLockDevice function
        //==================================================================

        public const int ME_LOCK_DEVICE_NO_FLAGS                    = 0x0;

        //==================================================================
        //  Defines meLockSubdevice function
        //==================================================================

        public const int ME_LOCK_SUBDEVICE_NO_FLAGS                 = 0x0;


        //==================================================================
        //  Defines common to error functions
        //==================================================================

        public const int ME_ERROR_MSG_MAX_COUNT                     = 256;

        //==================================================================
        //  Defines for meErrorSetDefaultProc function
        //==================================================================

        public const int ME_SWITCH_DISABLE                          = 0x00020001;
        public const int ME_SWITCH_ENABLE                           = 0x00020002;

        //==================================================================
        //Defines for meErrorGetLast function
        //==================================================================

        public const int ME_ERROR_GET_LAST_NO_FLAGS	                = 0x0;
        public const int ME_ERROR_GET_LAST_CLEAR                    = 0x1;

        //==================================================================
        //  Defines common to io functions
        //==================================================================

        public const int ME_REF_NONE                                = 0x0;
        public const int ME_REF_DIO_FIFO_LOW                        = 0x00030001;
        public const int ME_REF_DIO_FIFO_HIGH                       = 0x00030002;

        public const int ME_REF_CTR_PREVIOUS                        = 0x00040001;
        public const int ME_REF_CTR_INTERNAL_1MHZ                   = 0x00040002;
        public const int ME_REF_CTR_INTERNAL_10MHZ                  = 0x00040003;
        public const int ME_REF_CTR_EXTERNAL                        = 0x00040004;

        public const int ME_REF_AI_GROUND                           = 0x00050001;
        public const int ME_REF_AI_DIFFERENTIAL                     = 0x00050002;

        public const int ME_REF_AO_GROUND                           = 0x00060001;
        public const int ME_REF_AO_DIFFERENTIAL                     = 0x00060002;

        public const int ME_TRIG_CHAN_NONE                          = ME_VALUE_NOT_USED;
        public const int ME_TRIG_CHAN_DEFAULT                       = 0x00070001;
        public const int ME_TRIG_CHAN_SYNCHRONOUS                   = 0x00070002;

        public const int ME_TRIG_TYPE_NONE                          = ME_VALUE_NOT_USED;
        public const int ME_TRIG_TYPE_SW                            = 0x00080001;
        public const int ME_TRIG_TYPE_THRESHOLD                     = 0x00080002;
        public const int ME_TRIG_TYPE_WINDOW                        = 0x00080003;
        public const int ME_TRIG_TYPE_EDGE                          = 0x00080004;
        public const int ME_TRIG_TYPE_SLOPE                         = 0x00080005;
        public const int ME_TRIG_TYPE_EXT_DIGITAL                   = 0x00080006;
        public const int ME_TRIG_TYPE_EXT_ANALOG                    = 0x00080007;
        public const int ME_TRIG_TYPE_PATTERN                       = 0x00080008; // These two defines are the same
        public const int ME_TRIG_TYPE_PATTERN_MATCH                 = 0x00080008; // to preserve compatibility
        public const int ME_TRIG_TYPE_TIMER                         = 0x00080009;
        public const int ME_TRIG_TYPE_COUNT                         = 0x0008000A;
        public const int ME_TRIG_TYPE_FOLLOW                        = 0x0008000B;
        public const int ME_TRIG_TYPE_PATTERN_NO_MATCH              = 0x0008000C;
        public const int ME_TRIG_TYPE_EXT_DIGITAL_OR_COUNT          = 0x0008000D;
        public const int ME_TRIG_TYPE_PATTERN_MATCH_OR_COUNT        = 0x0008000E;
        public const int ME_TRIG_TYPE_PATTERN_NO_MATCH_OR_COUNT     = 0x0008000F;

        public const int ME_TRIG_EDGE_NONE                          = ME_VALUE_NOT_USED;
        public const int ME_TRIG_EDGE_ABOVE                         = 0x00090001;
        public const int ME_TRIG_EDGE_BELOW                         = 0x00090002;
        public const int ME_TRIG_EDGE_ENTRY                         = 0x00090003;
        public const int ME_TRIG_EDGE_EXIT                          = 0x00090004;
        public const int ME_TRIG_EDGE_RISING                        = 0x00090005;
        public const int ME_TRIG_EDGE_FALLING                       = 0x00090006;
        public const int ME_TRIG_EDGE_ANY                           = 0x00090007;

        public const int ME_TIMER_ACQ_START                         = 0x000A0001;
        public const int ME_TIMER_SCAN_START                        = 0x000A0002;
        public const int ME_TIMER_CONV_START                        = 0x000A0003;
        public const int ME_TIMER_FIO_TOTAL                         = 0x000A0004;
        public const int ME_TIMER_FIO_FIRST_PHASE                   = 0x000A0005;

        //==================================================================
        //  Defines for meIOFrequencyToTicks function
        //==================================================================

        public const int ME_IO_FREQUENCY_TO_TICKS_NO_FLAGS          = 0x0;
        public const int ME_IO_STREAM_FREQUENCY_TO_TICKS_NO_FLAGS   = ME_IO_FREQUENCY_TO_TICKS_NO_FLAGS;
        public const int ME_IO_FREQUENCY_TO_TICKS_MEPHISTO_SCOPE_OSCILLOSCOPE = 0x1;

        //==================================================================
        //  Defines for meIOIrqStart function
        //==================================================================

        public const int ME_IRQ_SOURCE_DIO_PATTERN                  = 0x000B0001;
        public const int ME_IRQ_SOURCE_DIO_MASK                     = 0x000B0002;
        public const int ME_IRQ_SOURCE_DIO_LINE                     = 0x000B0003;
        public const int ME_IRQ_SOURCE_DIO_OVER_TEMP                = 0x000B0004;
        public const int ME_IRQ_SOURCE_DIO_NORMAL_TEMP              = 0x000B0005;
        public const int ME_IRQ_SOURCE_DIO_CHANGE_TEMP              = 0x000B0006;

        public const int ME_IRQ_EDGE_NOT_USED                       = ME_VALUE_NOT_USED;
        public const int ME_IRQ_EDGE_RISING                         = 0x000C0001;
        public const int ME_IRQ_EDGE_FALLING                        = 0x000C0002;
        public const int ME_IRQ_EDGE_ANY                            = 0x000C0003;

        public const int ME_IO_IRQ_START_NO_FLAGS                   = 0x0;
        public const int ME_IO_IRQ_START_DIO_BIT                    = 0x1;
        public const int ME_IO_IRQ_START_DIO_BYTE                   = 0x2;
        public const int ME_IO_IRQ_START_DIO_WORD                   = 0x4;
        public const int ME_IO_IRQ_START_DIO_DWORD                  = 0x8;
        public const int ME_IO_IRQ_START_PATTERN_FILTERING          = 0x10;
        public const int ME_IO_IRQ_START_EXTENDED_STATUS            = 0x20;
        public const int ME_IO_IRQ_START_USE_PROPERTIES             = 0x40000000;

        //==================================================================
        //  Defines for meIOIrqWait function
        //==================================================================

        public const int ME_IO_IRQ_WAIT_NO_FLAGS                    = 0x0;
        public const int ME_IO_IRQ_WAIT_NORMAL_STATUS               = 0x1;
        public const int ME_IO_IRQ_WAIT_EXTENDED_STATUS             = 0x2;

        //==================================================================
        //  Defines for meIOIrqStop function
        //==================================================================

        public const int ME_IO_IRQ_STOP_NO_FLAGS                    = 0x0;

        //==================================================================
        //  Defines for meIOIrqSetCallback function
        //==================================================================

        public const int ME_IO_IRQ_SET_CALLBACK_NO_FLAGS            = 0x0;

        //==================================================================
        //  Defines for meIOResetDevice function
        //==================================================================

        public const int ME_IO_RESET_DEVICE_NO_FLAGS                = 0x0;

        //==================================================================
        //  Defines for meIOResetSubdevice function
        //==================================================================

        public const int ME_IO_RESET_SUBDEVICE_NO_FLAGS             = 0x0;

        //==================================================================
        //  Defines for meIOSingleConfig function
        //==================================================================

        public const int ME_SINGLE_CONFIG_DIO_INPUT                 = 0x000D0001;
        public const int ME_SINGLE_CONFIG_DIO_OUTPUT                = 0x000D0002;
        public const int ME_SINGLE_CONFIG_DIO_HIGH_IMPEDANCE        = 0x000D0003;
        public const int ME_SINGLE_CONFIG_DIO_SINK                  = 0x000D0004;
        public const int ME_SINGLE_CONFIG_DIO_SOURCE                = 0x000D0005;
        public const int ME_SINGLE_CONFIG_DIO_MUX32M                = 0x000D0006;
        public const int ME_SINGLE_CONFIG_DIO_DEMUX32               = 0x000D0007;
        public const int ME_SINGLE_CONFIG_DIO_BIT_PATTERN           = 0x000D0008;

        public const int ME_SINGLE_CONFIG_FIO_INPUT                 = 0x001E0001;
        public const int ME_SINGLE_CONFIG_FIO_OUTPUT                = 0x001E0002;
        public const int ME_SINGLE_CONFIG_FIO_HIGH_IMPEDANCE        = 0x001E0002;
        public const int ME_SINGLE_CONFIG_FIO_SINK                  = 0x001E0003;
        public const int ME_SINGLE_CONFIG_FIO_SOURCE                = 0x001E0004;

        public const int ME_SINGLE_CONFIG_CTR_8254_MODE_0           = 0x000E0001;
        public const int ME_SINGLE_CONFIG_CTR_8254_MODE_1           = 0x000E0002;
        public const int ME_SINGLE_CONFIG_CTR_8254_MODE_2           = 0x000E0003;
        public const int ME_SINGLE_CONFIG_CTR_8254_MODE_3           = 0x000E0004;
        public const int ME_SINGLE_CONFIG_CTR_8254_MODE_4           = 0x000E0005;
        public const int ME_SINGLE_CONFIG_CTR_8254_MODE_5           = 0x000E0006;

        public const int ME_IO_SINGLE_CONFIG_NO_FLAGS               = 0x00;
        public const int ME_IO_SINGLE_CONFIG_DIO_BIT                = 0x01;
        public const int ME_IO_SINGLE_CONFIG_DIO_BYTE               = 0x02;
        public const int ME_IO_SINGLE_CONFIG_DIO_WORD               = 0x04;
        public const int ME_IO_SINGLE_CONFIG_DIO_DWORD              = 0x08;
        public const int ME_IO_SINGLE_CONFIG_MULTISIG_LED_ON        = 0x10;
        public const int ME_IO_SINGLE_CONFIG_MULTISIG_LED_OFF       = 0x20;
        public const int ME_IO_SINGLE_CONFIG_AI_RMS                 = 0x40;

        //==================================================================
        //  Defines for meIOSingle function
        //==================================================================

        public const int ME_IO_SINGLE_NO_FLAGS                      = 0x0;

        public const int ME_DIR_INPUT                               = 0x000F0001;
        public const int ME_DIR_OUTPUT                              = 0x000F0002;
        public const int ME_DIR_SET_OFFSET                          = 0x000F0003;

        public const int ME_IO_SINGLE_TYPE_NO_FLAGS                 = 0x00;
        public const int ME_IO_SINGLE_TYPE_DIO_BIT                  = 0x01;
        public const int ME_IO_SINGLE_TYPE_DIO_BYTE                 = 0x02;
        public const int ME_IO_SINGLE_TYPE_DIO_WORD                 = 0x04;
        public const int ME_IO_SINGLE_TYPE_DIO_DWORD                = 0x08;
        public const int ME_IO_SINGLE_TYPE_TRIG_SYNCHRONOUS         = 0x10;
        public const int ME_IO_SINGLE_TYPE_NONBLOCKING              = 0x20;
        public const int ME_IO_SINGLE_TYPE_WRITE_NONBLOCKING        = ME_IO_SINGLE_TYPE_NONBLOCKING;
        public const int ME_IO_SINGLE_TYPE_READ_NONBLOCKING         = ME_IO_SINGLE_TYPE_NONBLOCKING;

        public const int ME_IO_SINGLE_TYPE_FIO_DIVIDER              = 0x40;
        public const int ME_IO_SINGLE_TYPE_FO_START_LOW             = 0x80;
        public const int ME_IO_SINGLE_TYPE_FO_START_SOFT            = 0x100;
        public const int ME_IO_SINGLE_TYPE_FI_LAST_VALUE            = 0x200;
        public const int ME_IO_SINGLE_TYPE_FO_UPDATE_ONLY           = 0x200; 
        public const int ME_IO_SINGLE_TYPE_FIO_TICKS_TOTAL          = 0x400;
        public const int ME_IO_SINGLE_TYPE_FIO_TICKS_FIRST_PHASE    = 0x800;
        public const int ME_IO_SINGLE_TYPE_FO_USE_PROPERTIES        = 0x40000000;


        //==================================================================
        //  Defines for meIOSingleTimeToTicks function
        //==================================================================
        public const int ME_IO_SINGLE_TIME_TO_TICKS_NO_FLAGS        = 0x0;

        //==================================================================
        //  Defines for meIOSingleTicksToTime function
        //==================================================================
        public const int ME_IO_SINGLE_TICKS_TO_TIME_NO_FLAGS        = 0x0;

        //==================================================================
        //  Defines for meIOSetChannelOffset function
        //==================================================================

        public const int ME_IO_SET_CHANNEL_OFFSET_NO_FLAGS          = 0x0;

        //==================================================================
        //  Defines for meIOStreamConfig function
        //==================================================================

        public const int ME_IO_STREAM_CONFIG_NO_FLAGS               = 0x0;
        public const int ME_IO_STREAM_CONFIG_BIT_PATTERN            = 0x1;
        public const int ME_IO_STREAM_CONFIG_WRAPAROUND             = 0x2;
        public const int ME_IO_STREAM_CONFIG_SAMPLE_AND_HOLD        = 0x4;

        public const int ME_IO_STREAM_CONFIG_INTERNAL_COUNTER       = unchecked((int)0x80000000);

        public const int ME_IO_STREAM_CONFIG_TYPE_NO_FLAGS          = 0x0;

        public const int ME_IO_STREAM_TRIGGER_TYPE_NO_FLAGS         = 0x0;

        //==================================================================
        //  Defines for meIOStreamRead function
        //==================================================================

        public const int ME_READ_MODE_BLOCKING                      = 0x00100001;
        public const int ME_READ_MODE_NONBLOCKING                   = 0x00100002;

        public const int ME_IO_STREAM_READ_NO_FLAGS                 = 0x0;
        public const int ME_IO_STREAM_READ_FRAMES                   = 0x1;

        //==================================================================
        //  Defines for meIOStreamWrite function
        //==================================================================

        public const int ME_WRITE_MODE_BLOCKING                     = 0x00110001;
        public const int ME_WRITE_MODE_NONBLOCKING                  = 0x00110002;
        public const int ME_WRITE_MODE_PRELOAD                      = 0x00110003;

        public const int ME_IO_STREAM_WRITE_NO_FLAGS                = 0x00000000;

        //==================================================================
        //  Defines for meIOStreamStart function
        //==================================================================

        public const int ME_IO_STREAM_START_NO_FLAGS                = 0x00000000;
        public const int ME_IO_STREAM_START_NONBLOCKING             = 0x00000020;
        
        public const int ME_START_MODE_BLOCKING                     = 0x00120001;
        public const int ME_START_MODE_NONBLOCKING                  = 0x00120002;

        public const int ME_IO_STREAM_START_TYPE_NO_FLAGS           = 0x0;
        public const int ME_IO_STREAM_START_TYPE_TRIG_SYNCHRONOUS   = 0x1;

        //==================================================================
        //  Defines for meIOStreamStop function
        //==================================================================

        public const int ME_IO_STREAM_STOP_NO_FLAGS                 = 0x00000000;
        public const int ME_IO_STREAM_STOP_NONBLOCKING              = 0x00000020;
        
        public const int ME_STOP_MODE_IMMEDIATE                     = 0x00130001;
        public const int ME_STOP_MODE_LAST_VALUE                    = 0x00130002;

        public const int ME_IO_STREAM_STOP_TYPE_NO_FLAGS            = 0x00000000;

        //==================================================================
        //  Defines for meIOStreamStatus function
        //==================================================================

        public const int ME_WAIT_NONE                               = 0x00140001;
        public const int ME_WAIT_IDLE                               = 0x00140002;
        public const int ME_WAIT_BUSY                               = 0x00140003;

        public const int ME_STATUS_INVALID                          = 0x00000000;
        public const int ME_STATUS_IDLE                             = 0x00150001;
        public const int ME_STATUS_BUSY                             = 0x00150002;
        public const int ME_STATUS_ERROR                            = 0x00150003;

        public const int ME_IO_STREAM_STATUS_NO_FLAGS               = 0x00000000;

        //==================================================================
        //  Defines for meIOStreamSetCallbacks function
        //==================================================================

        public const int ME_IO_STREAM_SET_CALLBACKS_NO_FLAGS        = 0x00000000;

        //==================================================================
        //  Defines for meIOStreamTimeToTicks function
        //==================================================================

        public const int ME_IO_TIME_TO_TICKS_NO_FLAGS               = 0x00000000;
        public const int ME_IO_STREAM_TIME_TO_TICKS_NO_FLAGS        = ME_IO_TIME_TO_TICKS_NO_FLAGS;
        public const int ME_IO_TIME_TO_TICKS_MEPHISTO_SCOPE_OSCILLOSCOPE = 0x00000001;

        //==================================================================
        //  Defines for module types
        //==================================================================

        public const int ME_MODULE_TYPE_MULTISIG_NONE               = 0x00000000;
        public const int ME_MODULE_TYPE_MULTISIG_DIFF16_10V         = 0x00160001;
        public const int ME_MODULE_TYPE_MULTISIG_DIFF16_20V         = 0x00160002;
        public const int ME_MODULE_TYPE_MULTISIG_DIFF16_50V         = 0x00160003;
        public const int ME_MODULE_TYPE_MULTISIG_CURRENT16_0_20MA   = 0x00160004;
        public const int ME_MODULE_TYPE_MULTISIG_RTD8_PT100         = 0x00160005;
        public const int ME_MODULE_TYPE_MULTISIG_RTD8_PT500         = 0x00160006;
        public const int ME_MODULE_TYPE_MULTISIG_RTD8_PT1000        = 0x00160007;
        public const int ME_MODULE_TYPE_MULTISIG_TE8_TYPE_B         = 0x00160008;
        public const int ME_MODULE_TYPE_MULTISIG_TE8_TYPE_E         = 0x00160009;
        public const int ME_MODULE_TYPE_MULTISIG_TE8_TYPE_J         = 0x0016000A;
        public const int ME_MODULE_TYPE_MULTISIG_TE8_TYPE_K         = 0x0016000B;
        public const int ME_MODULE_TYPE_MULTISIG_TE8_TYPE_N         = 0x0016000C;
        public const int ME_MODULE_TYPE_MULTISIG_TE8_TYPE_R         = 0x0016000D;
        public const int ME_MODULE_TYPE_MULTISIG_TE8_TYPE_S         = 0x0016000E;
        public const int ME_MODULE_TYPE_MULTISIG_TE8_TYPE_T         = 0x0016000F;
        public const int ME_MODULE_TYPE_MULTISIG_TE8_TEMP_SENSOR    = 0x00160010;

        //==================================================================
        //  Defines for meQuerySubdeviceCaps function
        //==================================================================

        public const int ME_CAPS_DIO_DIR_BIT                        = 0x00000001;
        public const int ME_CAPS_DIO_DIR_BYTE                       = 0x00000002;
        public const int ME_CAPS_DIO_DIR_WORD                       = 0x00000004;
        public const int ME_CAPS_DIO_DIR_DWORD                      = 0x00000008;
        public const int ME_CAPS_DIO_SINK_SOURCE                    = 0x00000010;
        public const int ME_CAPS_DIO_BIT_PATTERN_IRQ                = 0x00000020;
        public const int ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING       = 0x00000040;
        public const int ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING      = 0x00000080;
        public const int ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY          = 0x00000100;
        public const int ME_CAPS_DIO_OVER_TEMP_IRQ                  = 0x00000200;
        public const int ME_CAPS_DIO_NORMAL_TEMP_IRQ                = 0x00000400;


        public const int ME_CAPS_CTR_CLK_PREVIOUS                   = 0x00000001;
        public const int ME_CAPS_CTR_CLK_INTERNAL_1MHZ              = 0x00000002;
        public const int ME_CAPS_CTR_CLK_INTERNAL_10MHZ             = 0x00000004;
        public const int ME_CAPS_CTR_CLK_EXTERNAL                   = 0x00000008;

        public const int ME_CAPS_AI_TRIG_SIMULTANEOUS               = 0x00000001;
        public const int ME_CAPS_AI_FIFO                            = 0x00000002;
        public const int ME_CAPS_AI_FIFO_THRESHOLD                  = 0x00000004;
        public const int ME_CAPS_AI_SAMPLE_HOLD                     = 0x00000008;

        public const int ME_CAPS_AO_TRIG_SIMULTANEOUS               = 0x00000001;
        public const int ME_CAPS_AO_FIFO                            = 0x00000002;
        public const int ME_CAPS_AO_FIFO_THRESHOLD                  = 0x00000004;

        public const int ME_CAPS_FIO_SINK_SOURCE                    = 0x00000010;

        public const int ME_CAPS_EXT_IRQ_EDGE_RISING                = 0x00000001;
        public const int ME_CAPS_EXT_IRQ_EDGE_FALLING               = 0x00000002;
        public const int ME_CAPS_EXT_IRQ_EDGE_ANY                   = 0x00000004;

        //==================================================================
        //  Defines for meQuerySubdeviceCapsArgs function
        //==================================================================

        public const int ME_CAP_AI_FIFO_SIZE                        = 0x001D0000;
        public const int ME_CAP_AI_BUFFER_SIZE                      = 0x001D0001;
        public const int ME_CAP_AI_CHANNEL_LIST_SIZE                = 0x001D0002;
        public const int ME_CAP_AI_MAX_THRESHOLD_SIZE               = 0x001D0003;

        public const int ME_CAP_AO_FIFO_SIZE                        = 0x001F0000;
        public const int ME_CAP_AO_BUFFER_SIZE                      = 0x001F0001;
        public const int ME_CAP_AO_CHANNEL_LIST_SIZE                = 0x001F0002;
        public const int ME_CAP_AO_MAX_THRESHOLD_SIZE               = 0x001F0003;

        public const int ME_CAP_CTR_WIDTH                           = 0x00200000;

        public const int ME_CAP_FPGA_IN_FIFO_SIZE                   = 0x00300000;
        public const int ME_CAP_FPGA_IN_BUFFER_SIZE                 = 0x00300001;
        public const int ME_CAP_FPGA_OUT_FIFO_SIZE                  = 0x00300002;
        public const int ME_CAP_FPGA_OUT_BUFFER_SIZE                = 0x00300003;

        //==================================================================
        //  Defines common to query functions
        //==================================================================

        public const int ME_UNIT_INVALID                            = 0x00000000;
        public const int ME_UNIT_VOLT                               = 0x00170001;
        public const int ME_UNIT_AMPERE                             = 0x00170002;
        public const int ME_UNIT_ANY                                = 0x00170003;
        public const int ME_UNIT_HZ                                 = 0x00170004;

        public const int ME_TYPE_INVALID                            = 0x00000000;
        public const int ME_TYPE_AO                                 = 0x00180001;
        public const int ME_TYPE_AI                                 = 0x00180002;
        public const int ME_TYPE_DIO                                = 0x00180003;
        public const int ME_TYPE_DO                                 = 0x00180004;
        public const int ME_TYPE_DI                                 = 0x00180005;
        public const int ME_TYPE_CTR                                = 0x00180006;
        public const int ME_TYPE_EXT_IRQ                            = 0x00180007;
        public const int ME_TYPE_FIO                                = 0x00180008;
        public const int ME_TYPE_FO                                 = 0x00180009;
        public const int ME_TYPE_FI                                 = 0x0018000A;
        public const int ME_TYPE_FPGA                               = 0x0018000B;

        public const int ME_SUBTYPE_INVALID                         = 0x00000000;
        public const int ME_SUBTYPE_SINGLE                          = 0x00190001;
        public const int ME_SUBTYPE_STREAMING                       = 0x00190002;
        public const int ME_SUBTYPE_CTR_8254                        = 0x00190003;
        public const int ME_SUBTYPE_ANY                             = 0x00190004;
        public const int ME_SUBTYPE_CTR                             = 0x00190006;

        public const int ME_DEVICE_DRIVER_NAME_MAX_COUNT            = 64;
        public const int ME_DEVICE_NAME_MAX_COUNT                   = 64;

        public const int ME_DEVICE_DESCRIPTION_MAX_COUNT            = 256;

        public const int ME_BUS_TYPE_INVALID                        = 0x00000000;

        public const int ME_BUS_TYPE_ANY                            = 0x001A0000;
        public const int ME_BUS_TYPE_PCI                            = 0x001A0001;
        public const int ME_BUS_TYPE_USB                            = 0x001A0002;
        public const int ME_BUS_TYPE_LAN_PCI                        = 0x001A0101;
        public const int ME_BUS_TYPE_LAN_USB                        = 0x001A0102;

        public const int ME_PLUGGED_INVALID                         = 0x00000000;

        public const int ME_PLUGGED_ANY                             = 0x001B0000;
        public const int ME_PLUGGED_IN                              = 0x001B0001;
        public const int ME_PLUGGED_OUT                             = 0x001B0002;

        public const int ME_EXTENSION_TYPE_INVALID                  = 0x00000000;
        public const int ME_EXTENSION_TYPE_NONE                     = 0x001C0001;
        public const int ME_EXTENSION_TYPE_MUX32M                   = 0x001C0002;
        public const int ME_EXTENSION_TYPE_DEMUX32                  = 0x001C0003;
        public const int ME_EXTENSION_TYPE_MUX32S                   = 0x001C0004;
        public const int ME_EXTENSION_TYPE_ME1001                   = 0x001C0005;

        public const int ME_ACCESS_TYPE_INVALID                     = 0x00000000;
        public const int ME_ACCESS_TYPE_ANY                         = 0x001D0000;
        public const int ME_ACCESS_TYPE_LOCAL                       = 0x001D0001;
        public const int ME_ACCESS_TYPE_REMOTE                      = 0x001D0002;

        //==================================================================
        //  Defines for ConfigLoad
        //==================================================================
        public const int ME_CONF_LOAD_CUSTOM_DRIVER                 = 0x00001000;

        //==================================================================
        //  Defines for FPGA device
        //==================================================================
        
        public const uint ME_FPGA_FIRMWARE                          = 0xFF000001;
        public const uint ME_FPGA_SUBDEVICE                         = 0xFF000002;

        //==================================================================
        //  Defines for meUtilityPWM
        //==================================================================
        
        public const int ME_PWM_START_NO_FLAGS                      = 0x0;
        public const int ME_PWM_START_CONNECT_INTERNAL              = 0x1;

        //==================================================================
        //  Defines of flags for error handling
        //==================================================================
        
        public const int ME_ERRNO_CLEAR_FLAGS                       = 0x01;

        //==================================================================
        //  Defines of flags for device settings, used with property functions
        //==================================================================
        
        public const int ME_IO_PIN_VOLTAGE_LEVEL_5_VOLTS            = 0x0;
        public const int ME_IO_PIN_VOLTAGE_LEVEL_3_3_VOLTS          = 0x1;


        //==================================================================
        //  Defines of flags for property functions
        //==================================================================
        public const int ME_PROPERTY_TYPE_INVALID                   = 0x0;
        public const int ME_PROPERTY_TYPE_CONTAINER                 = 0x1;
        public const int ME_PROPERTY_TYPE_BOOL                      = 0x2;
        public const int ME_PROPERTY_TYPE_INT                       = 0x3;
        public const int ME_PROPERTY_TYPE_DOUBLE                    = 0x4;
        public const int ME_PROPERTY_TYPE_STRING                    = 0x5;
        public const int ME_PROPERTY_TYPE_DEFINE                    = 0x6;

        public const int ME_PROPERTY_ACCESS_READ_ONLY               = 0x0;
        public const int ME_PROPERTY_ACCESS_READ_WRITE              = 0x1;

        public const int ME_PROPERTY_BOOLEAN_FALSE                  = 0x00;
        public const int ME_PROPERTY_BOOLEAN_TRUE                   = 0x01;

        public const int ME_CALIBRATION_TYPE_FACTORY                = 0x001E0000;
        public const int ME_CALIBRATION_TYPE_USER_1                 = 0x001E0001;



        #endregion

        #region PropertyConstants

        //==================================================================
        //  System properties
        //==================================================================
        public const string ME_CONTAINER_NAME_ROOT = "\\\\";  // Root container (system container)
        
        public const string ME_ATTRIBUTE_NAME_NUMBER_OF_ELEMENTS    = "NumberOfElements";
        public const string ME_ATTRIBUTE_NAME_PROPERTY_NAME         = "PropertyName";
        public const string ME_ATTRIBUTE_NAME_PROPERTY_TYPE         = "PropertyType";
        public const string ME_ATTRIBUTE_NAME_PROPERTY_ACCESS       = "PropertyAccess";
        public const string ME_ATTRIBUTE_NAME_PROPERTY_DESCRIPTION  = "PropertyDescription";
        public const string ME_ATTRIBUTE_NAME_MIN_VALUE             = "MinValue";
        public const string ME_ATTRIBUTE_NAME_MAX_VALUE             = "MaxValue";
        public const string ME_ATTRIBUTE_NAME_LENGTH                = "Length";
        public const string ME_ATTRIBUTE_NAME_MAX_LENGTH            = "MaxLength";
        public const string ME_ATTRIBUTE_NAME_NUMBER_OF_SELECTIONS  = "NumberOfSelections";
        public const string ME_ATTRIBUTE_NAME_PREFIX_SELECTION      = "Selection";    // The selection with index # is "Selection#";
       
        
        //==================================================================
        //  System properties and containers
        //==================================================================

        public const string ME_PROPERTY_NAME_LIBRARY_VERSION        = "LibraryVersion";
        public const string ME_PROPERTY_NAME_MAIN_DRIVER_VERSION    = "MainDriverVersion";
        public const string ME_PROPERTY_NAME_NUMBER_OF_DEVICES      = "NumberOfDevices";
        public const string ME_CONTAINER_NAME_DEVICES               = "Devices";

        //==================================================================
        //  General device properties and containers - not device specific
        //==================================================================

        public const string ME_CONTAINER_NAME_PREFIX_DEVICE         = "Device";    // The container name for the device with index # is "Device#";
        public const string ME_PROPERTY_NAME_NAME                   = "Name";
        public const string ME_PROPERTY_NAME_DRIVER_NAME            = "DriverName";
        public const string ME_PROPERTY_NAME_DRIVER_VERSION         = "DriverVersion";
        public const string ME_PROPERTY_NAME_DESCRIPTION            = "Description";
        public const string ME_PROPERTY_NAME_SERIAL_NUMBER          = "SerialNumber";
        public const string ME_PROPERTY_NAME_BUS_TYPE               = "BusType";
        public const string ME_PROPERTY_NAME_ACCESS_TYPE            = "AccessType";
        public const string ME_PROPERTY_NAME_PLUGGED                = "Plugged";
        public const string ME_PROPERTY_NAME_FIRMWARE_SELECTABLE    = "FirmwareSelectable";
        public const string ME_PROPERTY_NAME_CURRENT_FIRMWARE_ID    = "CurrentFirmwareID";
        public const string ME_PROPERTY_NAME_NUMBER_OF_SUBDEVICES   = "NumberOfSubdevices";
        public const string ME_CONTAINER_NAME_SUBDEVICES            = "Subdevices";
        

        //==================================================================
        // Additional properties for PCI devices
        //==================================================================

        public const string ME_PROPERTY_NAME_PCI_VENDOR_ID          = "PCIVendorID";
        public const string ME_PROPERTY_NAME_PCI_DEVICE_ID          = "PCIDeviceID";
        public const string ME_PROPERTY_NAME_PCI_BUS_NUMBER         = "PCIBusNumber";
        public const string ME_PROPERTY_NAME_PCI_SLOT_NUMBER        = "PCISlotNumber";
        public const string ME_PROPERTY_NAME_PCI_FUNCTION_NUMBER    = "PCIFunctionNumber";
        

        //==================================================================
        // General sub-device properties and containers - not sub-device specific
        //==================================================================

        public const string ME_CONTAINER_NAME_PREFIX_SUBDEVICE      = "Subdevice";    // The container name for the sub-device with index # is "Subdevice#";
        public const string ME_PROPERTY_NAME_TYPE                   = "Type";
        public const string ME_PROPERTY_NAME_SUBTYPE                = "Subtype";
        public const string ME_PROPERTY_NAME_NUMBER_OF_CHANNELS     = "NumberOfChannels";
        public const string ME_PROPERTY_NAME_CONFIGURABLE           = "Configurable";
        public const string ME_PROPERTY_NAME_NUMBER_OF_CONFIGURATIONS = "NumberOfConfigurations";
        public const string ME_PROPERTY_NAME_CURRENT_CONFIGURATION  = "CurrentConfiguration";
        public const string ME_CONTAINER_NAME_CHANNELS              = "Channels";
        public const string ME_CONTAINER_NAME_PREFIX_CHANNEL        = "Channel"; // The container name for the channel with index # is "Channel#";
        public const string ME_CONTAINER_NAME_CONFIGURATIONS        = "Configurations";
        public const string ME_CONTAINER_NAME_PREFIX_CONFIGURATION  = "Configuration";    // The container name for the configuration with index # is "Configuration#";

        //==================================================================
        // Configuration properties
        //==================================================================

        public const string ME_PROPERTY_NAME_SUBDEVICE_TYPE         = "SubdeviceType";
        public const string ME_PROPERTY_NAME_SUBDEVICE_SUBTYPE      = "SubdeviceSubtype";
        public const string ME_PROPERTY_NAME_SUBDEVICE_NUMBER_OF_CHANNELS = "SubdeviceNumberOfChannels";

        //==================================================================
        // ME-5100 and ME-5001 - additional device properties
        //==================================================================

        public const string ME_PROPERTY_NAME_IO_PIN_VOLTAGE_LEVEL   = "IOPinVoltageLevel";
        public const string ME_PROPERTY_NAME_SUBDEVICES_0_AND_1_TERMINATED = "Subdevice0And1Terminated";
        public const string ME_PROPERTY_NAME_SUBDEVICES_2_AND_3_TERMINATED = "Subdevice2And3Terminated";

        //==================================================================
        // Additional sub-device properties for various sub-device types
        //==================================================================

        public const string ME_PROPERTY_NAME_TERMINATED             = "Terminated";
        public const string ME_PROPERTY_NAME_SYSTEM_CLOCK_FREQUENCY = "SystemClockFrequency";

        //==================================================================
        // Additional sub-device properties and containers for DI and DO sub-device types
        //==================================================================

        public const string ME_PROPERTY_NAME_TRIGGER_TYPE           = "TriggerType";
        public const string ME_PROPERTY_NAME_PREFIX_RISING_EDGE_BIT = "RisingEdgeBit";    // The name for the trigger condition on bit # is "RisingEdgeBit#";
        public const string ME_PROPERTY_NAME_PREFIX_FALLING_EDGE_BIT = "FallingEdgeBit";    // The name for the trigger condition on bit # is "FallngEdgeBit#";
        public const string ME_PROPERTY_NAME_INTERNAL_BUFFER_SIZE   = "InternalBufferSize";
        public const string ME_PROPERTY_NAME_SAMPLE_FREQUENCY       = "SampleFrequency";
        public const string ME_PROPERTY_NAME_SAMPLE_TIME            = "SampleTime";
        public const string ME_PROPERTY_NAME_START_TRIGGER_TYPE     = "StartTriggerType";
        public const string ME_PROPERTY_NAME_CONV_TRIGGER_TYPE      = "ConvTriggerType";
        public const string ME_PROPERTY_NAME_STOP_TRIGGER_TYPE      = "StopTriggerType";
        public const string ME_PROPERTY_NAME_START_STOP_PATTERN     = "StartStopPattern";
        public const string ME_PROPERTY_NAME_STOP_ON_COUNT          = "StopOnCount";
        public const string ME_PROPERTY_NAME_STOP_COUNT             = "StopCount";
        public const string ME_CONTAINER_NAME_INTERRUPT_CONDITION   = "InterruptCondition";
        public const string ME_CONTAINER_NAME_SINGLE_CONFIGURATION  = "SingleConfiguration";
        public const string ME_CONTAINER_NAME_EXT_DIGITAL_TRIGGER_CONDITION = "ExtDigitalTriggerCondition";
        public const string ME_CONTAINER_NAME_STREAMING_CONFIGURATION = "StreamingConfiguration";
        public const string ME_CONTAINER_NAME_START_AND_CONV_TRIGGER_CONDITION = "StartAndConvTriggerCondition";
        public const string ME_CONTAINER_NAME_STOP_TRIGGER_CONDITION = "StopTriggerCondition";
        public const string ME_CONTAINER_NAME_IRQ_CONFIGURATION      = "IRQConfiguration";

        //==================================================================
        // Additional sub-device properties for AI sub-device types
        //==================================================================
        
        public const string ME_PROPERTY_NAME_CALIBRATION             = "Calibration";
        public const string ME_PROPERTY_NAME_RISING_EDGE_TRIGGER_A1  = "RisingEdgeTriggerA1";
        public const string ME_PROPERTY_NAME_FALLING_EDGE_TRIGGER_A1 = "FallingEdgeTriggerA1";
        public const string ME_PROPERTY_NAME_RISING_EDGE_TRIGGER_A2  = "RisingEdgeTriggerA2";
        public const string ME_PROPERTY_NAME_FALLING_EDGE_TRIGGER_A2 = "FallingEdgeTriggerA2";
        public const string ME_PROPERTY_NAME_ACQUIRE_CHANNEL         = "AcquireChannel";

        //==================================================================
        // Additional sub-device properties and containers for DI sub-device types
        //==================================================================

        public const string ME_PROPERTY_NAME_RISING_EDGE_TRIGGER_A  = "RisingEdgeTriggerA";
        public const string ME_PROPERTY_NAME_FALLING_EDGE_TRIGGER_A = "FallingEdgeTriggerA";
        public const string ME_PROPERTY_NAME_USE_INTERNAL_TEST_COUNTER = "UseInternalTestCounter";
        public const string ME_PROPERTY_NAME_TRANSFER_SIZE          = "TransferSize";
        public const string ME_PROPERTY_NAME_IRQ_SOURCE             = "IRQSource";
        public const string ME_PROPERTY_NAME_BIT_PATTERN            = "BitPattern";
        public const string ME_PROPERTY_NAME_BIT_PATTERN_FILTERING  = "BitPatternFiltering";
        public const string ME_PROPERTY_CONTAINER_BIT_MASK          = "BitMask";

        //==================================================================
        // Additional sub-device properties and containers for DO sub-device types
        //==================================================================

        public const string ME_PROPERTY_NAME_RISING_EDGE_TRIGGER_B  = "RisingEdgeTriggerB";
        public const string ME_PROPERTY_NAME_FALLING_EDGE_TRIGGER_B = "FallingEdgeTriggerB";
        public const string ME_PROPERTY_NAME_IRQ_ON_EXCESS_TEMPERATURE = "IRQOnExcessTemperature";
        public const string ME_PROPERTY_NAME_IRQ_ON_NORMAL_TEMPERATURE = "IRQOnNormalTemperature";
        public const string ME_PROPERTY_NAME_STOP_PATTERN           = "StopPattern";
        public const string ME_PROPERTY_NAME_WRAPAROUND             = "Wraparound";
        public const string ME_PROPERTY_NAME_WRAPAROUND_LOOPS       = "WraparoundLoops";

        //==================================================================
        // Additional sub-device properties and containers for DO and FO sub-device types
        //==================================================================

        public const string ME_PROPERTY_NAME_OUTPUT_MODE            = "OutputMode";

        //==================================================================
        // Additional sub-device properties and containers for DIO sub-device types
        //==================================================================

        public const string ME_PROPERTY_NAME_DIRECTION              = "Direction";

        //==================================================================
        // Additional sub-device properties FI sub-device types
        //==================================================================

        public const string ME_PROPERTY_NAME_MIN_MEASURABLE_TOTAL_TICKS = "MinMeasurableTotalTicks";
        public const string ME_PROPERTY_NAME_MAX_MEASURABLE_TOTAL_TICK  = "MaxMeasurableTotalTicks";
        public const string ME_PROPERTY_NAME_MIN_MEASURABLE_FIRST_PHASE_TICKS = "MinMeasurableFirstPhaseTicks";
        public const string ME_PROPERTY_NAME_MAX_MEASURABLE_FIRST_PHASE_TICKS = "MaxMeasurableFirstPhaseTicks";
        public const string ME_PROPERTY_NAME_MIN_MEASURABLE_TOTAL_TIME  = "MinMeasurableTotalTime";
        public const string ME_PROPERTY_NAME_MAX_MEASURABLE_TOTAL_TIME  = "MaxMeasurableTotalTime";
        public const string ME_PROPERTY_NAME_MIN_MEASURABLE_FIRST_PHASE_TIME = "MinMeasurableFirstPhaseTime";
        public const string ME_PROPERTY_NAME_MAX_MEASURABLE_FIRST_PHASE_TIME = "MaxMeasurableFirstPhaseTime";

        //==================================================================
        // Additional sub-device properties for FO sub-device types
        //==================================================================

        public const string ME_PROPERTY_NAME_MIN_TOTAL_TICKS        = "MinTotalTicks";
        public const string ME_PROPERTY_NAME_MAX_TOTAL_TICK         = "MaxTotalTicks";
        public const string ME_PROPERTY_NAME_MIN_FIRST_PHASE_TICKS  = "MinFirstPhaseTicks";
        public const string ME_PROPERTY_NAME_MAX_FIRST_PHASE_TICKS  = "MaxFirstPhaseTicks";
        public const string ME_PROPERTY_NAME_MIN_TOTAL_TIME         = "MinTotalTime";
        public const string ME_PROPERTY_NAME_MAX_TOTAL_TIME         = "MaxTotalTime";
        public const string ME_PROPERTY_NAME_MIN_FIRST_PHASE_TIME   = "MinFirstPhaseTime";
        public const string ME_PROPERTY_NAME_MAX_FIRST_PHASE_TIME   = "MaxFirstPhaseTime";

        //==================================================================
        // FO sub-device channel properties
        //==================================================================

        public const string ME_PROPERTY_NAME_INVERTED               = "Inverted";
        public const string ME_PROPERTY_NAME_SYNCHRONOUS_START      = "SynchronousStart";
        public const string ME_PROPERTY_NAME_SOFT_START             = "SoftStart";
        
        //==================================================================
        // Counter sub-devices - additional sub-device properties
        //==================================================================

        public const string ME_PROPERTY_NAME_WC_WIDTH_IN_BITS       = "WidthInBits";
        public const string ME_PROPERTY_NAME_CLOCK_SOURCE           = "ClockSource";
        public const string ME_PROPERTY_NAME_COUNTER_MODE           = "CounterMode";

        #endregion


        #region ErrorCodes

        public const int ME_ERRNO_SUCCESS                       = 0;
        public const int ME_ERRNO_INVALID_DEVICE                = 1;
        public const int ME_ERRNO_INVALID_SUBDEVICE             = 2;
        public const int ME_ERRNO_INVALID_CHANNEL               = 3;
        public const int ME_ERRNO_INVALID_SINGLE_CONFIG         = 4;
        public const int ME_ERRNO_INVALID_REF                   = 5;
        public const int ME_ERRNO_INVALID_TRIG_CHAN             = 6;
        public const int ME_ERRNO_INVALID_TRIG_TYPE             = 7;
        public const int ME_ERRNO_INVALID_TRIG_EDGE             = 8;
        public const int ME_ERRNO_INVALID_TIMEOUT               = 9;
        public const int ME_ERRNO_INVALID_FLAGS                 = 10;
        public const int ME_ERRNO_OPEN                          = 11;
        public const int ME_ERRNO_CLOSE                         = 12;
        public const int ME_ERRNO_NOT_OPEN                      = 13;
        public const int ME_ERRNO_INVALID_DIR                   = 14;
        public const int ME_ERRNO_PREVIOUS_CONFIG               = 15;
        public const int ME_ERRNO_NOT_SUPPORTED                 = 16;
        public const int ME_ERRNO_SUBDEVICE_TYPE                = 17;
        public const int ME_ERRNO_USER_BUFFER_SIZE              = 18;
        public const int ME_ERRNO_LOCKED                        = 19;
        public const int ME_ERRNO_NO_MORE_SUBDEVICE_TYPE        = 20;
        public const int ME_ERRNO_TIMEOUT                       = 21;
        public const int ME_ERRNO_SIGNAL                        = 22;
        public const int ME_ERRNO_INVALID_IRQ_SOURCE            = 23;
        public const int ME_ERRNO_THREAD_RUNNING                = 24;
        public const int ME_ERRNO_START_THREAD                  = 25;
        public const int ME_ERRNO_CANCEL_THREAD                 = 26;
        public const int ME_ERRNO_NO_CALLBACK                   = 27;
        public const int ME_ERRNO_USED                          = 28;
        public const int ME_ERRNO_INVALID_UNIT                  = 29;
        public const int ME_ERRNO_INVALID_MIN_MAX               = 30;
        public const int ME_ERRNO_NO_RANGE                      = 31;
        public const int ME_ERRNO_INVALID_RANGE                 = 32;
        public const int ME_ERRNO_SUBDEVICE_BUSY                = 33;
        public const int ME_ERRNO_INVALID_LOCK                  = 34;
        public const int ME_ERRNO_INVALID_SWITCH                = 35;
        public const int ME_ERRNO_INVALID_ERROR_MSG_COUNT       = 36;
        public const int ME_ERRNO_INVALID_STREAM_CONFIG         = 37;
        public const int ME_ERRNO_INVALID_CONFIG_LIST_COUNT     = 38;
        public const int ME_ERRNO_INVALID_ACQ_START_TRIG_TYPE   = 39;
        public const int ME_ERRNO_INVALID_ACQ_START_TRIG_EDGE   = 40;
        public const int ME_ERRNO_INVALID_ACQ_START_TRIG_CHAN   = 41;
        public const int ME_ERRNO_INVALID_ACQ_START_TIMEOUT     = 42;
        public const int ME_ERRNO_INVALID_ACQ_START_ARG         = 43;
        public const int ME_ERRNO_INVALID_SCAN_START_TRIG_TYPE  = 44;
        public const int ME_ERRNO_INVALID_SCAN_START_ARG        = 45;
        public const int ME_ERRNO_INVALID_CONV_START_TRIG_TYPE  = 46;
        public const int ME_ERRNO_INVALID_CONV_START_ARG        = 47;
        public const int ME_ERRNO_INVALID_SCAN_STOP_TRIG_TYPE   = 48;
        public const int ME_ERRNO_INVALID_SCAN_STOP_ARG         = 49;
        public const int ME_ERRNO_INVALID_ACQ_STOP_TRIG_TYPE    = 50;
        public const int ME_ERRNO_INVALID_ACQ_STOP_ARG          = 51;
        public const int ME_ERRNO_SUBDEVICE_NOT_RUNNING         = 52;
        public const int ME_ERRNO_INVALID_READ_MODE             = 53;
        public const int ME_ERRNO_INVALID_VALUE_COUNT           = 54;
        public const int ME_ERRNO_INVALID_WRITE_MODE            = 55;
        public const int ME_ERRNO_INVALID_TIMER                 = 56;
        public const int ME_ERRNO_DEVICE_UNPLUGGED              = 57;
        public const int ME_ERRNO_USED_INTERNAL                 = 58;
        public const int ME_ERRNO_INVALID_DUTY_CYCLE            = 59;
        public const int ME_ERRNO_INVALID_WAIT                  = 60;
        public const int ME_ERRNO_CONNECT_REMOTE                = 61;
        public const int ME_ERRNO_COMMUNICATION                 = 62;
        public const int ME_ERRNO_INVALID_SINGLE_LIST           = 63;
        public const int ME_ERRNO_INVALID_MODULE_TYPE           = 64;
        public const int ME_ERRNO_INVALID_START_MODE            = 65;
        public const int ME_ERRNO_INVALID_STOP_MODE             = 66;
        public const int ME_ERRNO_INVALID_FIFO_IRQ_THRESHOLD    = 67;
        public const int ME_ERRNO_INVALID_POINTER               = 68;
        public const int ME_ERRNO_CREATE_EVENT                  = 69;
        public const int ME_ERRNO_LACK_OF_RESOURCES             = 70;
        public const int ME_ERRNO_CANCELLED                     = 71;
        public const int ME_ERRNO_RING_BUFFER_OVERFLOW          = 72;
        public const int ME_ERRNO_SOFTWARE_BUFFER_OVERFLOW      = ME_ERRNO_RING_BUFFER_OVERFLOW;
        public const int ME_ERRNO_RING_BUFFER_UNDERFLOW         = 73;
        public const int ME_ERRNO_SOFTWARE_BUFFER_UNDERFLOW     = ME_ERRNO_RING_BUFFER_UNDERFLOW;
        public const int ME_ERRNO_INVALID_IRQ_EDGE              = 74;
        public const int ME_ERRNO_INVALID_IRQ_ARG               = 75;
        public const int ME_ERRNO_INVALID_CAP                   = 76;
        public const int ME_ERRNO_INVALID_CAP_ARG_COUNT         = 77;
        public const int ME_ERRNO_INTERNAL                      = 78;
        public const int ME_ERRNO_VALUE_OUT_OF_RANGE            = 79;
        public const int ME_ERRNO_HARDWARE_FIFO_OVERFLOW        = 80;
        public const int ME_ERRNO_HARDWARE_BUFFER_OVERFLOW      = ME_ERRNO_HARDWARE_FIFO_OVERFLOW;
        public const int ME_ERRNO_HARDWARE_FIFO_UNDERFLOW       = 81;
        public const int ME_ERRNO_HARDWARE_BUFFER_UNDERFLOW     = ME_ERRNO_HARDWARE_FIFO_UNDERFLOW;
        public const int ME_ERRNO_CONFIG_LOAD_FAILED            = 82;
        public const int ME_ERRNO_PROPERTY_CONTAINER            = 83;
        public const int ME_ERRNO_PROPERTY_NOT_A_CONTAINER      = 84;
        public const int ME_ERRNO_PROPERTY_DATA_TYPE            = 85;
        public const int ME_ERRNO_PROPERTY_PATH                 = 86;
        public const int ME_ERRNO_PROPERTY_INDEX                = 87;
        public const int ME_ERRNO_PROPERTY_BUFFER_TOO_SMALL     = 88;
        public const int ME_ERRNO_PROPERTY_UNKNOWN              = 89;
        public const int ME_ERRNO_PROPERTY_ATTRIBUTE_UNKNOWN    = 90;
        public const int ME_ERRNO_PROPERTY_ATTRIBUTE_UNSUPPORTED = 91;
        public const int ME_ERRNO_PROPERTY_SELECTION_INVALID    = 92;
        public const int ME_ERRNO_PROPERTY_VALUE_INVALID        = 93;
        public const int ME_ERRNO_PROPERTY_READ_ONLY            = 94;

        //==============================================================================
        //  ME_ERRNO_INVALID_ERROR_NUMBER is always the error constant
        //  with the greatest number when new error codes are added
        //==============================================================================
        public const int ME_ERRNO_INVALID_ERROR_NUMBER          = 95;

        #endregion

        #region MEiDSTypes

        #region IOSingle

        /// <summary>
        ///    Contains the necessary parameters to 
        ///    read from or to write a single value to a subdevice.
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        public struct meIOSingle_t
        {
            /// <summary>
            ///    Index of the device to be accessed.
            /// </summary>
            public int iDevice;

            /// <summary>
            ///    Index of the sub-device to be accessed.
            /// </summary>
            public int iSubdevice;

            /// <summary>
            ///    Channel index resp. index of the group
            /// </summary>
            public int iChannel;

            /// <summary>
            ///    Direction of the single operation
            /// </summary>
            /// <remarks>
            /// ME_DIR_INPUT, ME_DIR_OUTPUT, ME_DIR_SET_OFFSET
            /// </remarks>
            public int iDir;

            /// <summary>
            /// Read resp. output a 32 bit digital value. Depending on port width always the lower significant bits are relevant
            /// </summary>
            public int iValue;

            /// <summary>
            /// Time-out value in milliseconds.
            /// </summary>
            public int iTimeOut;

            /// <summary>
            /// Extended settings depending on the operation
            /// </summary>
            public int iFlags;

            /// <summary>
            /// Error code returned by the particular entry
            /// </summary>
            public int iErrno;
        }

        #endregion


        /// <summary>
        /// Channel-list for analog and digital (bit pattern output) streaming operations
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        public struct meIOStreamConfig_t
        {
            /// <summary>
            /// Channel index. Depending on sub-device this can be analog inputs or outputs as well as a group of digital I/Os
            /// </summary>
            public int iChannel;

            /// <summary>
            /// Index of analog input or analog output range
            /// </summary>
            public int iStreamConfig;

            /// <summary>
            /// Defines the ground reference for analog inputs and outputs
            /// </summary>
            public int iRef;
            
            /// <summary>
            /// Extended settings
            /// </summary>
            public int iFlags;
        }

        /// <summary>
        /// Defines parameters for the start of streaming operations
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        public struct meIOStreamStart_t
        {
            /// <summary>
            /// Index of the device to be accessed
            /// </summary>
            public int iDevice;

            /// <summary>
            /// Index of the sub-device to be accessed
            /// </summary>
            public int iSubdevice;

            /// <summary>
            /// Blocking or Nonblocking start mode of the streaming operation
            /// </summary>
            /// <remarks>
            /// ME_START_MODE_BLOCKING (waits until the proper trigger signal occurs), ME_START_MODE_NONBLOCKING (Function returns immediately)
            /// </remarks>
            public int iStartMode;

            /// <summary>
            /// Time interval in milliseconds within the first trigger pulse must occur
            /// </summary>
            public int iTimeOut;

            /// <summary>
            /// Extended settings, Start particular subdevice only or several subdevices synchronously
            /// </summary>
            /// <remarks>
            /// ME_IO_STREAM_START_TYPE_NO_FLAGS default setting, ME_IO_STREAM_START_TYPE_TRIG_SYNCHRONOUS for Synchronous start
            /// </remarks>
            public int iFlags;

            /// <summary>
            /// If an error occurs, an error code will be set.
            /// </summary>
            public int iErrno;
        }

        /// <summary>
        /// StopList, defines end of input/output streaming operations
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        public struct meIOStreamStop_t
        {
            /// <summary>
            /// Index of the device to be accessed.
            /// </summary>
            public int iDevice;

            /// <summary>
            /// Index of the sub-device to be accessed.
            /// </summary>
            public int iSubdevice;

            /// <summary>
            /// Stop mode of the streaming operation
            /// </summary>
            /// <remarks>
            /// <see cref="ME_STOP_MODE_IMMEDIATE"/>: Streaming operation is stopped at once. 
            /// In case of an analog output 0V is attached to the pin.
            /// <see cref="ME_STOP_MODE_LAST_VALUE"/>: Output sub-device: operation is stopped on the last entry 
            /// in the DA-FIFO which is a defined value.
            /// Input sub-device: operation is stopped on the last entry from the channel-list.
            /// </remarks>
            public int iStopMode;

            /// <summary>
            /// <see cref="ME_IO_STREAM_STOP_TYPE_NO_FLAGS"/>: Default settings.
            /// </summary>
            /// <remarks>
            /// Output sub-device: all buffers are cleared.
            /// Input sub-device: Hardware buffer is cleared. No synchronization at all. 
            /// In mode <see cref="ME_STOP_MODE_IMMEDIATE"/> some data can be lost.
            /// </remarks>
            public int iFlags;

            /// <summary>
            /// If an error occurs, an error code will be set.
            /// </summary>
            public int iErrno;
        }


        /// <summary>
        /// Defines parameters to start and stop streaming operations.
        /// </summary>
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct meIOStreamTrigger_t
        {
            /// <summary>
            /// defines the trigger type for start of the whole operation (ME_TRIG_TYPE_SW, ME_TRIG_TYPE_EXT_ANALOG, ME_TRIG_TYPE_THRESHOLD...)
            /// </summary>
            public int iAcqStartTrigType;

            /// <summary>
            /// defines the trigger edge for start of a single conversion (ME_TRIG_EDGE_NONE, ME_TRIG_EDGE_RISING, ME_TRIG_EDGE_FALLING...)
            /// </summary>
            public int iAcqStartTrigEdge;

            /// <summary>
            /// choose whether triggering should be done separatly for each channel (standard) or if a channel should be started synchronously with other channels
            /// </summary>
            public int iAcqStartTrigChan;

            /// <summary>
            /// Offset time in number of ticks between "start" of the measurement and the first conversion.
            /// </summary>
            public int iAcqStartTicksLow;

            /// <summary>
            /// Higher significant part of the offset time (bits 63...32)
            /// </summary>
            public int iAcqStartTicksHigh;

            /// <summary>
            /// List of Start Arguments (threshold value in [V], slew rate in [V/Sample], lower threshold value of the window in [V])
            /// </summary>
            [MarshalAs(UnmanagedType.ByValArray, ArraySubType = UnmanagedType.I4, SizeConst = 10)]
            public int[] iAcqStartArgs;

            /// <summary>
            /// defines the trigger type for start of a scan (ME_TRIG_TYPE_TIMER, ME_TRIG_TYPE_FOLLOW, ME_TRIG_TYPE_EXT_DIGITAL...)
            /// </summary>
            public int iScanStartTrigType;

            /// <summary>
            /// Time interval in ticks between the start of two consecutive scans
            /// </summary>
            public int iScanStartTicksLow;

            /// <summary>
            /// Higher significant part of the scan-time (bits 63...32)
            /// </summary>
            public int iScanStartTicksHigh;

            /// <summary>
            /// reserved
            /// </summary>
            [MarshalAs(UnmanagedType.ByValArray, ArraySubType = UnmanagedType.I4, SizeConst = 10)]
            public int[] iScanStartArgs;

            /// <summary>
            /// defines the trigger type for start of a single conversion (ME_TRIG_TYPE_TIMER, ME_TRIG_TYPE_EXT_DIGITAL, ME_TRIG_TYPE_EXT_ANALOG)
            /// </summary>
            public int iConvStartTrigType;

            /// <summary>
            /// Chan interval in number of ticks between two conversions (Sample resp. output rate)
            /// </summary>
            public int iConvStartTicksLow;

            /// <summary>
            /// Higher significant part of the chan interval (bits 63...32)
            /// </summary>
            public int iConvStartTicksHigh;

            /// <summary>
            /// reserved
            /// </summary>
            [MarshalAs(UnmanagedType.ByValArray, ArraySubType = UnmanagedType.I4, SizeConst = 10)]
            public int[] iConvStartArgs;

            /// <summary>
            /// defines the trigger type for ending the scan (ME_TRIG_TYPE_NONE, ME_TRIG_TYPE_COUNT)
            /// </summary>
            public int iScanStopTrigType;

            /// <summary>
            /// Total number of conversions after which the scans will be ended and at the same time the measurement as a whole.
            /// </summary>
            public int iScanStopCount;

            /// <summary>
            /// MEphisto Scope in oscilloscope mode: trigger point in percent between 0%...100%
            /// </summary>
            [MarshalAs(UnmanagedType.ByValArray, ArraySubType = UnmanagedType.I4, SizeConst = 10)]
            public int[] iScanStopArgs;

            /// <summary>
            /// defines the trigger type for ending the whole operation (ME_TRIG_TYPE_NONE, ME_TRIG_TYPE_COUNT, ME_TRIG_TYPE_FOLLOW)
            /// </summary>
            public int iAcqStopTrigType;

            /// <summary>
            /// Number of scans (channel-list processings) after which the complete operation should be ended
            /// </summary>
            public int iAcqStopCount;

            /// <summary>
            /// reserved
            /// </summary>
            [MarshalAs(UnmanagedType.ByValArray, ArraySubType = UnmanagedType.I4, SizeConst = 10)]
            public int[] iArcStopArgs;

            /// <summary>
            /// ME_IO_STREAM_TRIGGER_TYPE_NO_FLAGS (default) (no flags available)
            /// </summary>
            public int iFlags;
        }

        #endregion
        

        #region Delegates

        /// <summary>
        /// Delegate type to be used for <see cref="meErrorSetUserProc"/>.
        /// </summary>
        /// <param name="functionName">
        /// String with the name of the faulty function where the error was detected.
        /// </param>
        /// <param name="errorCode">
        /// The error code.
        /// </param>
        /// <returns>
        /// No error possible. Return "0".
        /// </returns>
        public delegate int meErrorCB_t(string functionName, int errorCode);

        /// <summary>
        /// Delegate type to be used for <see cref="meIOStreamSetCallbacks"/>.
        /// </summary>
        /// <param name="device">
        /// Index of the device
        /// </param>
        /// <param name="subdevice">
        /// Index of the interrupt sub-device.
        /// </param>
        /// <param name="count">
        /// Input sub-device: number of values, which can be read
        /// Output sub-device: Free memory in the output buffer.
        /// </param>
        /// <param name="context">
        /// User-defined pointer.
        /// </param>
        /// <param name="errorCode">
        /// If an error occurs an error code will be returned.
        /// </param>
        /// <returns>
        /// Return the errorCode if the operation should stop when an error occured, otherwise return "0"
        /// </returns>
        public delegate int meIOStreamCB_t(int device, int subdevice, int count, IntPtr context, int errorCode);


        /// <summary>
        /// Delegate type to be used for <see cref="meIOIrqSetCallback"/>.
        /// </summary>
        /// <param name="device">
        /// Index of the device
        /// </param>
        /// <param name="subdevice">
        /// Index of the interrupt sub-device.
        /// </param>
        /// <param name="channel">
        /// Index of the interrupt channel within the sub-device
        /// </param>
        /// <param name="irqCount">
        /// Number of interrupts from the specified channel since starting.
        /// </param>
        /// <param name="value">
        /// Parameter returns the interrupt status.
        /// </param>
        /// <param name="context">
        /// User-defined pointer.
        /// </param>
        /// <param name="errorCode">
        /// Error code: see error reported by <see cref="meIOIrqWait"/>.
        /// </param>
        /// <returns>
        /// Return the errorCode if the operation should stop when an error occured, otherwise return "0"
        /// </returns>
        public delegate int meIOIrqCB_t(int device, int subdevice, int channel, int irqCount, int value, IntPtr context, int errorCode);

        #endregion

    #region MEiDSFunctions        
    /// <summary>
    /// Description of the library functions for using in C#-Projects.
    /// </summary>

        #region meOpen

        /// <summary>
        /// Initializes the function library.
        /// </summary>
        /// <param name="flags">Use <see cref="ME_OPEN_NO_FLAGS"/>.</param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_OPEN"/>: ME-iDS can not be properly opened. Usually driver is not loaded.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// Before the ME-iDS driver functions can be used this function must be called.
        /// Otherwise the function calls will fail.
        /// In case of an error <see cref="meClose"/> doesn't have to be called.
        /// </remarks>
        /// <seealso cref="meClose"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meOpen(int flags);

        #endregion


        #region meClose

        /// <summary>
        /// This function closes the connection to the function library.
        /// </summary>
        /// <param name="flags">
        /// Use <see cref="ME_CLOSE_NO_FLAGS"/>.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_CLOSE"/>: ME-iDS can not be properly closed.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// Call this function to release resources and to disconnect from the ME-iDS driver.
        /// </remarks>
        /// <seealso cref="meOpen"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meClose(int flags);

        #endregion


        #region meLockDriver

        /// <summary>
        /// Locks or unlocks the entire ME-iDS driver system.
        /// </summary>
        /// <param name="lockMode">
        /// The possible values are <see cref="ME_LOCK_SET"/>, <see cref="ME_LOCK_RELEASE"/>, <see cref="ME_LOCK_CHECK"/>.
        /// </param>
        /// <param name="flags">
        /// Use <see cref="ME_LOCK_DRIVER_NO_FLAGS"/>.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_LOCKED"/>: resource is locked by other application/task.
        /// <see cref="ME_ERRNO_USED"/>: resource is currently in use therefore lock can not be set.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// If another thread wants access to the driver system an error message is returned.
        /// </remarks>
        /// <seealso cref="meLockDevice"/>
        /// <seealso cref="meLockSubdevice"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meLockDriver(int lockMode, int flags);

        #endregion


        #region meLockDevice

        /// <summary>
        /// Locks or unlocks a ME-iDS device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="lockMode">
        /// The possible values are <see cref="ME_LOCK_SET"/>, <see cref="ME_LOCK_RELEASE"/>, <see cref="ME_LOCK_CHECK"/>.
        /// </param>
        /// <param name="flags">
        /// Use <see cref="ME_LOCK_DEVICE_NO_FLAGS"/>.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_LOCKED"/>: resource is locked by other application/task.
        /// <see cref="ME_ERRNO_USED"/>: resource is currently in use therefore lock can not be set.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// If another thread wants to access to a locked device an error message is returned.
        /// </remarks>
        /// <seealso cref="meLockDriver"/>
        /// <seealso cref="meLockSubdevice"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meLockDevice(int device, int lockMode, int flags);

        #endregion


        #region meLockSubdevice

        /// <summary>
        /// Locks or unlocks a sub-device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="lockMode">
        /// The possible values are <see cref="ME_LOCK_SET"/>, <see cref="ME_LOCK_RELEASE"/>, <see cref="ME_LOCK_CHECK"/>.
        /// </param>
        /// <param name="flags">
        /// Use <see cref="ME_LOCK_SUBDEVICE_NO_FLAGS"/>.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_LOCKED"/>: resource is locked by other application/task.
        /// <see cref="ME_ERRNO_USED"/>: resource is currently in use therefore lock can not be set.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// If another thread wants to access to a locked sub-device an error message is returned.
        /// </remarks>
        /// <seealso cref="meLockDevice"/>
        /// <seealso cref="meLockDriver"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meLockSubdevice(int device, int subdevice, int lockMode, int flags);

        #endregion


        #region meErrorGetLast

        /// <summary>
        /// Returns the last error code.
        /// </summary>
        /// <param name="errorCode">
        /// Variable that receives the error code.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_NO_FLAGS"/>: Default setting.
        /// <see cref="ME_ERRNO_CLEAR_FLAGS"/>: Do not report this error again.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: passed flags are not supported.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <seealso cref="meErrorGetLastMessage"/>
        /// <seealso cref="meErrorGetMessage"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meErrorGetLast(out int errorCode, int flags);

        #endregion


        #region meErrorGetLastMessage

        /// <summary>
        /// Returns the last error caused by an API function.
        /// </summary>
        /// <param name="buffer">
        /// Buffer to hold the error description text.
        /// </param>
        /// <param name="count">
        /// Buffer size in bytes for the error description text. Use the constant <see cref="ME_ERROR_MSG_MAX_COUNT"/>.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_INVALID_ERROR_MSG_COUNT"/>: reserved buffer is too small for description. 
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <seealso cref="meErrorGetMessage"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int meErrorGetLastMessage(StringBuilder buffer, int count);

        #endregion


        #region meErrorGetMessage

        /// <summary>
        /// Converts an error code returned from an API function to plain text.
        /// </summary>
        /// <param name="errorCode">
        /// The error code from the API function.
        /// </param>
        /// <param name="msg">
        /// Buffer to hold the error description text.
        /// </param>
        /// <param name="count">
        /// Buffer size in bytes for the error description text. Use the constant ME_ERROR_MSG_MAX_COUNT.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_INVALID_ERROR_NUMBER"/>: provided error code is not valid in ME-iDS.
        /// <see cref="ME_ERRNO_INVALID_ERROR_MSG_COUNT"/>: reserved buffer is too small for description.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <seealso cref="meErrorGetLastMessage"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int meErrorGetMessage(int errorCode, StringBuilder msg, int count);

        #endregion


        #region meErrorSetDefaultProc

        /// <summary>
        /// Installs a predefined global error logging routine for the entire ME-iDS.
        /// </summary>
        /// <param name="set">
        /// <see cref="ME_SWITCH_ENABLE"/>Installing the predefined error routine for global error logging.
        /// <see cref="ME_SWITCH_DISABLE"/>Uninstall the predefined error routine.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_INVALID_SWITCH"/>: passed action code is not supported.
        /// </returns>
        /// <remarks>
        /// This global error routine is automatically called if any function call returns an error. 
        /// Only one global error routine can be installed.
        /// The following infos are returned:
        /// -Name of the function causing the error
        /// -Short error description
        /// -Error code
        /// </remarks>
        /// <seealso cref="meErrorSetUserProc"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meErrorSetDefaultProc(int set);

        #endregion


        #region meErrorSetUserProc

        /// <summary>
        /// Installs a global user-defined error logging routine for the ME-iDS.
        /// </summary>
        /// <param name="errorProc">
        /// Delegate to the user-defined error logging routine.
        /// See <see cref="meErrorCB_t"/>
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// </returns>
        /// <remarks>
        /// This function is automatically called if any function call returns an error.
        /// Passing a NULL will uninstall a previously installed error routine.
        /// The following infos are returned:
        /// -Name of the function causing the error
        /// -Error code
        /// </remarks>
        /// <seealso cref="meErrorSetDefaultProc"/>
        /// <seealso cref="meErrorGetMessage"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meErrorSetUserProc(meErrorCB_t errorProc);

        #endregion


        #region meIOIrqSetCallback

        /// <summary>
        /// Installs a callback function which waits for an interrupt in the background.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="callback">
        /// Delegate to the user-defined callback function.
        /// See <see cref="meIOIrqCB_t"/>
        /// </param>
        /// <param name="callbackContext">
        /// User-defined pointer passed to the callback function.
        /// Can be used to pass a reference of the own instance through the callback routine
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_IRQ_SET_CALLBACK_NO_FLAGS"/>: No flags in use.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_CHANNEL"/>: no channel available on sub-device.
        /// <see cref="ME_ERRNO_LOCKED"/>: sub-device is protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// <see cref="ME_ERRNO_START_THREAD"/>: creating callback thread failed. (Linux only)
        /// <see cref="ME_ERRNO_THREAD_RUNNING"/>: callback thread is already running. (Windows only)
        /// </returns>
        /// <remarks>
        /// To deinstall/cancel the callback function (all registered instances) 
        /// for the selected sub-device call meIOIrqSetCallback() and pass null in 
        /// <paramref name="callback"/>.
        /// </remarks>
        /// <example>
        /// See also (<a href="http://msdn.microsoft.com/en-gb/magazine/cc301501.aspx">Call Unmanaged DLLs from C#, Killing Processes Cleanly</a>).
        /// <code>
        /// // the form contains a TextBox to display the values given by the callback function
        /// public partial class Form1 : Form
        /// {
        ///     meIDS.meIOIrqCB_t IrqCallback = new meIDS.meIOIrqCB_t(meIOIrqCB);   //create a delegate for Callback function
        ///     GCHandle objectHandle = new GCHandle();                             //global variable 
        ///    
        ///     private void MyStartIrqFunction(int deviceIndex, int subdeviceIndex)
        ///     {
        ///         objectHandle = GCHandle.Alloc(this);
        ///         
        ///         int meError = meIDS.meIOIrqSetCallback(deviceIndex,             //Device index
        ///                                                subdeviceIndex,          //Subdevice index
        ///                                                IrqCallback,             //Callback routine
        ///                                                (IntPtr)objectHandle,    //Callback context, GCHandle used to wrap instance of Form 
        ///                                                meIDS.ME_IO_IRQ_SET_CALLBACK_NO_FLAGS); //No flags in use
        ///
        ///         if (meError == meIDS.ME_ERRNO_SUCCESS)
        ///         {
        ///             //callback function is armed, meIDS.meIOIrqStart() may be called
        ///             meError = meIDS.meIOIrqStart(deviceIndex,                   //Device index
        ///                                          subdeviceIndex,                //Subdevice index
        ///                                          ...
        ///                                         );
        ///             ...
        ///         }
        ///     }
        ///
        ///     private static int meIOIrqCB(int idxDevice, int idxSubdevice, int idxChannel,
        ///                                         int irqCount, int value, IntPtr pContext, int errorCode)
        ///     {
        ///         if (errorCode == meIDS.ME_ERRNO_SUCCESS)
        ///         {
        ///             GCHandle objectHandle = GCHandle.FromIntPtr(pContext);          //get objectHandle of our instance
        ///             Form1 frm = (Form1)objectHandle.Target;
        ///             frm.DisplayValues(idxDevice, idxSubdevice, idxChannel, irqCount, value, pContext, errorCode);  //display values in textBox1
        ///         }
        ///         return (0);
        ///     }
        ///     
        ///     // This delegate enables asynchronous calls for setting the text in a textBox1
        ///     private delegate void DisplayValuesDelegate(int idxDevice, int idxSubdevice, int idxChannel,
        ///                                    int irqCount, int value, IntPtr context, int errorCode);
        /// 
        ///     // This method demonstrates a pattern for making thread-safe calls on a Windows Forms control.
        ///     private void DisplayValues(int idxDevice, int idxSubdevice, int idxChannel,
        ///                                    int irqCount, int value, IntPtr context, int errorCode)
        ///     {
        ///         // InvokeRequired compares the thread ID of the calling thread to the thread ID of the creating thread.
        ///         // If these threads are different, it returns true.
        ///         if (textBox1.InvokeRequired)
        ///         {
        ///             //The calling thread is different from the thread that created the TextBox.
        ///             //The method creates a SetTextCallback and calls itself asynchronously using the Invoke method.
        ///             DisplayValuesDelegate dispDelegate = new DisplayValuesDelegate(DisplayValues);
        ///             this.Invoke(dispDelegate, new object[] {idxDevice,  idxSubdevice, idxChannel, irqCount, value, context, errorCode});
        ///         }
        ///         else
        ///         {
        ///             //The calling thread is the same as the thread that created the TextBox
        ///             //The Text property is set directly.
        ///             textBox1.AppendText("Device index: " + idxDevice.ToString()
        ///                        + ",  Subdevice index: " + idxSubdevice.ToString()
        ///                        + String.Format(",  Value hex: 0x{0:X}", value)      //display value in hexadecimal
        ///                        + ",  dec: " + value.ToString()                      //display value in decimal
        ///                        + Environment.NewLine);
        ///         }
        ///     }
        ///     
        ///     private void MyStopIrqFunction(int deviceIndex, int subdeviceIndex)
        ///     {
        ///         //disable generating IRQs
        ///         meIDS.meIOIrqStop(deviceIndex, 
        ///                           subdevIndex, 
        ///                           0, 
        ///                           meIDS.ME_IO_IRQ_STOP_NO_FLAGS);
        ///                           
        ///         //Don't forget to call Free for objectHandle
        ///         if (objectHandle.IsAllocated)                   
        ///         {
        ///             objectHandle.Free();                        // do this when callback cannot be called anymore
        ///         }
        ///     }
        /// }
        /// </code>
        /// </example>
        /// <seealso cref="meIOIrqWait"/>
        /// <seealso cref="meIOIrqStart"/>
        /// <seealso cref="meIOIrqStop"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOIrqSetCallback(int device, int subdevice, meIOIrqCB_t callback, IntPtr callbackContext, int flags);

        #endregion


        #region meIOIrqStart

        /// <summary>
        /// Starts the interrupt handler for the given interrupt sub-device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="channel">
        /// Index of the interrupt channel within the selected sub-device.
        /// </param>
        /// <param name="irqSource">
        /// Selection of interrupt source
        /// <see cref="ME_IRQ_SOURCE_DIO_LINE"/>: Interrupt source is a dedicated external interrupt input.
        /// <see cref="ME_IRQ_SOURCE_DIO_PATTERN"/>: Operation mode "Bit Pattern Match". 
        /// When the current bit pattern at the digital port matches the reference bit pattern to be passed in <paramref name="irqArg"/> an interrupt is triggered.
        /// <see cref="ME_IRQ_SOURCE_DIO_MASK"/>: Operation mode "Bit Pattern Change". 
        /// On change of at least one bit, masked as "sensitive" an interrupt is triggered.
        /// <see cref="ME_IRQ_SOURCE_DIO_OVER_TEMP"/>: On overheating of the driver circuit an interrupt is triggered.
        /// </param>
        /// <param name="irqEdge">
        /// Selection of the edge on which an interrupt should be triggered.
        /// <see cref="ME_IRQ_EDGE_NOT_USED"/>: Choosing an edge is not supported
        /// <see cref="ME_IRQ_EDGE_RISING"/>: Interrupt on rising edge
        /// <see cref="ME_IRQ_EDGE_FALLING"/>: Interrupt on falling edge
        /// <see cref="ME_IRQ_EDGE_ANY"/>: Interrupt on rising or falling edge
        /// </param>
        /// <param name="irqArg">
        /// Argument to configure the modes "Bit Pattern Match" and "Bit Pattern Change"
        /// In other cases pass the value "0" here.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_IRQ_START_NO_FLAGS"/>: No flags in use. Default settings will be used.
        /// <see cref="ME_IO_IRQ_START_DIO_BIT"/>: The reference bit pattern is one bit wide.
        /// <see cref="ME_IO_IRQ_START_DIO_BYTE"/>: The reference bit pattern is one byte wide (8 bit).
        /// <see cref="ME_IO_IRQ_START_DIO_WORD"/>: The reference bit pattern is one word wide (16 bit).
        /// <see cref="ME_IO_IRQ_START_DIO_DWORD"/>: The reference bit pattern is one long-word wide (32 bit).
        /// <see cref="ME_IO_IRQ_START_PATTERN_FILTERING"/>: (for "Bit Pattern Match" only): Enables filtering of the result.
        /// <see cref="ME_IO_IRQ_START_EXTENDED_STATUS"/>: (for "Bit Pattern Change" only): Set extended IRQ status format as default.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_CHANNEL"/>: no channel available on sub-device.
        /// <see cref="ME_ERRNO_INVALID_IRQ_SOURCE"/>: wrong mode or not supported by sub-device.
        /// <see cref="ME_ERRNO_INVALID_IRQ_EDGE"/>: set edge not supported.
        /// <see cref="ME_ERRNO_INVALID_IRQ_ARG"/>: wrong configuration argument / argument not supported.
        /// <see cref="ME_ERRNO_LOCKED"/>: sub-device is protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// <see cref="ME_ERRNO_START_THREAD"/>: creating callback thread failed. (Linux only)
        /// </returns>
        /// <remarks>
        /// On demand an user-defined callback function can be installed by the function <see cref="meIOIrqSetCallback"/>() which is then called on each interrupt.
        /// </remarks>
        /// <seealso cref="meIOIrqSetCallback"/>
        /// <seealso cref="meIOIrqWait"/>
        /// <seealso cref="meIOIrqStop"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOIrqStart(int device, int subdevice, int channel, int irqSource, int irqEdge, int irqArg, int flags);

        #endregion


        #region meIOIrqStop

        /// <summary>
        /// Stops a previously started interrupt handler
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="channel">
        /// Index of the interrupt channel within the selected sub-device.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_IRQ_STOP_NO_FLAGS"/>: No flags in use. Default settings will be used.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_CHANNEL"/>: no channel available on sub-device.
        /// <see cref="ME_ERRNO_LOCKED"/>: sub-device is protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// </returns>
        /// <seealso cref="meIOIrqStart"/>
        /// <seealso cref="meIOIrqSetCallback"/>
        /// <seealso cref="meIOIrqWait"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOIrqStop(int device, int subdevice, int channel, int flags);

        #endregion


        #region meIOIrqWait

        /// <summary>
        /// Waits as long as an interrupt occurs.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="channel">
        /// Index of the interrupt channel within the selected sub-device.
        /// </param>
        /// <param name="irqCount">
        /// Returns the number of interrupts from the specified channel since starting.
        /// </param>
        /// <param name="value">
        /// Returns the interrupt status. "Simple format": One status bit per IRQ line. "Extended format": Two status bits per IRQ line.
        /// </param>
        /// <param name="timeout">
        /// Time-out value in milliseconds. If 0 is passed the function will wait indefinitely.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_IRQ_WAIT_NO_FLAGS"/>: No flags in use. Default settings will be used.
        /// <see cref="ME_IO_IRQ_WAIT_NORMAL_STATUS"/>: Use "simple format" for interrupt status, see <paramref name="value"/>.
        /// <see cref="ME_IO_IRQ_WAIT_EXTENDED_STATUS"/>: Use "extended format" for interrupt status, see <paramref name="value"/>.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_CHANNEL"/>: no channel available on sub-device.
        /// <see cref="ME_ERRNO_LOCKED"/>: sub-device is protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// <see cref="ME_ERRNO_CANCELLED"/>: Sub-device was reset.
        /// <see cref="ME_ERRNO_SIGNAL"/>: Driver was unloaded.
        /// <see cref="ME_ERRNO_PREVIOUS_CONFIG"/>: Sub-device wrongly configured.
        /// </returns>
        /// <remarks>
        /// This function is completely independent from <seealso cref="meIOIrqStart"/>. 
        /// In multi-threading applications there is no need for synchronization. 
        /// meIOIrqWait() can be called before or after <seealso cref="meIOIrqStart"/> and waits for the first interrupt event. 
        /// If an interrupt occurs before meIOIrqWait() is called the function returns immediately and reports it.
        /// You must decide whether you want to install a callback function by <seealso cref="meIOIrqSetCallback"/>. 
        /// before calling the function <seealso cref="meIOIrqStart"/>.
        /// </remarks>
        /// <seealso cref="meIOIrqStart"/>
        /// <seealso cref="meIOIrqSetCallback"/>
        /// <seealso cref="meIOIrqStop"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOIrqWait(int device, int subdeviceI, int channel, out int irqCount, out int value, int timeout, int flags);

        #endregion


        #region meIOResetDevice

        /// <summary>
        /// The device will be reset.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be reset.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_RESET_DEVICE_NO_FLAGS"/>: no flags available
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_LOCKED"/>: device or some of sub-devices are protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// All currently executed operations of the specific device are cancelled:
        /// - All hardware actions are stopped
        /// - Hardware is set to default (idle) state
        /// - Internal states are cleared
        /// - Buffers are flushed (emptied)
        /// - Interrupt counters are set to zero
        /// </remarks>
        /// <seealso cref="meIOResetSubdevice"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOResetDevice(int device, int flags);

        #endregion


        #region meIOResetSubdevice

        /// <summary>
        /// The sub-device will be reset.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be reset.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_RESET_SUBDEVICE_NO_FLAGS"/>: no flags available 
        /// verfgbar.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_LOCKED"/>: device or some of sub-devices are protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// All currently executed operations of the specific sub-device are cancelled:
        /// - All hardware actions are stopped
        /// - Hardware is set to default (idle) state
        /// - Internal states are cleared
        /// - Buffers are flushed (emptied)
        /// - Interrupt counters are set to zero
        /// </remarks>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOResetSubdevice(int device, int subdevice, int flags);

        #endregion


        #region meIOSingleConfig

        /// <summary>
        /// Prepares a sub-device for a "single operation".
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="channel">
        /// Index of the channel within the selected sub-device.
        /// </param>
        /// <param name="singleConfig">
        /// Configuration of measurement ranges, digital ports and counters.
        /// </param>
        /// <param name="reference">
        /// Depending on the subdevice type it can be a ground reference, 
        /// the clock source of the counters or <see cref="ME_REF_NONE"/> 
        /// for standard digital I/O and frequency I/O. 
        /// </param>
        /// <param name="trigChan">
        /// Trigger channel, if supported by the sub-device.
        /// </param>
        /// <param name="trigType">
        /// Trigger type for starting of input/output
        /// </param>
        /// <param name="trigEdge">
        /// Choose the appropriate trigger edge
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_SINGLE_CONFIG_NO_FLAGS"/>: No flags used. Default settings.
        /// <see cref="ME_IO_SINGLE_CONFIG_DIO_BIT"/>: Digital input/output operation by bit.
        /// <see cref="ME_IO_SINGLE_CONFIG_DIO_BYTE"/>: Digital input/output operation by byte (8 bit).
        /// <see cref="ME_IO_SINGLE_CONFIG_DIO_WORD"/>: Digital input/output operation by word (16 bit).
        /// <see cref="ME_IO_SINGLE_CONFIG_DIO_DWORD"/>: Digital input/output operation by long-word (32 bit).
        /// for the channel passed in parameter <paramref name="channel"/> and all channels beyond.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_CHANNEL"/>: no channel available on sub-device.
        /// <see cref="ME_ERRNO_LOCKED"/>: device or some of sub-devices are protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// <see cref="ME_ERRNO_INVALID_REF"/>: parameter <paramref name="reference"/> is not correct.
        /// <see cref="ME_ERRNO_INVALID_TRIG_CHAN"/>: parameter <paramref name="trigChan"/> is not correct.
        /// <see cref="ME_ERRNO_INVALID_TRIG_TYPE"/>: parameter <paramref name="trigType"/> is not correct.
        /// <see cref="ME_ERRNO_INVALID_TRIG_EDGE"/>: parameter <paramref name="trigEdge"/> is not correct.
        /// <see cref="ME_ERRNO_INVALID_SINGLE_CONFIG"/>: parameter <paramref name="singleConfig"/> is not correct.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// Basically the operation starts after calling the function <seealso cref="meIOSingle"/> 
        /// corresponding to the trigger conditions described here.
        /// </remarks>
        /// <seealso cref="meIOSingle"/>
        /// <seealso cref="meIOStreamConfig"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOSingleConfig(int device, int subdevice, int channel, int singleConfig, int refrence, int trigChan, int trigType, int trigEdge, int flags);


        #endregion


        #region meIOSingle

        /// <summary>
        /// Read/write operations can be processed.
        /// </summary>
        /// <param name="singleList">
        /// Reference to a list of type <see cref="meIOSingle_t"/>.
        /// </param>
        /// <param name="count">
        /// Number of entries in <paramref name="singleList"/>.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_SINGLE_NO_FLAGS"/> No extended options. Default settings. 
        /// Execution is stopped on first error.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_CHANNEL"/>: no channel available on sub-device.
        /// <see cref="ME_ERRNO_LOCKED"/>: device or some of sub-devices are protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// <see cref="ME_ERRNO_TIMEOUT"/>: timeout condition occurred.
        /// <see cref="ME_ERRNO_PREVIOUS_CONFIG"/>: sub-device was not configured for required operation.
        /// <see cref="ME_ERRNO_SUBDEVICE_BUSY"/>: sub-device is performing other operation.
        /// </returns>
        /// <remarks>
        /// If for one or more list entries an external trigger source has been selected, 
        /// the function waits until the appropriate trigger signal occurs. 
        /// I. e. if operation 1 with external trigger is executed in blocking mode, 
        /// operation 2 waits until the external trigger pulse of operation 1 occurs.
        /// </remarks>
        /// <seealso cref="meIOSingleConfig"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOSingle(ref meIOSingle_t singleList, int count, int flags);

        #endregion


        #region meIOSingleTicksToTime

        /// <summary>
        /// Converts the number of ticks into the desired time, e. g. period [s] for e. g. frequency measurement.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="timer">
        /// Ticks will be calculated in dependency of the sub-device type and the 
        /// timer specified here.
        /// <see cref="ME_TIMER_FIO_TOTAL"/>: The period is converted into seconds.
        /// <see cref="ME_TIMER_FIO_FIRST_PHASE"/>: The duration of the first phase of the period is converted into seconds.
        /// </param>
        /// <param name="ticksLow">
        /// The number of ticks (lower significant part, bits 31...0) from parameter 
        /// value of the function <see cref="meIOSingle"/> are passed here.
        /// </param>
        /// <param name="ticksHigh">
        /// The number of ticks (higher significant part, bits 63...32) from parameter 
        /// value of the function <see cref="meIOSingle"/> are passed here. This 
        /// parameter is resevered for future enhancements.
        /// </param>
        /// <param name="time">
        /// Pointer to a double value, which returns the calculated time in seconds, e. g. period [s].
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_SINGLE_TICKS_TO_TIME_NO_FLAGS"/>: (default)
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_TIMER"/>: not supported timer ID.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of passed flags are not supported
        /// </returns>
        /// <remarks>
        /// The conversion and the allowed value range depend on each sub-device and 
        /// their properties. If hardware limits are exceeded, always the limit values are returned. 
        /// If you need the dimensions frequency and duty-cycle you can calculate 
        /// them easily by the return values from <paramref name="time"/>. It applies: 
        /// Frequency [Hz] = 1/period [s] 
        /// Duty-cycle [%] = ("Duration of the first phase of the period" [s] / period [s]) * 100
        /// </remarks>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOSingleTicksToTime(int device, int subdevice, int timer, int ticksLow, int ticksHigh, ref double time, int flags);
        #endregion


        #region meIOSingleTimeToTicks

        /// <summary>
        /// Converts a given time period [s] into the number of "ticks" to be passed to the 
        /// timer in the function meIOSingleConfig.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="timer">
        /// Ticks will be calculated in dependency of the sub-device type and the 
        /// timer specified here. The values <paramref name="ticksLow"/> and <paramref name="ticksHigh"/> 
        /// can be passed to the parameter value of the function <see cref="meIOSingle"/>.
        /// <see cref="ME_TIMER_FIO_TOTAL"/>: The period is converted into ticks. 
        /// <see cref="ME_TIMER_FIO_FIRST_PHASE"/>: The duration of the first phase of the period is converted into ticks.
        /// </param>
        /// <param name="time">
        /// (in) Pointer to a double value, passing the asked time in seconds, e. g. the period [s] 
        /// to be converted into ticks. If you pass invalid values a corresponding error code will be returned.
        /// (out) If the asked time cannot be realized exactly by the hardware the 
        /// value next lower to it will be returned here. The corresponding ticks are 
        /// returned in the parameters <paramref name="ticksLow"/> and <paramref name="ticksHigh"/>.
        /// </param>
        /// <param name="ticksLow">
        /// Pointer to an integer value, which contains the lower significant 32 bits 
        /// (31...0) of the calculated ticks. To be passed to the parameter 
        /// value of the function <see cref="meIOSingle"/>. 
        /// </param>
        /// <param name="ticksHigh">
        /// Pointer to an integer value, which contains the higher significant 32 bits 
        /// (63...32) of the calculated ticks. This parameter is resevered for future enhancements.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_SINGLE_TIME_TO_TICKS_NO_FLAGS"/>: (default)
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_TIMER"/>: not supported timer ID.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of passed flags are not supported
        /// </returns>
        /// <remarks>
        /// The conversion and the allowed value range depend on each sub-device and 
        /// their properties. If hardware limits are exceeded, an error message is returned.
        /// </remarks>
        /// <seealso cref="meIOSingle"/>
        /// <seealso cref="meIOSingleTicksToTime"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOSingleTimeToTicks(int device, int subdevice, int timer, ref double time, out int ticksLow, out int ticksHigh, int flags);
        #endregion


        #region meIOSetChannelOffset

        /// <summary>
        /// The anlog input ranges can be adjusted.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="channel">
        /// Index of the channel whose offset should be adjusted.
        /// </param>
        /// <param name="range">
        /// Index of the measurement range to be used for the measurement.
        /// </param>
        /// <param name="offset">
        /// (in) Reference to a double value, passing the asked offset
        /// (out) If the asked offset value cannot be realized exactly 
        /// by the hardware the currently adjusted value will be returned.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_SET_CHANNEL_OFFSET_NO_FLAGS"/>: No flags used. Default settings.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_CHANNEL"/>: no channel available on sub-device.
        /// <see cref="ME_ERRNO_LOCKED"/>: device or some of sub-devices are protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// </returns>
        /// <seealso cref="meQueryNumberRanges"/>
        /// <seealso cref="meQueryRangeByMinMax"/>
        /// <seealso cref="meQueryRangeInfo"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOSetChannelOffset(int device, int subdevice, int channel, int range, ref double offset, int flags);

        #endregion


        #region meIOStreamConfig

        /// <summary>
        /// Configures the hardware for a timer-controlled streaming operation.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="configList">
        /// Reference to a list of type <see cref="meIOStreamConfig_t"/>.
        /// </param>
        /// <param name="count">
        /// Number of entries in <paramref name="configList"/>.
        /// </param>
        /// <param name="trigger">
        /// Pointer to a structure of type <see cref="meIOStreamTrigger_t"/>. 
        /// </param>
        /// <param name="fifoIrqThreshold">
        /// Number of values to be read or written (reloaded) in one package.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_STREAM_CONFIG_NO_FLAGS"/>: Flag for continuous mode (default setting).
        /// MEphisto Scope: analog acquisition
        /// <see cref="ME_IO_STREAM_CONFIG_BIT_PATTERN"/>: Bit pattern output and FIFO redirection. 
        /// MEphisto Scope: Digital acquisition.
        /// <see cref="ME_IO_STREAM_CONFIG_WRAPAROUND"/>: Flag for wraparound mode (periodical output).
        /// Disables software buffer. Used only in combination with ME_IO_STREAM_CONFIG_WRAPAROUND flag.
        /// <see cref="ME_IO_STREAM_CONFIG_SAMPLE_AND_HOLD"/>: Enables "Sample Hold" feature.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_CHANNEL"/>: no channel available on sub-device.
        /// <see cref="ME_ERRNO_LOCKED"/>: device or some of sub-devices are protected.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// <see cref="ME_ERRNO_INVALID_REF"/>: parameter <see cref="meIOStreamConfig_t"/>.iRef is not correct.
        /// <see cref="ME_ERRNO_INVALID_ACQ_START_TRIG_CHAN"/>: parameter <paramref name="meIOStreamConfig_t"/>.iAcqStartTrigChan is not correct.
        /// <see cref="ME_ERRNO_INVALID_ACQ_START_TRIG_EDGE"/>: parameter <paramref name="meIOStreamConfig_t"/>.iAcqStartTrigEdge is not correct.
        /// <see cref="ME_ERRNO_INVALID_STREAM_CONFIG"/>: parameter <paramref name="meIOStreamConfig_t"/>.iStreamConfig is not correct.
        /// <see cref="ME_ERRNO_TIMEOUT"/>: timeout condition occurred.
        /// <see cref="ME_ERRNO_PREVIOUS_CONFIG"/>: sub-device was not configured for required operation.
        /// <see cref="ME_ERRNO_SUBDEVICE_BUSY"/>: sub-device is performing other operation.
        /// <see cref="ME_ERRNO_INVALID_FIFO_IRQ_THRESHOLD"/>: parameter <paramref name="iFifoIrqThreshold"/> is not valid (too big).
        /// <see cref="ME_ERRNO_INVALID_CONFIG_LIST_COUNT"/>: Wrong iCount of configuration list.
        /// <see cref="ME_ERRNO_INVALID_ACQ_START_TRIG_TYPE"/>: parameter <paramref name="meIOStreamConfig_t"/>.iAcqStartTrigType is not correct.
        /// <see cref="ME_ERRNO_INVALID_ACQ_START_ARG"/>: interval <paramref name="meIOStreamConfig_t"/>.iAcqStartTicks is not correct.
        /// <see cref="ME_ERRNO_INVALID_SCAN_START_ARG"/>: interval <paramref name="meIOStreamConfig_t"/>.iScanStartTicks is not correct.
        /// <see cref="ME_ERRNO_INVALID_CONV_START_ARG"/>: interval <paramref name="meIOStreamConfig_t"/>.iConvStartTicks is not correct.
        /// <see cref="ME_ERRNO_INVALID_ACQ_STOP_TRIG_TYPE"/>: parameter <paramref name="meIOStreamConfig_t"/>.iConvStartTrigType is not correct.
        /// <see cref="ME_ERRNO_INVALID_SCAN_STOP_TRIG_TYPE"/>: parameter <paramref name="meIOStreamConfig_t"/>.iConvStartTrigType is not correct.
        /// <see cref="ME_ERRNO_INVALID_ACQ_STOP_ARG"/>: parameter <paramref name="meIOStreamConfig_t"/>.iAcqStopCount is not correct.
        /// <see cref="ME_ERRNO_INVALID_SCAN_STOP_ARG"/>: parameter <paramref name="meIOStreamConfig_t"/>.iScanStopCount is not correct
        /// </returns>
        /// <remarks>
        /// For periodic bit pattern output and for periodic DEMUX operation 
        /// the constant ME_IO_STREAM_CONFIG_WRAPAROUND must be ORed 
        /// with the constant ME_IO_STREAM_CONFIG_ BIT_PATTERN.
        /// For analog and digital (bit pattern output) streaming operations 
        /// a channel-list must be created, which contains an entry for 
        /// each channel (channel index, measurement range...) of type meIOStreamConfig_t.
        /// Additionally a trigger structure (meIOStreamTrigger_t) is required, 
        /// which defines numerous settings like start/stop conditions, timer settings, 
        /// trigger sources and trigger edges valid for the whole operation.
        /// The operation is always started by the function meIOStreamStart() 
        /// either at once (software start) or in accordance with the start 
        /// conditions defined in the function meIOStreamConfig(). 
        /// Stop the operation either according to the stop conditions 
        /// defined in the trigger structure or by calling the function meIOStreamStop().
        /// </remarks>
        /// <seealso cref="meIOStreamFrequencyToTicks"/>
        /// <seealso cref="meIOStreamFrequencyToTicks"/>
        /// <seealso cref="meIOStreamStart"/>
        /// <seealso cref="meIOStreamStop"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOStreamConfig(int device, int subdevice, ref meIOStreamConfig_t configList, int count, ref meIOStreamTrigger_t trigger, int fifoIrqThreshold, int flags);

        #endregion


        #region meIOStreamRead

        /// <summary>
        /// Reads values from the data buffer during a timer-controlled acquisition.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="readMode">
        /// <see cref="ME_READ_MODE_BLOCKING"/>: (BLOCKING mode) The function waits until the number of 
        /// measurement values specified in parameter <paramref name="count"/> has been acquired.
        /// Important: Call can block for ever!
        /// <see cref="ME_READ_MODE_NONBLOCKING"/>: (NONBLOCKING mode) The function returns immediately 
        /// either with the number of measurement values available when calling 
        /// the function or with the number specified in <paramref name="count"/>. 
        /// Each time the lower value is valid.
        /// </param>
        /// <param name="values">
        /// Reference to a data buffer to be filled up.
        /// </param>
        /// <param name="count">
        /// (in) Size of the data buffer to be read in number of measurement values.
        /// (out) The pointer returns the number of values actually read from the data buffer.
        /// If in BLOCKING mode the acquisition has been cancelled, the return value can also be less than the value passed.
        /// BLOCKING mode:
        /// Number of values to be read  as a rule a multiple of the channel-list length, 
        /// but this is not necessary (see also meUtilityExtractValues() on page 169).
        /// NONBLOCKING mode:
        /// Number of values to be read  if you want to read a multiple of channel-list 
        /// length use the constant ME_IO_STREAM_ READ_FRAMES in the parameter <paramref name="flags"/>.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_STREAM_READ_NO_FLAGS"/>: Default settings.
        /// <see cref="ME_IO_STREAM_READ_FRAMES"/>: Reading a multiple of channel-list length in NONBLOCKING mode.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// <see cref="ME_ERRNO_INVALID_VALUE_COUNT"/>: <paramref name="count"/> is lower than zero.
        /// <see cref="ME_ERRNO_INVALID_READ_MODE"/>: not supported read mode detected.
        /// <see cref="ME_ERRNO_SUBDEVICE_NOT_RUNNING"/>: buffer is empty and sub-device is not running a streaming operation.
        /// <see cref="ME_ERRNO_HARDWARE_BUFFER_OVERFLOW"/>: error during acquisition. Reading data from FIFO was too slow.
        /// <see cref="ME_ERRNO_SOFTWARE_BUFFER_OVERFLOW"/>: no place in buffer for new data. Reading data from buffer was too slow.
        /// </returns>
        /// <remarks>
        /// The user has to allocate a data buffer, to which the measurement values are written to. 
        /// Using the execution mode BLOCKING the function meIOStreamRead() returns when the last 
        /// value has been read. In NONBLOCKING mode the function returns at once 
        /// with the available measurement values. For reading the data a callback function can also be used.
        /// </remarks>
        /// <seealso cref="meIOStreamStatus"/>
        /// <seealso cref="meIOStreamSetCallbacks"/>
        /// <seealso cref="meUtilityDigitalToPhysical"/>
        /// <seealso cref="meUtilityExtractValues"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOStreamRead(int device, int subdevice, int readMode, out int values, ref int count, int flags);

        #endregion


        #region meIOStreamWrite

        /// <summary>
        /// Writes data to a buffer for timer-controlled analog resp. digital output.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="writeMode">
        /// <see cref="ME_WRITE_MODE_BLOCKING"/>: The function waits until the number of values specified 
        /// in parameter <paramref name="count"/> can be written to the internal buffer.
        /// Important: Call can block for ever!
        /// <see cref="ME_WRITE_MODE_NONBLOCKING"/>: With this option the function writes as many values 
        /// to the internal buffer as there is space at the moment the function was called 
        /// (max. number specified in <paramref name="count"/>).
        /// <see cref="ME_WRITE_MODE_PRELOAD"/>: Pre-loading the data buffer for the first time. 
        /// Data will be written directly to the hardware buffer. 
        /// If there are more data as there is space in the FIFO the rest is stored in an internal buffer.
        /// Note: This is non-blocking writing.
        /// </param>
        /// <param name="values">
        /// Pointer to a data buffer (data stream) with the values resp. bit patterns to be output. 
        /// Use the function <seealso cref="meUtilityPhysicalToDigital"/> for easy conversion of physical 
        /// values (e. g. voltage) into digital values.
        /// </param>
        /// <param name="count">
        /// (in): Number of values to be loaded into the data buffer.
        /// (out): The pointer returns the number of values which could be written into the data buffer actually. 
        /// The number will never be greater, but can also be less, 
        /// if there is less memory for the number of values passed.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_STREAM_WRITE_NO_FLAGS"/>: Default settings  no flags available
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// <see cref="ME_ERRNO_INVALID_VALUE_COUNT"/>: <paramref name="count"/> is lower than zero.
        /// Note: When <paramref name="count"/> is set to zero ME_ERRNO_SUCCESS is returned.
        /// <see cref="ME_ERRNO_INVALID_WRITE_MODE"/>: not supported write mode detected.
        /// <see cref="ME_ERRNO_PREVIOUS_CONFIG"/>: device is configured to work in single mode.
        /// <see cref="ME_ERRNO_HARDWARE_BUFFER_UNDERFLOW"/>: error during streaming. Writing data to FIFO was too slow.
        /// Hardware wraparound: More data than space in FIFO.
        /// <see cref="ME_ERRNO_SOFTWARE_BUFFER_UNDERFLOW"/>: (wraparound only) no place in buffer for data.
        /// <see cref="ME_ERRNO_SUBDEVICE_NOT_RUNNING"/>: The internal hardware state machine is stopped but 
        /// logical status show that should be working. No data in FIFO but software buffer is not empty
        /// </returns>
        /// <remarks>
        /// For the values to be output allocate a data buffer of defined size for each sub-device to be used.
        /// </remarks>
        /// <seealso cref="meIOStreamRead"/>
        /// <seealso cref="meUtilityPhysicalToDigital"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOStreamWrite(int device, int subdevice, int writeMode, ref int values, ref int count, int flags);

        #endregion


        #region meIOStreamSetCallbacks

        /// <summary>
        /// Installs different callback functions that wait for an event on a streaming sub-device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="startCB">
        /// Delegate to an user-defined function. This function is called when streaming operation starts. 
        /// If the function exits with a return value different than <see cref="ME_ERRNO_SUCCESS"/> 
        /// streaming is instantly stopped, <seealso cref="meIOStreamStop"/> is executed.
        /// </param>
        /// <param name="startCBContext">
        /// User-defined pointer passed to start callback function. 
        /// This parameter is optional. If you don't want to use this functionality pass NULL.
        ///  </param>
        /// <param name="newValuesCB">
        /// Pointer to an user-defined function. This function is called when the buffer status is changing. 
        /// If the function exits with a return value different than <see cref="ME_ERRNO_SUCCESS"/> 
        /// streaming is instantly stopped, <seealso cref="meIOStreamStop"/> is executed.
        /// </param>
        /// <param name="newValuesCBContext">
        /// User-defined pointer passed to new values callback function. This parameter is optional. 
        /// If you don't want to use this functionality pass NULL
        /// </param>
        /// <param name="endCB">
        /// Pointer to an user-defined function. This function is called when streaming operation stops.
        /// </param>
        /// <param name="endCBContext">
        /// User-defined pointer passed to stop callback function. This parameter is optional. 
        /// If you don't want to use this functionality pass NULL.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_STREAM_SET_CALLBACKS_NO_FLAGS"/>: (no flags available)
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// To deinstall/cancel all registered callback instances for the selected sub-device call 
        /// meIOStreamSetCallbacks() with all callback pointers 
        /// (<paramref name="startCB"/>, <paramref name="newValuesCB"/> and <paramref name="endCB"/>) set to NULL.
        /// </remarks>
        /// <seealso cref="meIOStreamStart"/>
        /// <seealso cref="meIOStreamStop"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOStreamSetCallbacks(int device, int subdevice, meIOStreamCB_t startCB, IntPtr startCBContext, meIOStreamCB_t newValuesCB, IntPtr newValuesCBContext, meIOStreamCB_t endCB, IntPtr endCBContext, int flags);

        #endregion


        #region meIOStreamStart

        /// <summary>
        /// Starts streaming operations.
        /// </summary>
        /// <param name="startList">
        /// Pointer to a list of type meIOStreamStart_t, by which one or more streaming operations can be started. 
        /// The start is done immediately after calling the function corresponding to the start conditions. 
        /// If for one or more list entries the <see cref="meIOStreamStart_t"/>.iStartMode
        /// </param>
        /// <param name="count">
        /// Number of meIOStreamStart_t entries in <paramref name="startList"/>.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_STREAM_START_NO_FLAGS"/>: Default settings. Execute list up to the first error. 
        /// Returned value corresponds with the first non-zero <see cref="meIOStreamStart_t"/>.iErrno field.
        /// <see cref="ME_IO_STREAM_START_NONBLOCKING"/>: (Linux only) Execute whole start list. 
        /// When this flag is set execution is processed although an error occured for some entries. 
        /// The function returns ME_ERRNO_SUCCESS when no global error was detected. 
        /// <see cref="meIOStreamStart_t"/>.iErrno fields have to be checked.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of the passed flags are not supported.
        /// <see cref="ME_ERRNO_INVALID_TIMEOUT"/>: timeout lower than 0.
        /// <see cref="ME_ERRNO_INVALID_START_MODE"/>: not supported start mode detected.
        /// <see cref="ME_ERRNO_PREVIOUS_CONFIG"/>: sub-device is not configured correctly to proceed streaming operation.
        /// <see cref="ME_STATUS_ERROR"/>: previous operation ended with an error. Reset has to be called to clear this state.
        /// <see cref="ME_ERRNO_LACK_OF_RESOURCES"/>: (output only) operation can not be proceed. Buffers are empty. (Linux only)
        /// <see cref="ME_ERRNO_TIMEOUT"/>: Operation didn't start on time.
        /// <see cref="ME_ERRNO_START_THREAD"/>: creating callback thread failed. (Windows only)
        /// </returns>
        /// <remarks>
        /// Returning of the function depends on the <see cref="meIOStreamStart_t"/>.iStartMode
        /// (BLOCKING or NONBLOCKING) and trigger conditions defined in the function <see cref="meIOStreamConfig"/>.
        /// </remarks>
        /// <seealso cref="meIOStreamStart_t"/>
        /// <seealso cref="meIOStreamConfig"/>
        /// <seealso cref="meIOStreamStop"/>
        /// <seealso cref="meIOResetDevice"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOStreamStart(ref meIOStreamStart_t startList, int count, int flags);

        #endregion


        #region meIOStreamStatus

        /// <summary>
        /// Checks the status of streaming operation.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="wait">
        /// Behaviour of return of this function:
        /// <see cref="ME_WAIT_NONE"/>: Check current status. 
        /// Function returns the current state of operation immediately in parameter <paramref name="status"/>.
        /// <see cref="ME_WAIT_IDLE"/>: In case of an output operation the function waits until all values have been output. 
        /// The function returns with <see cref="ME_STATUS_IDLE"/> in parameter <paramref name="status"/>.
        /// Important: Call can block for ever!
        /// <see cref="ME_WAIT_BUSY"/>: (Linux only)
        /// The function blocks while the status is <see cref="ME_STATUS_BUSY"/>
        /// Important: Call can block for ever!
        /// </param>
        /// <param name="status">
        /// Pointer which returns the current state of operation of the specified sub-device.
        /// <see cref="ME_STATUS_IDLE"/>: Streaming operation has finished.
        /// <see cref="ME_STATUS_BUSY"/>: Streaming operation is still running.
        /// <see cref="ME_STATUS_ERROR"/>: Error occured, e. g. data stream was interrupted.
        /// </param>
        /// <param name="count">
        /// Input sub-device: Number of values which can be read
        /// Output sub-device: Free memory in the output buffer (in number of values)
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_STREAM_STATUS_NO_FLAGS"/>: Default settings  no flags available
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: unsupported flag detected.
        /// <see cref="ME_ERRNO_INVALID_WAIT"/>: not supported wait mode detected.
        /// </returns>
        /// <remarks>
        /// Depending on input or output operation this function is used to check whether 
        /// all measurement values have been acquired or whether an output operation is still running.
        /// With the parameter <paramref name="wait"/> you can control, whether the function 
        /// should return the current state at once, or whether you want to wait 
        /// until the input resp. output operation has been ended.
        /// </remarks>
        /// <seealso cref="meIOStreamRead"/>
        /// <seealso cref="meIOStreamWrite"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOStreamStatus(int device, int subdevice, int wait, out int status, out int count, int flags);

        #endregion


        #region meIOStreamStop

        /// <summary>
        /// An "infinite" operation can be cancelled at once or stopped in a defined way.
        /// </summary>
        /// <param name="stopList">
        /// Pointer to a list of type <see cref="meIOStreamStop_t"/> to end one or several input/output operations. 
        /// Stopping is done in accordance to parameter <see cref="meIOStreamStop_t"/>.iStopMode.
        /// </param>
        /// <param name="count">
        /// Number of <see cref="meIOStreamStop_t"/> entries in <paramref name="stopList"/>.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_STREAM_STOP_NO_FLAGS"/>: Default settings. 
        /// Execute list up to the first error. 
        /// Returned value corresponds with the first non-zero <see cref="meIOStreamStop_t"/>.iErrno field.
        /// <see cref="ME_IO_STREAM_STOP_NONBLOCKING"/>: (Linux only)
        /// Execute whole stop list one by one. When this flag is set execution is processed 
        /// although an error occured for some entries. The function returns ME_SUCCESS 
        /// when no global error was detected. <see cref="meIOStreamStop_t"/>.iErrno fields have to be checked.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: unsupported flag detected.
        /// <see cref="ME_ERRNO_INVALID_STOP_MODE"/>: not supported stop mode detected.
        /// <see cref="ME_ERRNO_PREVIOUS_CONFIG"/>: sub-device is not configured correctly to proceed streaming operation.
        /// <see cref="ME_STATUS_ERROR"/>: previous operation ended with an error. Reset has to be called to clear this state.
        /// </returns>
        /// <remarks>
        /// With meIOStreamStop you have the possibility to stop an output operation by the last entry 
        /// in the FIFO which is a known value.
        /// If in the parameters iAcqStopCount resp. iScanStopCount of the function meIOStreamConfig() 
        /// stop conditions have been defined, calling the function meIOStreamStop() is not necessary.
        /// Configuration of the sub-device remains preserved (channel-list, timer...) 
        /// so that a restart with the function <see cref="meIOStreamStart"/> is possible without new configuration.
        /// In opposite to this using the function <see cref="meIOResetDevice"/> deletes the whole configuration of the device.
        /// </remarks>
        /// <seealso cref="meIOStreamStop_t"/>
        /// <seealso cref="meIOResetDevice"/>
        /// <seealso cref="meIOStreamStart"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOStreamStop(ref meIOStreamStop_t stopList, int count, int flags);

        #endregion


        #region meIOStreamFrequencyToTicks

        /// <summary>
        /// Converts a given frequency [Hz] into the number of "ticks"
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="timer">
        /// Ticks will be calculated in dependency of the sub-device to which the specified timer belongs to. 
        /// The values <paramref name="ticksLow"/> and <paramref name="ticksHigh"/> are passed 
        /// in the trigger structure of the function <see cref="meIOStreamConfig"/> 
        /// <see cref="ME_TIMER_ACQ_START"/>: <see cref="meIOStreamTrigger_t"/>.iAcqStartTicks 
        /// should be calculated for passing to the parameter of the same name. 
        /// <see cref="ME_TIMER_SCAN_START"/>: <see cref="meIOStreamTrigger_t"/>.iScanStartTicks 
        /// should be calculated for passing to the parameter of the same name.  
        /// <see cref="ME_TIMER_CONV_START"/>: <see cref="meIOStreamTrigger_t"/>.iConvStartTicks 
        /// should be calculated for passing to the parameter of the same name. 
        /// </param>
        /// <param name="frequency">
        /// Pointer to a double value, passing the asked frequency [Hz] to be converted into ticks.
        /// If you pass 0 the maximum frequency will be returned.
        /// If the asked frequency cannot be realized exactly by the hardware the frequency next higher 
        /// to it will be returned as an approximation. 
        /// The corresponding ticks are returned in the parameters <paramref name="ticksLow"/> and <paramref name="ticksHigh"/>.
        /// </param>
        /// <param name="ticksLow">
        /// Pointer to an integer value, which contains the lower significant 32 bits (31...0) of the calculated ticks. 
        /// Will be passed to the appropriate parameter startTicksLow of the function <see cref="meIOStreamConfig"/>.
        /// </param>
        /// <param name="ticksHigh">
        /// Pointer to an integer value, which contains the higher significant 32 bits (63...32) of the calculated ticks. 
        /// Will be passed to the appropriate parameter startTicksHigh of the function <see cref="meIOStreamConfig"/>.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_FREQUENCY_TO_TICKS_NO_FLAGS"/>: (default) MEphisto Scope: data-logging mode
        /// <see cref="ME_IO_FREQUENCY_TO_TICKS_MEPHISTO_SCOPE_OSCILLOSCOPE"/>: MEphisto Scope: oscilloscope mode
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_TIMER"/>: not supported timer ID.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// The conversion and the allowed value range depend on each sub-device and their timers. 
        /// If hardware limits are exceeded, always the limit values are returned.
        /// Tip: Passing "0" in parameter <paramref name="frequency"/> of this function returns the maximum frequency allowed. 
        /// In dependency of parameter <paramref name="timer"/> of this function the return values of 
        /// <paramref name="ticksLow"/> and <paramref name="ticksHigh"/> can be passed to the corresponding parameters 
        /// <see cref="meIOStreamTrigger_t"/>.iAcqStartTicks, <see cref="meIOStreamTrigger_t"/>.iConvStartTicks 
        /// and <see cref="meIOStreamTrigger_t"/>.iScanStartTicks in the trigger structure of function <see cref="meIOStreamConfig"/>.
        /// </remarks>
        /// <seealso cref="meIOStreamTimeToTicks"/>
        /// <seealso cref="meIOStreamConfig"/>
        /// <seealso cref="meIOStreamTrigger_t"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOStreamFrequencyToTicks(int device, int subdevice, int timer, ref double frequency, out int ticksLow, out int ticksHigh, int flags);

        #endregion


        #region meIOStreamTimeToTicks

        /// <summary>
        /// Converts a given time period [s] into the number of "ticks".
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="timer">
        /// Ticks will be calculated in dependency of the sub-device to which the specified timer belongs to. 
        /// The values <paramref name="ticksLow"/> and <paramref name="ticksHigh"/> are passed 
        /// in the trigger structure of the function <see cref="meIOStreamConfig"/> 
        /// <see cref="ME_TIMER_ACQ_START"/>: <see cref="meIOStreamTrigger_t"/>.iAcqStartTicks 
        /// should be calculated for passing to the parameter of the same name. 
        /// <see cref="ME_TIMER_SCAN_START"/>: <see cref="meIOStreamTrigger_t"/>.iScanStartTicks 
        /// should be calculated for passing to the parameter of the same name.  
        /// <see cref="ME_TIMER_CONV_START"/>: <see cref="meIOStreamTrigger_t"/>.iConvStartTicks 
        /// should be calculated for passing to the parameter of the same name. 
        /// </param>
        /// <param name="time">
        /// (in): Pointer to a double value, passing the asked period [s] to be converted into ticks. 
        /// If you pass "0" the minimum period will be returned.
        /// (out): If the asked period cannot be realized exactly by the hardware the 
        /// period next lower to it will be returned as an approximation. The corresponding ticks 
        /// are returned in the parameters <paramref name="ticksLow"/> and <paramref name="ticksHigh"/>.
        /// </param>
        /// <param name="ticksLow">
        /// Pointer to an integer value, which contains the lower significant 32 bits (31...0) of the calculated ticks. 
        /// Will be passed to the appropriate parameter startTicksLow of the function <see cref="meIOStreamConfig"/>.
        /// </param>
        /// <param name="ticksHigh">
        /// Pointer to an integer value, which contains the higher significant 32 bits (63...32) of the calculated ticks. 
        /// Will be passed to the appropriate parameter startTicksHigh of the function <see cref="meIOStreamConfig"/>.
        /// </param>
        /// <param name="flags">
        /// <see cref="ME_IO_TIME_TO_TICKS_NO_FLAGS"/>: (default) MEphisto Scope: data-logging mode
        /// <see cref="ME_IO_TIME_TO_TICKS_MEPHISTO_SCOPE_OSCILLOSCOPE"/>: MEphisto Scope: oscilloscope mode
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_TIMER"/>: not supported timer ID.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: some of passed flags are not supported.
        /// </returns>
        /// <remarks>
        /// The conversion and the allowed value range depend on each sub-device and their timers. 
        /// If hardware limits are exceeded, always the limit values are returned. 
        /// Tip: Passing "0" in parameter <paramref name="time"/> of this function returns the 
        /// minimum frequency allowed. In dependency of parameter <paramref name="timer"/> of 
        /// this function the return values of <paramref name="ticksLow"/> and <paramref name="ticksHigh"/> 
        /// can be passed to the corresponding parameters <see cref="meIOStreamTrigger_t"/>.iAcqStartTicks, 
        /// <see cref="meIOStreamTrigger_t"/>.iConvStartTicks and <see cref="meIOStreamTrigger_t"/>.iScanStartTicks 
        /// in the trigger structure of function <see cref="meIOStreamConfig"/>.
        /// </remarks>
        /// <seealso cref="IOStreamFrequencyToTicks"/>
        /// <seealso cref="meIOStreamConfig"/>
        /// <seealso cref="meIOStreamTrigger_t"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIOStreamTimeToTicks(int device, int subdevice, int timer, ref double time, out int ticksLow, out int ticksHigh, int flags);

        #endregion


        #region meQueryDescriptionDevice

        /// <summary>
        /// Determines the device description.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="description">
        /// Buffer to hold the device description.
        /// </param>
        /// <param name="count">
        /// Buffer size in bytes for device description. 
        /// Recommended: <see cref="ME_DEVICE_DESCRIPTION_MAX_COUNT"/>.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_USER_BUFFER_SIZE"/>: count smaller than 1.
        /// </returns>
        /// <seealso cref="meQueryInfoDevice"/>
        /// <seealso cref="meQueryNameDevice"/>
        /// <seealso cref="meQueryNumberDevices"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int meQueryDescriptionDevice(int device, StringBuilder description, int count);

        #endregion


        #region meQueryInfoDevice

        /// <summary>
        /// Determines detailed information of the device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="vendorId">
        /// Pointer returns vendor ID of the device.
        /// 0x1402 for Meilhaus PCI boards, 0x1B04 for USB devices like e. g. ME-1 (ME-Synapse USB).
        /// </param>
        /// <param name="deviceId">
        /// Pointer returns the device ID.
        /// </param>
        /// <param name="serialNo">
        /// Pointer returns the serial number of the device.
        /// </param>
        /// <param name="busType">
        /// Pointer returns the bus type by which the device is connected with the PC (PCI/cPCI, USB).
        /// </param>
        /// <param name="busNo">
        /// PCI only: Pointer returns the PCI bus number, if several PCI buses are available in your system (if one bus it is always "0")
        /// </param>
        /// <param name="slot">
        /// PCI only: Slot number of the board to be accessed.
        /// </param>
        /// <param name="funcNo">
        /// PCI: Function number.
        /// </param>
        /// <param name="plugged">
        /// Pointer indicates whether a device is physically available.
        /// <see cref="ME_PLUGGED_IN"/>: Device physically available.
        /// <see cref="ME_PLUGGED_OUT"/>: Device registered with the ME-iDC (ME-Config-Tool) but not connected with the PC.
        /// <see cref="ME_PLUGGED_INVALID"/>: invalid return value
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// </returns>
        /// <remarks>
        /// This is a PCI orientated function and some parameters have no meaning for 
        /// ME-Synapse USB or ME-Synapse LAN.
        /// </remarks>
        /// <seealso cref="meQueryDescriptionDevice"/>
        /// <seealso cref="meQueryNameDevice"/>
        /// <seealso cref="meQueryNumberDevices"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryInfoDevice(int device,
                                                   out int vendorId,
                                                   out int deviceId,
                                                   out int serialNo,
                                                   out int busType,
                                                   out int busNo,
                                                   out int slot,
                                                   out int funcNo,
                                                   out int plugged);

        #endregion


        #region meQueryNameDevice

        /// <summary>
        /// Determines the device name.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="name">
        /// Buffer for the device name.
        /// </param>
        /// <param name="count">
        /// Buffer size in bytes for device name. Recommended: <see cref="ME_DEVICE_NAME_MAX_COUNT"/>.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_USER_BUFFER_SIZE"/>: count smaller than 1.
        /// </returns>
        /// <seealso cref="meQueryInfoDevice"/>
        /// <seealso cref="meQueryDescriptionDevice"/>
        /// <seealso cref="meQueryNumberDevices"/>
        /// <seealso cref="meQueryNameDeviceDriver"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int meQueryNameDevice(int device, StringBuilder name, int count);

        #endregion


        #region meQueryNameDeviceDriver

        /// <summary>
        /// Determines the name of the device specific driver module.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="name">
        /// Pointer to a buffer for the driver module.
        /// </param>
        /// <param name="count">
        /// Buffer size in bytes for the driver module. 
        /// Recommended: <see cref="ME_DEVICE_DRIVER_NAME_MAX_COUNT"/>.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_USER_BUFFER_SIZE"/>: count smaller than 1.
        /// </returns>
        /// <seealso cref="meQueryVersionDeviceDriver"/>
        /// <seealso cref="meQueryVersionMainDriver"/>
        /// <seealso cref="meQueryVersionLibrary"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int meQueryNameDeviceDriver(int device, StringBuilder name, int count);

        #endregion


        #region meQueryNumberDevices

        /// <summary>
        /// Determines the number of devices recognized by the ME-iDS.
        /// </summary>
        /// <param name="count">
        /// Contains the number of recognized devices after the function returned.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <remarks>
        /// Not all devices have to be physically present.
        /// This can be tested with the function <see cref="meQueryInfoDevice"/>.
        /// </remarks>
        /// <seealso cref="meQueryInfoDevice"/>
        /// <seealso cref="meQueryNumberSubdevices"/>
        /// <seealso cref="meQueryNumberChannels"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryNumberDevices(out int count);

        #endregion


        #region meQueryNumberSubdevices

        /// <summary>
        /// Determines the number of sub-devices on a queried device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="count">
        /// Number of sub-devices on the device.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// </returns>
        /// <seealso cref="meQueryNumberDevices"/>
        /// <seealso cref="meQueryNumberChannels"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryNumberSubdevices(int device, out int count);

        #endregion


        #region meQueryNumberChannels

        /// <summary>
        /// Determines the number of channels of a sub-device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="count">
        /// Returns the number of channels of the specified sub-device.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// </returns>
        /// <seealso cref="meQueryNumberDevices"/>
        /// <seealso cref="meQueryNumberSubdevices"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryNumberChannels(int device, int subdevice, out int count);

        #endregion


        #region meQueryNumberRanges

        /// <summary>
        /// Determines the number of ranges which support the specified measurement range(s).
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="unit">
        /// Restrict the query to the specified measurement range(s).
        /// <see cref="ME_UNIT_ANY"/>: Query for all ranges
        /// <see cref="ME_UNIT_VOLT"/>: Query only for voltage ranges
        /// <see cref="ME_UNIT_AMPERE"/>: Query only for current ranges
        /// </param>
        /// <param name="count">
        /// Returns the number of ranges which support the specified measurement range(s).
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_NOT_SUPPORTED"/>: function is not supported by sub-device.
        /// </returns>
        /// <seealso cref="meQueryRangeInfo"/>
        /// <seealso cref="meQueryRangeByMinMax"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryNumberRanges(int device, int subdevice, int unit, out int count);

        #endregion


        #region meQueryRangeByMinMax

        /// <summary>
        /// Determines the appropriate measurement range by given limits
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="unit">
        /// Restrict the query to the specified measurement range(s).
        /// <see cref="ME_UNIT_ANY"/>: Query for all ranges
        /// <see cref="ME_UNIT_VOLT"/>: Query only for voltage ranges
        /// <see cref="ME_UNIT_AMPERE"/>: Query only for current ranges
        /// </param>
        /// <param name="min">
        /// (in) Passing the lower limit of the searched range.
        /// (out) Returns the lower limit for the range determined.
        /// </param>
        /// <param name="max">
        /// (in) Passing the upper limit of the searched range. 
        /// (out) Returns the upper limit for the range determined.
        /// </param>
        /// <param name="maxData">
        /// Returns the maximum resolution for the range determined (e. g. 65535 (0xFFFF) for 16 bit resolution).
        /// </param>
        /// <param name="rangeIndex">
        /// The index of the best fitting measurement range is returned. 
        /// Always the smallest range is chosen, which includes the range limits searched for.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_UNIT"/>: unit's type not supported on sub-device.
        /// <see cref="ME_ERRNO_INVALID_MIN_MAX"/>: lower limit is higher than upper.
        /// <see cref="ME_ERRNO_NO_RANGE"/>: fitted range not found.
        /// </returns>
        /// <seealso cref="meQueryRangeInfo"/>
        /// <seealso cref="meQueryNumberRanges"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryRangeByMinMax(int device, int subdevice, int unit, ref double min, ref double max, out int maxData, out int rangeIndex);

        #endregion


        #region meQueryRangeInfo

        /// <summary>
        /// Determines details of the specified measurement range.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device that is queried.
        /// </param>
        /// <param name="range">
        /// Index of measurement range that is queried.
        /// </param>
        /// <param name="unit">
        /// Pointer, which returns the physical unit of the specified measurement range.
        /// <see cref="ME_UNIT_INVALID"/>: Invalid return value
        /// <see cref="ME_UNIT_VOLT"/>: Voltage range
        /// <see cref="ME_UNIT_AMPERE"/>: Current range
        /// </param>
        /// <param name="min">
        /// Reference to the lower limit of the requested range. 
        /// It applies to the physical unit specified in parameter <paramref name="unit"/>.
        /// </param>
        /// <param name="max">
        /// Reference to the upper limit of the requested range.
        /// It applies to the physical unit specified in parameter <paramref name="unit"/>.
        /// </param>
        /// <param name="maxData">
        /// Returns the maximum resolution of the measurement range (e. g. for 16 bit resolution the value 65535 (0xFFFF) is returned).
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// </returns>
        /// <seealso cref="meQueryNumberRanges"/>
        /// <seealso cref="meQueryRangeByMinMax"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryRangeInfo(int device, int subdevice, int range, out int unit, out double min, out double max, out int maxData);

        #endregion


        #region meQuerySubdeviceByType

        /// <summary>
        /// Determines the index of the first available sub-device which matches the features searching for.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="startSubdevice">
        /// Index of the sub-device the query starts with.
        /// </param>
        /// <param name="type">
        /// Type of the sub-device searched for:
        /// <see cref="ME_TYPE_AI"/> Analog acquisition
        /// <see cref="ME_TYPE_AO"/>: Analog output
        /// <see cref="ME_TYPE_DIO"/>: Digital input/output (bi-directional)
        /// <see cref="ME_TYPE_DO"/>: Digital output
        /// <see cref="ME_TYPE_DI"/>: Digital input
        /// <see cref="ME_TYPE_FIO"/>: Frequency input/output (bi-directional)
        /// <see cref="ME_TYPE_FO"/>: Frequency output
        /// <see cref="ME_TYPE_FI"/>: Frequency input
        /// <see cref="ME_TYPE_CTR"/>: Counter
        /// <see cref="ME_TYPE_EXT_IRQ"/>: External interrupt
        /// </param>
        /// <param name="subType">
        /// The capabilities of the sub-device searched for can be specified more detailed if necessary:
        /// <see cref="ME_SUBTYPE_ANY"/>: Sub-type doesn't matter.
        /// <see cref="ME_SUBTYPE_SINGLE"/>: Acquisition resp. output of a single value.
        /// <see cref="ME_SUBTYPE_STREAMING"/>: Continuous acquisition resp. output by using special hardware capabilities (e. g. FIFOs).
        /// <see cref="ME_SUBTYPE_CTR_8254"/>: Sub-device with a counter of type 8254
        /// </param>
        /// <param name="subdevice">
        /// Reference to the index of the first matching sub-device.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_NO_MORE_SUBDEVICE_TYPE"/>: Matching sub-device not found.
        /// </returns>
        /// <seealso cref="meQuerySubdeviceType"/>
        /// <seealso cref="meQuerySubdeviceCaps"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQuerySubdeviceByType(int device, int startSubdevice, int type, int subType, out int subdevice);

        #endregion


        #region meQuerySubdeviceType

        /// <summary>
        /// Determines the type and subtype of the specified sub-device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device that is queried.
        /// </param>
        /// <param name="type">
        /// Reference to the sub-device type:
        /// <see cref="ME_TYPE_AO"/>: Analog output
        /// <see cref="ME_TYPE_AI"/> Analog input
        /// <see cref="ME_TYPE_DIO"/>: Digital input/output (bi-directional)
        /// <see cref="ME_TYPE_DO"/>: Digital output
        /// <see cref="ME_TYPE_DI"/>: Digital input
        /// <see cref="ME_TYPE_FIO"/>: Frequency input/output (bi-directional)
        /// <see cref="ME_TYPE_FO"/>: Frequency output
        /// <see cref="ME_TYPE_FI"/>: Frequency input
        /// <see cref="ME_TYPE_CTR"/>: Counter
        /// <see cref="ME_TYPE_EXT_IRQ"/>: External interrupt
        /// </param>
        /// <param name="subType">
        /// Returns the subtype of the sub-device:
        /// <see cref="ME_SUBTYPE_SINGLE"/>: Sub-device is able to acquire resp. output single values.
        /// <see cref="ME_SUBTYPE_STREAMING"/>: The sub-device can acquire values continuously resp. output a data stream.
        /// <see cref="ME_SUBTYPE_CTR_8254"/>: Sub-device with a counter of type 8254
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// </returns>
        /// <seealso cref="meQueryInfoDevice"/>
        /// <seealso cref="meQuerySubdeviceByType"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQuerySubdeviceType(int device, int subdevice, out int type, out int subType);

        #endregion


        #region meQuerySubdeviceCaps

        /// <summary>
        /// Determines the special capabilities of a sub-device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device that is queried.
        /// </param>
        /// <param name="caps">
        /// Reference to a bit-coded integer value which returns the special capabilities of the specified sub-device.
        /// If several capabilities apply, the values are ORed bitwise.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// </returns>
        /// <seealso cref="meQuerySubdeviceCapsArgs"/>
        /// <seealso cref="meQuerySubdeviceType"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQuerySubdeviceCaps(int device, int subdevice, out int caps);

        #endregion


        #region meQuerySubdeviceCapsArgs

        /// <summary>
        /// Determines detailed information of a specific capability of a sub-device.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device that is queried.
        /// </param>
        /// <param name="cap">
        /// The capability to be queried.
        /// </param>
        /// <param name="args">
        /// Reference to an array of type integer, which returns the queried values.
        /// </param>
        /// <param name="count">
        /// Number of values in parameter <paramref name="args"/>.  
        /// As a rule "1", if the returned values fit into an integer.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_CAP"/>: Passed code is invalid or not supported on sub-device.
        /// <see cref="ME_ERRNO_INVALID_CAP_ARG_COUNT"/>: Parameter <paramref name="count"/> doesn't fit for queried capability code.
        /// </returns>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQuerySubdeviceCapsArgs(int device, int subdevice, int cap, ref int args, int count);

        #endregion


        #region meQueryVersionLibrary

        /// <summary>
        /// Determines the version number of the library.
        /// </summary>
        /// <param name="version">
        /// Reference to the version number of the library.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <remarks>
        /// Can be called before calling <see cref="meOpen"/>.
        /// </remarks>
        /// <seealso cref="meQueryVersionDeviceDriver"/>
        /// <seealso cref="meQueryVersionMainDriver"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryVersionLibrary(out int version);

        #endregion


        #region meQueryVersionMainDriver

        /// <summary>
        /// Determines the version number of the main driver.
        /// </summary>
        /// <param name="version">
        /// Reference to the version number of the main driver.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <remarks>
        /// The two higher significant bytes (main version, sub version) must be the same one as the 
        /// version number of the device specific driver module (see <see cref="meQueryVersionDeviceDriver"/>). 
        /// The lower significant bytes (build number) can differ.
        /// </remarks>
        /// <seealso cref="meQueryVersionDeviceDriver"/>
        /// <seealso cref="meQueryVersionLibrary"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryVersionMainDriver(out int version);

        #endregion


        #region meQueryVersionDeviceDriver

        /// <summary>
        /// Determines the version number of the device specific driver module.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="version">
        /// Reference to the version number of the main driver.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// </returns>
        /// <remarks>
        /// The version number must be the same one as the version number of the main driver 
        /// (see <see cref="meQueryVersionMainDriver"/>).
        /// </remarks>
        /// <seealso cref="meQueryVersionLibrary"/>
        /// <seealso cref="meQueryVersionMainDriver"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meQueryVersionDeviceDriver(int device, out int version);

        #endregion


        #region meUtilityExtractValues

        /// <summary>
        /// Extracts the values of the specified channel from an array of values taking the channel-list into account.
        /// </summary>
        /// <param name="channel">
        /// Channel index whose values should be extracted.
        /// </param>
        /// <param name="dataBuffer">
        /// Reference to the data buffer allocated for the function <see cref="meIOStreamRead"/>.
        /// </param>
        /// <param name="dataCount">
        /// Number of measurement values in data buffer <paramref name="dataBuffer"/>.
        /// </param>
        /// <param name="configList">
        /// Pointer to the channel-list, which was passed to the function <see cref="meIOStreamConfig"/>.
        /// </param>
        /// <param name="configListCount">
        /// Number of channel-list entries in <paramref name="configList"/>.
        /// </param>
        /// <param name="chanBuffer">
        /// Pointer to an array with the extracted values of the specified channel.
        /// </param>
        /// <param name="chanBufferCount">
        /// (in) Passing the size of the array <paramref name="chanBuffer"/> in number of values.
        /// (out) The function returns the number values actually written to <paramref name="chanBuffer"/>.
        /// Feld <paramref name="chanBuffer"/> channBuffer kopiert.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <remarks>
        /// To extract the channels for several channels the function must be called separately. 
        /// Note: If channel is not on list the function returns ME_ERRNO_SUCCESS 
        /// and <paramref name="chanBufferCount"/> is set to "0".
        /// </remarks>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meUtilityExtractValues(int channel, ref int dataBuffer, int dataCount, ref meIOStreamConfig_t configList, int configListCount, ref int chanBuffer, ref int chanBufferCount);

        #endregion


        #region meUtilityDigitalToPhysical

        /// <summary>
        /// Conversion of the standardized digital values into the appropriate physical unit.
        /// </summary>
        /// <param name="min">
        /// The lower limit of the range e. g. -10[V].
        /// </param>
        /// <param name="max">
        /// The upper limit of the range e. g. +10[V].
        /// </param>
        /// <param name="maxData">
        /// The maximum resolution of the range e. g. 65536 at 16 bit resolution.
        /// </param>
        /// <param name="data">
        /// Digital value (0...65535) to be converted.
        /// </param>
        /// <param name="moduleType">
        /// Support for ME-MultiSig module.
        /// If for the current calculation no plug-on module must be taken into account,
        /// pass the constant: <see cref="ME_MODULE_TYPE_MULTISIG_NONE"/>  
        /// No plug-on module used (standard)
        /// </param>
        /// <param name="refValue">
        /// For a RTD module or a thermo-couple module. Otherwise use: ME_VALUE_NOT_USED.
        /// </param>
        /// <param name="physical">
        /// Result in the corresponding physical unit [V], [A], [C].
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_VALUE_OUT_OF_RANGE"/>: passed value is lower than "0" or bigger than <paramref name="maxData"/>.
        /// <see cref="ME_ERRNO_INVALID_MIN_MAX"/>: passed <paramref name="maxData"/> is not valid.
        /// <see cref="ME_ERRNO_INVALID_MODULE_TYPE"/>: passed module type is not supported.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <remarks>
        /// If you read data from an input sub-device in streaming mode, you should 
        /// apply the function <see cref="meUtilityExtractValues"/> to the array of values before 
        /// calling this function. Only by that way it is guarenteed, that different gain 
        /// factors and the usage of different plug-on modules in combination with the 
        /// ME-MultiSig system can be taken into account when calculating.
        /// If you want to apply the function to a whole array of values we recommend 
        /// the function <see cref="meUtilityDigitalToPhysicalV"/>.
        /// The parameters <paramref name="min"/> and <paramref name="max"/> must correspond with the limits of 
        /// the measurement range choosen in the functions <see cref="meIOSingleConfig"/> 
        /// resp. <see cref="meIOStreamConfig"/>. 
        /// The parameters <paramref name="min"/>, <paramref name="max"/> and <paramref name="physical"/> must be given 
        /// always in the same decimal power of the respective base unit (e. g. either "mV" or "V"). 
        /// The physical unit is not relevant for calculation.
        /// </remarks>
        /// <seealso cref="meQueryRangeInfo"/>
        /// <seealso cref="meUtilityExtractValues"/>
        /// <seealso cref="meUtilityDigitalToPhysicalV"/>
        /// <seealso cref="meIOSingleConfig"/>
        /// <seealso cref="meIOStreamConfig"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meUtilityDigitalToPhysical(double min, double max, int maxData, int data, int moduleType, double refValue, out double physical);

        #endregion


        #region meUtilityDigitalToPhysicalV

        /// <summary>
        /// Conversion of a whole array of standardized digital values into the appropriate physical unit.
        /// </summary>
        /// <param name="min">
        /// The lower limit of the range.
        /// </param>
        /// <param name="max">
        /// The upper limit of the range.
        /// </param>
        /// <param name="maxData">
        /// The maximum resolution of the range.
        /// </param>
        /// <param name="dataBuffer">
        /// Pointer to an array of digital values to be converted.
        /// </param>
        /// <param name="dataCount">
        /// Number of values in the array.
        /// </param>
        /// <param name="moduleType">
        /// Support for ME-MultiSig module.
        /// If for the current calculation no plug-on module must be taken into account,
        /// pass the constant: <see cref="ME_MODULE_TYPE_MULTISIG_NONE"/>  
        /// No plug-on module used (standard)
        /// </param>
        /// <param name="refValue">
        /// For a RTD module or a thermo-couple module. Otherwise use: ME_VALUE_NOT_USED.
        /// </param>
        /// <param name="physicalBuffer">
        /// Pointer to an array for the results in the appropriate physical unit [V], 
        /// [A], [C]. Has to be able to store <paramref name="dataCount"/> values.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_VALUE_OUT_OF_RANGE"/>: passed value is lower than "0" or bigger than <paramref name="maxData"/>.
        /// <see cref="ME_ERRNO_INVALID_MIN_MAX"/>: passed <paramref name="maxData"/> is not valid.
        /// <see cref="ME_ERRNO_INVALID_MODULE_TYPE"/>: passed module type is not supported.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <seealso cref="meQueryRangeInfo"/>
        /// <seealso cref="meUtilityDigitalToPhysical"/>
        /// <seealso cref="meIOSingleConfig"/>
        /// <seealso cref="meIOStreamConfig"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meUtilityDigitalToPhysicalV(double min, double max, int maxData, ref int dataBuffer, int dataCount, int moduleType, double refValue, ref double physicalBuffer);
        
        #endregion


        #region meUtilityPhysicalToDigital

        /// <summary>
        /// Conversion of values to be output in standardized digital values which are appropriate for the converter.
        /// </summary>
        /// <param name="min">
        /// The lower limit of the range, e. g. -10[V].
        /// </param>
        /// <param name="max">
        /// The upper limit of the range, e. g. +10[V].
        /// </param>
        /// <param name="maxData">
        /// The maximum resolution of the range, e. g. 65536 at 16 bit resolution.
        /// </param>
        /// <param name="physical">
        /// Voltage or current value to be converted, e. g. +0.75[V].
        /// </param>
        /// <param name="data">
        /// Result as a digital value to be output.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_VALUE_OUT_OF_RANGE"/>: passed value is lower than <paramref name="min"/> or bigger than <paramref name="max"/>.
        /// <see cref="ME_ERRNO_INVALID_MIN_MAX"/>: passed <paramref name="maxData"/> is not valid.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointer is NULL.
        /// </returns>
        /// <seealso cref="meQueryRangeInfo"/>
        /// <seealso cref="meUtilityPhysicalToDigitalV"/>
        /// <seealso cref="meUtilityDigitalToPhysical"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meUtilityPhysicalToDigital(double min, double max, int maxData, double physical, out int data);


        #endregion


        #region meUtilityPhysicalToDigitalV

        /// <summary>
        /// Conversion of a whole array of values to be output in standardized digital values.
        /// </summary>
        /// <param name="min">
        /// The lower limit of the range, e. g. -10[V].
        /// </param>
        /// <param name="max">
        /// The upper limit of the range, e. g. +10[V].
        /// </param>
        /// <param name="maxData">
        /// The maximum resolution of the range, e. g. 65536 at 16 bit resolution.
        /// </param>
        /// <param name="physicalBuffer">
        /// Pointer to an array with the voltage or current values to be converted.
        /// </param>
        /// <param name="physicalCount">
        /// Number of values in the array.
        /// </param>
        /// <param name="dataBuffer">
        /// Pointer to an array with the digital values to be output. Has to be able to store <paramref name="physicalCount"/> values.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_VALUE_OUT_OF_RANGE"/>: passed value is lower than <paramref name="min"/> or bigger than <paramref name="max"/>.
        /// <see cref="ME_ERRNO_INVALID_MIN_MAX"/>: passed <paramref name="maxData"/> is not valid.
        /// <see cref="ME_ERRNO_INVALID_POINTER"/>: passed pointers are NULL.
        /// </returns>
        /// <seealso cref="meQueryRangeInfo"/>
        /// <seealso cref="meUtilityPhysicalToDigital"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meUtilityPhysicalToDigitalV(double min, double max, int maxData, ref double physicalBuffer, int physicalCount, ref int dataBuffer);

        #endregion


        #region meUtilityPWMStart

        /// <summary>
        /// configures the counter device 8254 for the operation mode "pulse width modulation" and starts the operation.
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed.
        /// </param>
        /// <param name="subdevice1">
        /// Index of the sub-device counter 0 (used as prescaler).
        /// </param>
        /// <param name="subdevice2">
        /// Index of the sub-device counter 1.
        /// </param>
        /// <param name="subdevice3">
        /// Index of the sub-device counter 2.
        /// </param>
        /// <param name="reference">
        /// Defines the clock source for counter 0 (CLK_0)
        /// <see cref="ME_REF_CTR_INTERNAL_1MHZ"/>: Clock source is the internal 1 MHz crystal oszillator.
        /// <see cref="ME_REF_CTR_INTERNAL_10MHZ"/>: Clock source is the internal 10 MHz crystal oszillator.
        /// <see cref="ME_REF_CTR_EXTERNAL"/>: Clock source is an external oszillator.
        /// </param>
        /// <param name="prescaler">
        /// Value for the prescaler (counter 0) in the range 2...65535.
        /// </param>
        /// <param name="dutyCycle">
        /// TDuty cycle of the output signal from 1% 99% adjustable in steps of 1%.
        /// </param>
        /// <param name="flags">
        /// Flag for extended options:
        /// <see cref="ME_PWM_START_NO_FLAGS"/>: default settings
        /// <see cref="ME_PWM_START_CONNECT_INTERNAL"/>: If supported by hardware (e. g. ME-1400 series), 
        /// connect OUT_1 with GATE_2 internally. This reduces the number of external connections.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_FLAGS"/>: not supported flag detected.
        /// <see cref="ME_ERRNO_INVALID_REF"/>: used signal source not available.
        /// <see cref="ME_ERRNO_INVALID_DUTY_CYCLE"/>: value outside of supported range.
        /// <see cref="ME_ERRNO_NOT_SUPPORTED"/>: used sub-device is not a counter or internal connection can not be done
        /// </returns>
        /// <remarks>
        /// A further usage of the counters 0...2 is not possible in this operation mode. 
        /// The signal is available at OUT_2 of the specified counter device. 
        /// Depending on device type the base clock (max. 10 MHz) must be provided externally 
        /// or (if supported by the hardware) an on-board crystal oszillator can be used. 
        /// Counter 0 is used as a prescaler. The frequency of the output signal is max. 50 kHz
        /// and can be calculated as follows:
        /// <code>fOut_2 = baseClock / prescaler * 100;  //with prescaler = 2Exp(16-1)</code>
        /// The duty cycle can be set between 1...99% in steps of 1%
        /// </remarks>
        /// <seealso cref="meUtilityPWMStop"/>
        /// <seealso cref="meUtilityPWMRestart"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meUtilityPWMStart(int device, int subdevice1, int subdevice2, int subdevice3, int reference, int prescaler, int dutyCycle, int flags);

        #endregion


        #region meUtilityPWMRestart

        /// <summary>
        /// Restarts a PWM operation stopped by the function <see cref="meUtilityPWMStop()"/>
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed
        /// </param>
        /// <param name="subdevice1">
        /// Index of the sub-device counter 0 used as prescaler
        /// </param>
        /// <param name="reference">
        /// Defines the clock source for counter 0 (CLK_0)
        /// <see cref="ME_REF_CTR_INTERNAL_1MHZ"/>: Clock source is the internal 1 MHz crystal oszillator.
        /// <see cref="ME_REF_CTR_INTERNAL_10MHZ"/>: Clock source is the internal 10 MHz crystal oszillator.
        /// <see cref="ME_REF_CTR_EXTERNAL"/>: Clock source is an external oszillator.
        /// </param>
        /// <param name="prescaler">
        /// Value for the prescaler (counter 0) in the range 2...65535.
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_REF"/>: used signal source not available.
        /// <see cref="ME_ERRNO_NOT_SUPPORTED"/>: used sub-device is no counter.
        /// </returns>
        /// <seealso cref="meUtilityPWMStop"/>
        /// <seealso cref="meUtilityPWMStart"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meUtilityPWMRestart(int device, int subdevice1, int reference, int prescaler);
        
        #endregion
        
        
        #region meUtilityPWMStop

        /// <summary>
        /// Stops a PWM operation started by the function <see cref="meUtilityPWMStart"/> 
        /// </summary>
        /// <param name="device">
        /// Index of the device to be accessed
        /// </param>
        /// <param name="subdevice1">
        /// Index of the sub-device counter 0 used as prescaler
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_NOT_OPEN"/>: ME-iDS is not properly open.
        /// <see cref="ME_ERRNO_INVALID_DEVICE"/>: no device mapped to requested index.
        /// <see cref="ME_ERRNO_INVALID_SUBDEVICE"/>: on requested device no sub-device mapped to requested index.
        /// <see cref="ME_ERRNO_NOT_SUPPORTED"/>: used sub-device is no counter.
        /// </returns>
        /// <seealso cref="meUtilityPWMStart"/>
        /// <seealso cref="meUtilityPWMRestart"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meUtilityPWMStop(int device, int subdevice1);

        #endregion
        

        #region mePropertyGetIntA

        /// <summary>
        /// Read an integer value from the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as ANSI-string
        /// </param>
        /// <param name="propertyValue">
        /// Variable that receives the integer value
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertyGetIntA is the ANSI character version of mePropertyGetIntW
        /// </remarks>
        /// <seealso cref="mePropertyGetIntW"/>
        /// <seealso cref="mePropertySetIntA"/>
        /// <seealso cref="mePropertySetIntW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int mePropertyGetIntA([MarshalAs(UnmanagedType.LPStr)]String propertyPath, out int propertyValue);

        #endregion


        #region mePropertyGetIntW

        /// <summary>
        /// Read an integer value from the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as UTF-16 Unicode-string
        /// </param>
        /// <param name="propertyValue">
        /// Variable that receives the integer value
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertyGetIntW is the wide character version of mePropertyGetIntA
        /// </remarks>
        /// <seealso cref="mePropertyGetIntA"/>
        /// <seealso cref="mePropertySetIntA"/>
        /// <seealso cref="mePropertySetIntW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int mePropertyGetIntW([MarshalAs(UnmanagedType.LPWStr)]String propertyPath, out int propertyValue);

        #endregion


        #region mePropertyGetDoubleA

        /// <summary>
        /// Read a double value from the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as ANSI-string
        /// </param>
        /// <param name="propertyValue">
        /// Variable that receives the double value
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertyGetDoubleA is the ANSI character version of mePropertyGetDoubleW
        /// </remarks>
        /// <seealso cref="mePropertyGetDoubleW"/>
        /// <seealso cref="mePropertySetDoubleA"/>
        /// <seealso cref="mePropertySetDoubleW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int mePropertyGetDoubleA([MarshalAs(UnmanagedType.LPStr)]String propertyPath, out double propertyValue);

        #endregion


        #region mePropertyGetDoubleW

        /// <summary>
        /// Read a double value from the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as UTF-16 Unicode-string
        /// </param>
        /// <param name="propertyValue">
        /// Variable that receives the double value
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertyGetDoubleW is the wide character version of mePropertyGetDoubleA
        /// </remarks>
        /// <seealso cref="mePropertyGetDoubleA"/>
        /// <seealso cref="mePropertySetDoubleA"/>
        /// <seealso cref="mePropertySetDoubleW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int mePropertyGetDoubleW([MarshalAs(UnmanagedType.LPWStr)]String propertyPath, out double propertyValue);

        #endregion


        #region mePropertyGetStringA

        /// <summary>
        /// Read a string value from the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as ANSI-string
        /// </param>
        /// <param name="propertyValue">
        /// Variable that receives the ANSI-string value
        /// </param>
        /// <param name="count">
        /// Buffer length in characters including the terminating null character
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_PROPERTY_BUFFER_TOO_SMALL"/>: the buffer is not large enough to hold the string
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertyGetStringA is the ANSI character version of mePropertyGetStringW
        /// The required buffer length can be determined with the property attribute "Length" 
        /// </remarks>
        /// <seealso cref="mePropertyGetStringW"/>
        /// <seealso cref="mePropertySetStringA"/>
        /// <seealso cref="mePropertySetStringW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int mePropertyGetStringA([MarshalAs(UnmanagedType.LPStr)]String propertyPath, StringBuilder propertyValue, int count);

        #endregion


        #region mePropertyGetStringW

        /// <summary>
        /// Read a string value from the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as UTF-16 Unicode-string
        /// </param>
        /// <param name="propertyValue">
        /// Variable that receives the UTF-16 Unicode-string value
        /// </param>
        /// <param name="count">
        /// Buffer length in characters, not bytes, including the terminating null character
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// <see cref="ME_ERRNO_PROPERTY_BUFFER_TOO_SMALL"/>: the buffer is not large enough to hold the string
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// The required buffer length can be determined with the property attribute "Length" 
        /// mePropertyGetStringW is the wide character version of mePropertyGetStringA
        /// </remarks>
        /// <seealso cref="mePropertyGetStringA"/>
        /// <seealso cref="mePropertySetStringA"/>
        /// <seealso cref="mePropertySetStringW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int mePropertyGetStringW([MarshalAs(UnmanagedType.LPWStr)]String propertyPath, StringBuilder propertyValue, int count);

        #endregion


        #region mePropertySetIntA

        /// <summary>
        /// Set an integer value of the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as ANSI-string
        /// </param>
        /// <param name="propertyValue">
        /// The integer value to be set
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertySetIntA is the ANSI character version of mePropertySetIntW
        /// </remarks>
        /// <seealso cref="mePropertySetIntW"/>
        /// <seealso cref="mePropertyGetIntA"/>
        /// <seealso cref="mePropertyGetIntW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int mePropertySetIntA([MarshalAs(UnmanagedType.LPStr)]String propertyPath, int propertyValue);

        #endregion


        #region mePropertySetIntW

        /// <summary>
        /// Set an integer value of the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as UTF-16 Unicode-string
        /// </param>
        /// <param name="propertyValue">
        /// The integer value to be set
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertySetIntW is the wide character version of mePropertySetIntA
        /// </remarks>
        /// <seealso cref="mePropertySetIntA"/>
        /// <seealso cref="mePropertyGetIntA"/>
        /// <seealso cref="mePropertyGetIntW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int mePropertySetIntW([MarshalAs(UnmanagedType.LPWStr)]String propertyPath, int propertyValue);

        #endregion


        #region mePropertySetDoubleA

        /// <summary>
        /// Set a double value of the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as ANSI-string
        /// </param>
        /// <param name="propertyValue">
        /// The double value to be set
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertySetIntA is the ANSI character version of mePropertySetDoubleW
        /// </remarks>
        /// <seealso cref="mePropertySetDoubleW"/>
        /// <seealso cref="mePropertyGetDoubleA"/>
        /// <seealso cref="mePropertyGetDoubleW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int mePropertySetDoubleA([MarshalAs(UnmanagedType.LPStr)]String propertyPath, double propertyValue);

        #endregion


        #region mePropertySetDoubleW

        /// <summary>
        /// Set a double value of the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as UTF-16 Unicode-string
        /// </param>
        /// <param name="propertyValue">
        /// The double value to be set
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertySetDoubleW is the wide character version of mePropertySetDoubleA
        /// </remarks>
        /// <seealso cref="mePropertySetDoubleA"/>
        /// <seealso cref="mePropertyGetDoubleA"/>
        /// <seealso cref="mePropertyGetDoubleW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int mePropertySetDoubleW([MarshalAs(UnmanagedType.LPWStr)]String propertyPath, double propertyValue);

        #endregion


        #region mePropertySetStringA

        /// <summary>
        /// Set a string value of the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as ANSI-string
        /// </param>
        /// <param name="propertyValue">
        /// The ANSI-string value to be set
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertySetStringA is the ANSI character version of mePropertyGetStringW
        /// </remarks>
        /// <seealso cref="mePropertySetStringW"/>
        /// <seealso cref="mePropertyGetStringA"/>
        /// <seealso cref="mePropertyGetStringW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int mePropertySetStringA([MarshalAs(UnmanagedType.LPStr)]String propertyPath, [MarshalAs(UnmanagedType.LPStr)]String propertyValue);

        #endregion


        #region mePropertySetStringW

        /// <summary>
        /// Set a string value of the given property
        /// </summary>
        /// <param name="propertyPath">
        /// Property path as UTF-16 Unicode-string
        /// </param>
        /// <param name="propertyValue">
        /// The UTF-16 Unicode-string value to be set
        /// </param>
        /// <returns>
        /// <see cref="ME_ERRNO_SUCCESS"/>: the function returned successfully.
        /// The return value is an error code if there is a failure.
        /// </returns>
        /// <remarks>
        /// mePropertySetStringW is the wide character version of mePropertySetStringA
        /// </remarks>
        /// <seealso cref="mePropertySetStringA"/>
        /// <seealso cref="mePropertyGetStringA"/>
        /// <seealso cref="mePropertyGetStringW"/>
        [DllImport("meIDSmain.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int mePropertySetStringW([MarshalAs(UnmanagedType.LPWStr)]String propertyPath, [MarshalAs(UnmanagedType.LPWStr)]String propertyValue);

        #endregion

    #endregion //MEiDSFunctions

    }
}

