﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using meIDS_CS;
using meIDSTools;

namespace TestConsole_meIOExtIrqCallback_CS
{
    class TestConsole_meIOExtIrqCallback
    {
 	    struct SExtIrqSubddevice
	    {
            public int m_iDeviceIndex;

		    public int m_iSubdeviceIndex;

		    public int m_iNumberOfEdgeTypes;

		    public int[] m_arrEdgeTypes;
	    };

        static void Main(string[] args)
        {
            int i_number_of_devices = TestConsoleGeneric.TestConsoleMEIDSProlog();

            if (i_number_of_devices <= 0)
            {
                return;
            }

	        // ---------------------------------------------------------------------
        	
	        // Count the AI sub-devices in the system 

            int i_number_ext_irq_subdevices = 0;

	        int i_me_error = 0;

	        for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	        {
		        int i_vendor_id;
		        int i_device_id;
		        int i_serial_no;
		        int i_bus_type;
		        int i_bus_no;
		        int i_dev_no;
		        int i_func_no;
		        int i_plugged;
        		
		        i_me_error = meIDS.meQueryInfoDevice(	index_device,				    // Device index
										                out i_vendor_id,				// Vendor ID returned here				
										                out i_device_id,				// Device ID returned here
										                out i_serial_no,				// Serial number returned here
										                out i_bus_type,				    // Bus type returned here
										                out i_bus_no,					// Bus number returned here
										                out i_dev_no,					// Bus device number returned here
										                out i_func_no,					// Bus function number returned here
										                out i_plugged				);	// Plugged status returned here

		        if(i_plugged != meIDS.ME_PLUGGED_IN)
		        {
			        continue;
		        }

		        int index_subdevice = 0;
        		
		        int i_number_of_subdevices;

		        i_me_error = meIDS.meQueryNumberSubdevices(	index_device,				    // Device index
												            out i_number_of_subdevices);	// Number of subdevices returned here
        		
		        while(true)
		        {
			        i_me_error = meIDS.meQuerySubdeviceByType(  index_device,					// Device
												                index_subdevice,				// Begin search at this sub-device 
                                                                meIDS.ME_TYPE_EXT_IRQ,			// Type of sub-device to search for
												                meIDS.ME_SUBTYPE_ANY,			// Sub-type of sub-device to search for
												                out index_subdevice	    );		// sub-device index returned here

			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
				        break;
			        }

                    ++i_number_ext_irq_subdevices;

			        if(++index_subdevice >= i_number_of_subdevices)
			        {
				        break;
			        }
		        }
	        }

            if (i_number_ext_irq_subdevices == 0)
	        {
                System.Console.Write("****    No AI sub-devices found in system    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        meIDS.meClose(0);
        		
		        return;
	        }

            // Make a list of the External IRQ sub-devices 

            SExtIrqSubddevice[] arrSubdevices = new SExtIrqSubddevice[i_number_ext_irq_subdevices];

            i_number_ext_irq_subdevices = 0;

	        for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	        {
		        int i_vendor_id;
		        int i_device_id;
		        int i_serial_no;
		        int i_bus_type;
		        int i_bus_no;
		        int i_dev_no;
		        int i_func_no;
		        int i_plugged;
        		
		        i_me_error = meIDS.meQueryInfoDevice(	index_device,				// Device index
										                out i_vendor_id,			// Vendor ID returned here				
										                out i_device_id,			// Device ID returned here
										                out i_serial_no,			// Serial number returned here
										                out i_bus_type,				// Bus type returned here
										                out i_bus_no,				// Bus number returned here
										                out i_dev_no,				// Bus device number returned here
										                out i_func_no,				// Bus function number returned here
										                out i_plugged				);	// Plugged status returned here

		        if(i_plugged != meIDS.ME_PLUGGED_IN)
		        {
			        continue;
		        }

		        int index_subdevice = 0;
        		
		        int i_number_of_subdevices;

		        i_me_error = meIDS.meQueryNumberSubdevices(	index_device,				    // Device index
												            out i_number_of_subdevices);	// Number of subdevices returned here
        		
		        while(true)
		        {
			        i_me_error = meIDS.meQuerySubdeviceByType(  index_device,					// Device
												                index_subdevice,				// Begin search at this sub-device 
                                                                meIDS.ME_TYPE_EXT_IRQ,	    	// Type of sub-device to search for
												                meIDS.ME_SUBTYPE_ANY,			// Sub-type of sub-device to search for
												                out index_subdevice	    );		// sub-device index returned here

			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
				        break;
			        }

                    arrSubdevices[i_number_ext_irq_subdevices] = new SExtIrqSubddevice();

                    arrSubdevices[i_number_ext_irq_subdevices].m_iDeviceIndex = index_device;

                    arrSubdevices[i_number_ext_irq_subdevices].m_iSubdeviceIndex = index_subdevice;

                    arrSubdevices[i_number_ext_irq_subdevices].m_arrEdgeTypes = new int[3];

                    int i_caps;

                    i_me_error = meIDS.meQuerySubdeviceCaps(index_device,				// Device
                                                            index_subdevice,			// Begin search at this sub-device 
                                                            out i_caps      );			// sub-device caps returned here

                    int i_number_edge_types = 0;

                    if ((i_caps & meIDS.ME_CAPS_EXT_IRQ_EDGE_RISING) == meIDS.ME_CAPS_EXT_IRQ_EDGE_RISING)
                    {
                        arrSubdevices[i_number_ext_irq_subdevices].m_arrEdgeTypes[i_number_edge_types] = meIDS.ME_IRQ_EDGE_RISING;

                        ++i_number_edge_types;
                    }

                    if ((i_caps & meIDS.ME_CAPS_EXT_IRQ_EDGE_FALLING) == meIDS.ME_CAPS_EXT_IRQ_EDGE_FALLING)
                    {
                        arrSubdevices[i_number_ext_irq_subdevices].m_arrEdgeTypes[i_number_edge_types] = meIDS.ME_IRQ_EDGE_FALLING;

                        ++i_number_edge_types;
                    }

                    if ((i_caps & meIDS.ME_CAPS_EXT_IRQ_EDGE_ANY) == meIDS.ME_CAPS_EXT_IRQ_EDGE_ANY)
                    {
                        arrSubdevices[i_number_ext_irq_subdevices].m_arrEdgeTypes[i_number_edge_types] = meIDS.ME_IRQ_EDGE_ANY;

                        ++i_number_edge_types;
                    }

                    arrSubdevices[i_number_ext_irq_subdevices].m_iNumberOfEdgeTypes = i_number_edge_types;

                    ++i_number_ext_irq_subdevices;

                    if (++index_subdevice >= i_number_of_subdevices)
                    {
                        break;
                    }
                }
	        }

            System.Console.Write("{0} External IRQ sub-devices found in System\n\n", i_number_ext_irq_subdevices);

            for (int index_subdevice = 0; index_subdevice < i_number_ext_irq_subdevices; index_subdevice++)
            {
                System.Console.Write("{0}.   Device: {1}  Sub-device: {2} Supports ",
                                        index_subdevice + 1,
                                        arrSubdevices[index_subdevice].m_iDeviceIndex,
                                        arrSubdevices[index_subdevice].m_iSubdeviceIndex);

                for (int index_edge_type = 0; index_edge_type < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes; index_edge_type++)
                {
                    if (arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == meIDS.ME_IRQ_EDGE_RISING)
                    {
                        System.Console.Write("Rising edge");
                    }
                    else if (arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == meIDS.ME_IRQ_EDGE_FALLING)
                    {
                        System.Console.Write("Falling edge");
                    }
                    else // if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == meIDS.ME_IRQ_EDGE_ANY)
                    {
                        System.Console.Write("Any edge");
                    }

                    if (index_edge_type < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes - 1)
                    {
                        System.Console.Write(", ");
                    }
                    else
                    {
                        System.Console.Write("\n");
                    }
                }
            }

	        System.Console.Write("Press any key to continue\n");

            System.Console.ReadKey(true);

	        System.Console.Write("\n\n");

            string read_line;

	        bool b_terminate = false;
        	
            while(true)
	        {
		        int index_subdevice;

                if (i_number_ext_irq_subdevices > 1)
		        {
			        while(true)
			        {
				        System.Console.Write("Choose Subdevice:\n\n");

                        for (index_subdevice = 0; index_subdevice < i_number_ext_irq_subdevices; index_subdevice++)
                        {
                            System.Console.Write("{0}.   Device: {1}  Sub-device: {2} Supports ",
                                                    index_subdevice + 1,
                                                    arrSubdevices[index_subdevice].m_iDeviceIndex,
                                                    arrSubdevices[index_subdevice].m_iSubdeviceIndex);

                            for (int index_edge_type = 0; index_edge_type < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes; index_edge_type++)
                            {
                                if (arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == meIDS.ME_IRQ_EDGE_RISING)
                                {
                                    System.Console.Write("Rising edge");
                                }
                                else if (arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == meIDS.ME_IRQ_EDGE_FALLING)
                                {
                                    System.Console.Write("Falling edge");
                                }
                                else // if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == meIDS.ME_IRQ_EDGE_ANY)
                                {
                                    System.Console.Write("Any edge");
                                }

                                if (index_edge_type < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes - 1)
                                {
                                    System.Console.Write(", ");
                                }
                                else
                                {
                                    System.Console.Write("\n");
                                }
                            }
                        }

                        System.Console.Write("\nYour choice (1 - {0}) (RETURN to terminate): ", i_number_ext_irq_subdevices); 

                        read_line = System.Console.ReadLine();
                        
				        System.Console.Write("\n\n");
        				
				        if(read_line.Length == 0)
				        {
					        b_terminate = true;
        					
					        break;
				        }
        				
				        index_subdevice = System.Convert.ToInt32(read_line);

                        if ((index_subdevice < 1) || (index_subdevice > i_number_ext_irq_subdevices))
				        {
					        continue;
				        }

				        break;
			        }

			        if(b_terminate)
			        {
				        break;
			        }

			        --index_subdevice;
		        }
		        else
		        {
			        index_subdevice = 0;
		        }

                if (b_terminate)
                {
                    break;
                }

                int index_edge;

                if (arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes > 1)
                {
                    while (true)
                    {
                        System.Console.Write("Choose trigger edge:\n\n");

                        for (index_edge = 0; index_edge < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes; index_edge++)
                        {
                            System.Console.Write("    {0}.   ", index_edge + 1);

                            if (arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == meIDS.ME_IRQ_EDGE_RISING)
                            {
                                System.Console.Write("Rising edge\n");
                            }
                            else if (arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == meIDS.ME_IRQ_EDGE_FALLING)
                            {
                                System.Console.Write("Falling edge\n");
                            }
                            else // if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == meIDS.ME_IRQ_EDGE_ANY)
                            {
                                System.Console.Write("Any edge\n");
                            }
                        }

                        System.Console.Write("\nYour choice (1 - {0}) (RETURN to terminate): ", arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes);

                        read_line = System.Console.ReadLine();

                        System.Console.Write("\n\n");

                        if (read_line.Length == 0)
                        {
                            b_terminate = true;

                            break;
                        }

                        index_edge = System.Convert.ToInt32(read_line);

                        if ((index_edge < 1) || (index_edge > arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes))
                        {
                            continue;
                        }

                        break;
                    }

                    if (b_terminate)
                    {
                        break;
                    }

                    --index_edge;
                }
                else
                {
                    index_edge = 0;
                }

                if (b_terminate)
                {
                    break;
                }

                meIDS.meIOIrqCB_t IrqCB = new meIDS.meIOIrqCB_t(IrqCallback);

		        i_me_error = meIDS.meIOIrqSetCallback(  arrSubdevices[index_subdevice].m_iDeviceIndex,		// Device index
										                arrSubdevices[index_subdevice].m_iSubdeviceIndex,	// Subdevice index
                                                        IrqCB,						        				// Callback routine
                                                        IntPtr.Zero,										// Callback contextz
                                                        meIDS.ME_IO_IRQ_SET_CALLBACK_NO_FLAGS           );	// Flags


		        System.Console.Write("Ready to monitor ");

		        if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == meIDS.ME_IRQ_EDGE_RISING)
		        {
			        System.Console.Write("Rising edge ");
		        }
		        else if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == meIDS.ME_IRQ_EDGE_FALLING)
		        {
			        System.Console.Write("Falling edge ");
		        }
		        else // if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == meIDS.ME_IRQ_EDGE_ANY)
		        {
			        System.Console.Write("Any edge ");
		        }

		        System.Console.Write("on Device: {0}, Subdevice: {1}\n\n", 
				                        arrSubdevices[index_subdevice].m_iDeviceIndex, 
				                        arrSubdevices[index_subdevice].m_iSubdeviceIndex);

		        System.Console.Write("Press any key to start monitoring external interrupts\n\n");
        			
		        System.Console.Write("Press the ESC key at any time to stop monitoring interrupts\n\n");
		
                System.Console.ReadKey(true);

		        i_me_error =  meIDS.meIOIrqStart(	arrSubdevices[index_subdevice].m_iDeviceIndex,					// Device index
									                arrSubdevices[index_subdevice].m_iSubdeviceIndex,				// Subdevice index
									                0,																// Channel index 	
									                meIDS.ME_IRQ_SOURCE_DIO_LINE,									// IRQ source
									                arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge],		// Type of signal edge
									                meIDS.ME_VALUE_NOT_USED,										// Additional argument - Not required here
									                meIDS.ME_IO_IRQ_START_NO_FLAGS								);	// Flags

		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
                    System.Console.Write("****    meIOIrqStart - Error: {0}    ****\n\n", i_me_error);

                    System.Console.Write("Press any key to terminate\n\n");

                    System.Console.ReadKey(true);

			        meIDS.meClose(0);
        			
			        return;
		        }

		        System.Console.Write("Monitoring interrupts\n\n");

                ConsoleKeyInfo cki;

                while (true)
                {
                    if (Console.KeyAvailable == true)
                    {
                        cki = Console.ReadKey(true);

                        if (cki.Key == ConsoleKey.Escape)
                        {
                            break;
                        }

                    }
                }
        			
		        i_me_error = meIDS.meIOIrqStop(	arrSubdevices[index_subdevice].m_iDeviceIndex,				// Device index
									            arrSubdevices[index_subdevice].m_iSubdeviceIndex,			// Subdevice index
									            0,															// Channel index 	
									            meIDS.ME_IO_IRQ_STOP_NO_FLAGS								);		// Flags

	
		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
                    System.Console.Write("****    meIOIrqStop - Error: {0}    ****\n\n", i_me_error);

                    System.Console.Write("Press any key to terminate\n\n");

                    System.Console.ReadKey(true);

			        meIDS.meClose(0);
        			
			        return;
		        }

		        System.Console.Write("Stopped monitoring interrupts\n\n");

		        System.Console.Write("Press any key to continue or ESC to exit\n\n");

                cki = Console.ReadKey(true);

		        if(cki.Key == ConsoleKey.Escape)
		        {
			        System.Console.Write("\n\n");

			        break;
		        }
	        }

	        // ---------------------------------------------------------------------
        	
	        meIDS.meClose(0);

            System.Console.Write("Program completed successfully - Press any key to terminate\n\n");

            System.Console.ReadKey(true);
        	
	        return;
        }
     
        private static int IrqCallback(int iDevice, int iSubdevice, int iChannel, 
							                int iIrqCount, int iValue, IntPtr pContext, int iErrorCode)
        {
	        if(iErrorCode == meIDS.ME_ERRNO_SUCCESS)
	        {
		        System.Console.Write("Interrupt: IRQ Count: {0}  Value: {1}\n", iIrqCount, iValue);
	        }
	        else
	        {
		        System.Console.Write("****    IrqCallback - Error: {0}    ****\n\n", iErrorCode);
	        }

	        return(0);
        }
   }
}

