﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using meIDS_CS;
using meIDSTools;

namespace TestConsole_meIOSingleWriteAnalog_CS
{
    class TestConsole_meIOSingleWriteAnalog
    {
        public class SAORange
        {
            public int m_iUnit;

            public double m_dPhysicalMin;
            public double m_dPhysicalMax;

            public int m_iDigitalMax;
        }

        private const int MAX_RANGES = 16;

        public class SAOSubddevice
        {
            public int m_iDeviceIndex;

            public int m_iSubdeviceIndex;

            public int m_iNumberOfChannels;

            public int m_iNumberOfRanges;

            public SAORange[] m_arrRanges;

            public SAOSubddevice()
            {
                m_arrRanges = new SAORange[16];
            }
        }

        static void Main(string[] args)
        {
            int i_number_of_devices = TestConsoleGeneric.TestConsoleMEIDSProlog();

            if (i_number_of_devices <= 0)
            {
                return;
            }

	        // ---------------------------------------------------------------------
        	
	        // Count the AO sub-devices in the system 

	        int i_number_ao_subdevices = 0;

	        int i_me_error = 0;

	        for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	        {
		        int i_vendor_id;
		        int i_device_id;
		        int i_serial_no;
		        int i_bus_type;
		        int i_bus_no;
		        int i_dev_no;
		        int i_func_no;
		        int i_plugged;
        		
		        i_me_error = meIDS.meQueryInfoDevice(	index_device,				// Device index
										                out i_vendor_id,				// Vendor ID returned here				
										                out i_device_id,				// Device ID returned here
										                out i_serial_no,				// Serial number returned here
										                out i_bus_type,				// Bus type returned here
										                out i_bus_no,					// Bus number returned here
										                out i_dev_no,					// Bus device number returned here
										                out i_func_no,					// Bus function number returned here
										                out i_plugged				);	// Plugged status returned here

		        if(i_plugged != meIDS.ME_PLUGGED_IN)
		        {
			        continue;
		        }

		        int index_subdevice = 0;
        		
		        int i_number_of_subdevices;

		        i_me_error = meIDS.meQueryNumberSubdevices(	index_device,				    // Device index
												            out i_number_of_subdevices);	// Number of subdevices returned here
        		
		        while(true)
		        {
			        i_me_error = meIDS.meQuerySubdeviceByType(  index_device,					// Device
												                index_subdevice,				// Begin search at this sub-device 
												                meIDS.ME_TYPE_AO,				// Type of sub-device to search for
												                meIDS.ME_SUBTYPE_ANY,			// Sub-type of sub-device to search for
												                out index_subdevice	    );		// sub-device index returned here

			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
				        break;
			        }

			        ++i_number_ao_subdevices;

			        if(++index_subdevice >= i_number_of_subdevices)
			        {
				        break;
			        }
		        }
	        }

	        if(i_number_ao_subdevices == 0)
	        {
                System.Console.Write("****    No AO sub-devices found in system    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        meIDS.meClose(0);
        		
		        return;
	        }

	        // Make a list of the AO sub-devices 

            SAOSubddevice[] arrSubdevices = new SAOSubddevice[i_number_ao_subdevices];

            i_number_ao_subdevices = 0;

	        for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	        {
		        int i_vendor_id;
		        int i_device_id;
		        int i_serial_no;
		        int i_bus_type;
		        int i_bus_no;
		        int i_dev_no;
		        int i_func_no;
		        int i_plugged;
        		
		        i_me_error = meIDS.meQueryInfoDevice(	index_device,				// Device index
										                out i_vendor_id,			// Vendor ID returned here				
										                out i_device_id,			// Device ID returned here
										                out i_serial_no,			// Serial number returned here
										                out i_bus_type,				// Bus type returned here
										                out i_bus_no,				// Bus number returned here
										                out i_dev_no,				// Bus device number returned here
										                out i_func_no,				// Bus function number returned here
										                out i_plugged				);	// Plugged status returned here

		        if(i_plugged != meIDS.ME_PLUGGED_IN)
		        {
			        continue;
		        }

		        int index_subdevice = 0;
        		
		        int i_number_of_subdevices;

		        i_me_error = meIDS.meQueryNumberSubdevices(	index_device,				    // Device index
												            out i_number_of_subdevices);	// Number of subdevices returned here
        		
		        while(true)
		        {
			        i_me_error = meIDS.meQuerySubdeviceByType(  index_device,					// Device
												                index_subdevice,				// Begin search at this sub-device 
												                meIDS.ME_TYPE_AO,						// Type of sub-device to search for
												                meIDS.ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												                out index_subdevice	    );			// sub-device index returned here

			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
				        break;
			        }

			        arrSubdevices[i_number_ao_subdevices] = new SAOSubddevice();
                    
                    arrSubdevices[i_number_ao_subdevices].m_iDeviceIndex = index_device;

			        arrSubdevices[i_number_ao_subdevices].m_iSubdeviceIndex = index_subdevice;

			        int i_number_of_channels;

			        i_me_error =  meIDS.meQueryNumberChannels(  index_device,					// Device index
												                index_subdevice,				// Subdevice index, 
												                out i_number_of_channels	);	// Number of ramges returned here


			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
                        System.Console.Write("****    meQueryNumberChannels - Error: {0}    ****\n\n", i_me_error);

                        System.Console.Write("Press any key to terminate\n\n");

                        System.Console.ReadKey(true);

                        meIDS.meClose(0);
        				
				        return;
			        }

			        arrSubdevices[i_number_ao_subdevices].m_iNumberOfChannels = i_number_of_channels;

			        int i_number_of_ranges;

			        i_me_error =  meIDS.meQueryNumberRanges(	index_device,					// Device index
												                index_subdevice,				// Subdevice index, 
												                meIDS.ME_UNIT_ANY,				// Unit
												                out i_number_of_ranges	);		// Number of ramges returned here


			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
                        System.Console.Write("****    meQueryNumberRanges - Error: {0}    ****\n\n", i_me_error);

                        System.Console.Write("Press any key to terminate\n\n");

                        System.Console.ReadKey(true);

				        meIDS.meClose(0);
        				
				        return;
			        }

			        if(i_number_of_ranges > MAX_RANGES)
			        {
				        i_number_of_ranges = MAX_RANGES;
			        }

                    arrSubdevices[i_number_ao_subdevices].m_iNumberOfRanges = i_number_of_ranges;

			        for(int index_range = 0; index_range < i_number_of_ranges; index_range++)
			        {
				        int i_unit;

				        double d_phys_min;
				        double d_phys_max;

				        int i_digital_max;

				        i_me_error = meIDS.meQueryRangeInfo(index_device,				// Device index
												            index_subdevice,			// Subdevice index, 
												            index_range,				// Range index
												            out i_unit,					// Unit returned here - not require
												            out d_phys_min,				// Physical minimum returned here
												            out d_phys_max,				// Physical maximum returned here
												            out i_digital_max		);	// Digital maximum value returned here

				        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
				        {
                            System.Console.Write("****    meQueryRangeInfo - Error: {0}    ****\n\n", i_me_error);

                            System.Console.Write("Press any key to terminate\n\n");

                            System.Console.ReadKey(true);

					        meIDS.meClose(0);
        					
					        return;
				        }

                        arrSubdevices[i_number_ao_subdevices].m_arrRanges[index_range] = new SAORange();

                        arrSubdevices[i_number_ao_subdevices].m_arrRanges[index_range].m_iUnit = i_unit;

                        arrSubdevices[i_number_ao_subdevices].m_arrRanges[index_range].m_dPhysicalMin = d_phys_min;
                        arrSubdevices[i_number_ao_subdevices].m_arrRanges[index_range].m_dPhysicalMax = d_phys_max;

                        arrSubdevices[i_number_ao_subdevices].m_arrRanges[index_range].m_iDigitalMax = i_digital_max;
			        }

                    ++i_number_ao_subdevices;

			        if(++index_subdevice >= i_number_of_subdevices)
			        {
				        break;
			        }
		        }
	        }

	        System.Console.Write("{0} AO sub-devices found in System\n\n", i_number_ao_subdevices);

            for (int index_subdevice = 0; index_subdevice < i_number_ao_subdevices; index_subdevice++)
	        {
		        System.Console.Write("{0}.   Device: {1}  Sub-device: {2}  Channels: {3}  Ranges: {4}\n",
			                            index_subdevice + 1,
			                            arrSubdevices[index_subdevice].m_iDeviceIndex,
			                            arrSubdevices[index_subdevice].m_iSubdeviceIndex,
			                            arrSubdevices[index_subdevice].m_iNumberOfChannels,
			                            arrSubdevices[index_subdevice].m_iNumberOfRanges		);
	        }

	        System.Console.Write("Press any key to continue\n");

            System.Console.ReadKey(true);

	        System.Console.Write("\n\n");

            string read_line;

	        bool b_terminate = false;
        	
            while(true)
	        {
		        int index_subdevice;

                if (i_number_ao_subdevices > 1)
		        {
			        while(true)
			        {
				        System.Console.Write("Choose Subdevice:\n\n");

				        for(index_subdevice = 0; index_subdevice < i_number_ao_subdevices; index_subdevice++)
				        {
		                    System.Console.Write("{0}.   Device: {1}  Sub-device: {2}  Channels: {3}  Ranges: {4}\n",
					                                index_subdevice + 1,
					                                arrSubdevices[index_subdevice].m_iDeviceIndex,
					                                arrSubdevices[index_subdevice].m_iSubdeviceIndex,
					                                arrSubdevices[index_subdevice].m_iNumberOfChannels,
					                                arrSubdevices[index_subdevice].m_iNumberOfRanges		);
				        }

                        System.Console.Write("\nYour choice (1 - {0}) (RETURN to terminate): ", i_number_ao_subdevices); 

				        read_line = System.Console.ReadLine();
                        
				        System.Console.Write("\n\n");
        				
				        if(read_line.Length == 0)
				        {
					        b_terminate = true;
        					
					        break;
				        }
        				
				        index_subdevice = System.Convert.ToInt32(read_line);
                        
				        if( (index_subdevice < 1)||(index_subdevice > i_number_ao_subdevices) )
				        {
					        continue;
				        }

				        break;
			        }

			        if(b_terminate)
			        {
				        break;
			        }

			        --index_subdevice;
		        }
		        else
		        {
			        index_subdevice = 0;
		        }

		        int index_channel = 0;

		        if(arrSubdevices[index_subdevice].m_iNumberOfChannels > 1)
		        {
			        while(true)
			        {
				        System.Console.Write("Channel number (0 - {0}) (RETURN to terminate): ", arrSubdevices[index_subdevice].m_iNumberOfChannels - 1); 

				        read_line = System.Console.ReadLine();
                        
				        System.Console.Write("\n\n");
        				
				        if(read_line.Length == 0)
				        {
					        b_terminate = true;
        					
					        break;
				        }
        				
				        index_channel = System.Convert.ToInt32(read_line);
                        
				        if( (index_channel < 0)||(index_channel >= arrSubdevices[index_subdevice].m_iNumberOfChannels) )
				        {
					        continue;
				        }
        				
				        break;
			        }
        		
			        if(b_terminate)
			        {
				        break;
			        }
		        }
		        else
		        {
			        index_channel = 0;
		        }

		        int index_range = 0;

		        if(arrSubdevices[index_subdevice].m_iNumberOfRanges > 1)
		        {
			        while(true)
			        {
				        System.Console.Write("Choose Range:\n\n");

				        for(index_range = 0; index_range < arrSubdevices[index_subdevice].m_iNumberOfRanges; index_range++)
				        {
					        System.Console.Write("{0}.   {1:F2} - {2:F2} {3}\n",
					        index_range + 1,
					        arrSubdevices[index_subdevice].m_arrRanges[index_range].m_dPhysicalMin,
					        arrSubdevices[index_subdevice].m_arrRanges[index_range].m_dPhysicalMax,
					        ( (arrSubdevices[index_subdevice].m_arrRanges[index_range].m_iUnit == meIDS.ME_UNIT_VOLT) ? "  Volt" : "Ampere" ) );
				        }
        				
				        System.Console.Write("\nYour choice (1 - {0}) (RETURN to terminate): ", arrSubdevices[index_subdevice].m_iNumberOfRanges);  

				        read_line = System.Console.ReadLine();
                        
				        System.Console.Write("\n\n");

                        if (read_line.Length == 0)
				        {
					        b_terminate = true;
        					
					        break;
				        }
        				
				        index_range = System.Convert.ToInt32(read_line);

				        if( (index_range < 1)||(index_range > arrSubdevices[index_subdevice].m_iNumberOfRanges) )
				        {
					        continue;
				        }
        				
				        break;
			        }

			        if(b_terminate)
			        {
				        break;
			        }

			        --index_range;
		        }
		        else
		        {
			        index_range = 0;
		        }
        	
		        // Configure the device

		        i_me_error = meIDS.meIOSingleConfig(	arrSubdevices[index_subdevice].m_iDeviceIndex,			// Device index
										                arrSubdevices[index_subdevice].m_iSubdeviceIndex,		// Subdevice index, 
										                index_channel,											// Channel index
										                index_range,											// Range index
                                                        meIDS.ME_REF_AO_GROUND,									// Reference
										                meIDS.ME_TRIG_CHAN_DEFAULT,								// Trigger channel - standard
										                meIDS.ME_TRIG_TYPE_SW,									// Trigger type - software
										                meIDS.ME_VALUE_NOT_USED,								// Trigger edge - not applicable
										                meIDS.ME_IO_SINGLE_CONFIG_NO_FLAGS					);	// Flags

		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
                    System.Console.Write("****    meIOSingleConfig - Error: {0}    ****\n\n", i_me_error);

                    System.Console.Write("Press any key to terminate\n\n");

                    System.Console.ReadKey(true);

			        meIDS.meClose(0);
        			
			        return;
		        }

                // Fetch the range information which will be required to convert from 
                // physical units to a digital value for the output below

                double d_phys_min;

                double d_phys_max;

                int i_digital_max;

                int i_unit;

                i_me_error = meIDS.meQueryRangeInfo( arrSubdevices[index_subdevice].m_iDeviceIndex,		// Device index
                                                    arrSubdevices[index_subdevice].m_iSubdeviceIndex,	// Subdevice index, 
                                                    index_range,										// Range index
                                                    out i_unit,												// Unit returned here
                                                    out d_phys_min,										// Physical minimum returned here
                                                    out d_phys_max,										// Physical maximum returned here
                                                    out i_digital_max                               );	// Digital maximum value returned here

                if (i_me_error != meIDS.ME_ERRNO_SUCCESS)
                {
                    System.Console.Write("****    meQueryRangeInfo - Error: {0}    ****\n\n", i_me_error);

                    System.Console.Write("Press any key to terminate\n\n");

                    System.Console.ReadKey(true);

                    meIDS.meClose(0);

                    return;
                }

                while (true)
		        {
                    double d_value;

                    System.Console.Write("Output value V ( {0:F2}  - {1:F2} ) (RETURN to terminate): ", d_phys_min, d_phys_max);

                    read_line = System.Console.ReadLine();

                    System.Console.Write("\n\n");

                    if (read_line.Length == 0)
                    {
                        break;
                    }

                    d_value = System.Convert.ToDouble(read_line);

                    if ((d_value < d_phys_min) || (d_phys_max < d_value))
                    {
                        continue;
                    }

                    int i_digital_value;

                    i_me_error = meIDS.meUtilityPhysicalToDigital(  d_phys_min,				// Minimum physiacal value
                                                                    d_phys_max,				// Maximum physical value
                                                                    i_digital_max,			// Maximum digital value
                                                                    d_value,				// Physical value to convert
                                                                    out i_digital_value);	// Corresponding digital value for meIOSingle returned here

                    if (i_me_error != meIDS.ME_ERRNO_SUCCESS)
                    {
                        System.Console.Write("****    meUtilityPhysicalToDigital - Error: {0}    ****\n\n", i_me_error);

                        System.Console.Write("Press any key to terminate\n\n");

                        System.Console.ReadKey(true);

                        meIDS.meClose(0);

                        return;
                    }

                    meIDS.meIOSingle_t[] io_single = new meIDS.meIOSingle_t[1];

			        io_single[0].iDevice = arrSubdevices[index_subdevice].m_iDeviceIndex;

			        io_single[0].iSubdevice = arrSubdevices[index_subdevice].m_iSubdeviceIndex;
			        io_single[0].iChannel	= index_channel;
                    io_single[0].iDir       = meIDS.ME_DIR_OUTPUT;
                    io_single[0].iValue     = i_digital_value;
			        io_single[0].iTimeOut	= 0;							        // No timeout - not required for software output
			        io_single[0].iFlags		= meIDS.ME_IO_SINGLE_TYPE_NO_FLAGS;
        			
			        i_me_error = meIDS.meIOSingle(  ref io_single[0],				   // Output list 
									                1,							    // Number of elements in the above list 
									                meIDS.ME_IO_SINGLE_NO_FLAGS	);	// Flags

			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
                        System.Console.Write("****    meIOSingle - Error: {0}    ****\n\n", i_me_error);

                        System.Console.Write("Press any key to terminate\n\n");

                        System.Console.ReadKey(true);
        				
				        meIDS.meClose(0);
        				
				        return;
			        }

                    System.Console.Write(   "Output {0:F2} V ( {1} ) to Device: {2}  Sub-device: {3}\n\n\n",
                                            d_value,
                                            i_digital_value,
                                            arrSubdevices[index_subdevice].m_iDeviceIndex,
                                            arrSubdevices[index_subdevice].m_iSubdeviceIndex                );
                }
	        }

	        // ---------------------------------------------------------------------
        	
	        meIDS.meClose(0);

            System.Console.Write("Program completed successfully - Press any key to terminate\n\n");

            System.Console.ReadKey(true);
        	
	        return;
        }
    }
}
