using System;
using System.Collections.Generic;
using System.Text;
using meIDS_CS;

namespace meIDSTools
{
    public sealed class TestConsoleGeneric
    {
        public static int TestConsoleMEIDSProlog()
        {
            System.Console.BufferWidth = 120;

            System.Console.BufferHeight = 2000;

	        // Enable the default error procedure

            meIDS.meErrorSetDefaultProc(meIDS.ME_SWITCH_ENABLE);

	        // Print the library version

            int i_version_library;

            int i_me_error = meIDS.meQueryVersionLibrary(out i_version_library);

            if (i_me_error != meIDS.ME_ERRNO_SUCCESS)
	        {
                System.Console.Write("****    QueryVersionLibrary - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        return (-1);
	        }

            System.Console.Write("Library version: {0}.{1}.{2}.{3}\n", (i_version_library >> 24) & 0x000000FF, (i_version_library >> 16) & 0x000000FF, (i_version_library >> 8) & 0x000000FF, i_version_library & 0x000000FF);

            i_me_error = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS);

            if (i_me_error != meIDS.ME_ERRNO_SUCCESS)
	        {
                System.Console.Write("****    Open - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        return(-1);
	        }

            int i_version_main_meIDS;

            i_me_error = meIDS.meQueryVersionMainDriver(out i_version_main_meIDS);

            if (i_me_error != meIDS.ME_ERRNO_SUCCESS)
            {
                System.Console.Write("****    QueryVersionMainmeIDS - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        return(-1);
	        }

            System.Console.Write("Main meIDS version: {0}.{1}.{2}.{3}\n", (i_version_main_meIDS >> 24) & 0x000000FF, (i_version_main_meIDS >> 16) & 0x000000FF, (i_version_main_meIDS >> 8) & 0x000000FF, i_version_main_meIDS & 0x000000FF);

            // Query the number of devices

            int i_number_of_devices;

            i_me_error = meIDS.meQueryNumberDevices(out i_number_of_devices);

            if (i_me_error != meIDS.ME_ERRNO_SUCCESS)
            {
                System.Console.Write("****    QueryNumberDevices - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

                return(-1);
            }

            if (i_number_of_devices == 0)
            {
                System.Console.Write("****    No ME-IDS devices found in system    ****\n\n");

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

                return(0);
            }

            System.Console.Write("Number of Devices: {0}\n\n", i_number_of_devices);

            StringBuilder sz_buffer = new StringBuilder(1024);

	        for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	        {
		        int i_vendor_id;
		        int i_device_id;
		        int i_serial_no;
		        int i_bus_type;
		        int i_bus_no;
		        int i_dev_no;
		        int i_func_no;
		        int i_plugged;

		        i_me_error = meIDS.meQueryInfoDevice(index_device, out i_vendor_id, out i_device_id, out i_serial_no, 
												        out i_bus_type, out i_bus_no, out i_dev_no, out i_func_no, out i_plugged);

		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
			        System.Console.Write("****    meQueryInfoDevice - Error: {0}    ****\n", i_me_error);
        			
			        System.Console.Write("Press any key to continue\n\n");
        			
                    System.Console.ReadKey(true);
        			
			        continue;
		        }
        		
		        System.Console.Write(	"{0}Device: {1} - {2}\n----------\n\n",
				                        (i_plugged == meIDS.ME_PLUGGED_IN) ? "" : "! ", 
				                        index_device, 
				                        (i_plugged == meIDS.ME_PLUGGED_IN) ? "Plugged in" : "NOT plugged in");

                i_me_error = meIDS.meQueryNameDevice(index_device, sz_buffer, sz_buffer.Capacity);

		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
			        System.Console.Write("****    meQueryNameDevice - Error: %d    ****\n", i_me_error);
        			
			        System.Console.Write("Press any key to continue\n\n");
        			
                    System.Console.ReadKey(true);
       			
			        continue;
		        }
        		
		        System.Console.Write("    Device name: {0}\n", sz_buffer);

                i_me_error = meIDS.meQueryDescriptionDevice(index_device, sz_buffer, sz_buffer.Capacity);

		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
			        System.Console.Write("****    meQueryNameDevice - Error: %d    ****\n", i_me_error);
        			
			        System.Console.Write("Press any key to continue\n\n");
        			
                    System.Console.ReadKey(true);
        			
			        continue;
		        }
        		
		        System.Console.Write("    Device description: {0}\n", sz_buffer);

                i_me_error = meIDS.meQueryNameDeviceDriver(index_device, sz_buffer, sz_buffer.Capacity);

		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
			        System.Console.Write("****    meQueryNameDevicemeIDS - Error: %d    ****\n", i_me_error);
        			
			        System.Console.Write("Press any key to continue\n\n");
        			
                    System.Console.ReadKey(true);
        			
			        continue;
		        }
        		
		        System.Console.Write("    Device meIDS name: {0}\n", sz_buffer);

		        int i_version_device_meIDS;

		        i_me_error = meIDS.meQueryVersionDeviceDriver(index_device, out i_version_device_meIDS);

		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
			        System.Console.Write("****    meQueryVersionDevicemeIDS - Error: %d    ****\n", i_me_error);
        			
			        System.Console.Write("Press any key to continue\n\n");
        			
                    System.Console.ReadKey(true);
        			
			        continue;
		        }

                System.Console.Write("    Device meIDS version: : {0}.{1}.{2}.{3}\n", (i_version_device_meIDS >> 24) & 0x000000FF, (i_version_device_meIDS >> 16) & 0x000000FF, (i_version_device_meIDS >> 8) & 0x000000FF, i_version_device_meIDS & 0x000000FF);

		        int i_number_of_subdevices;

                i_me_error = meIDS.meQueryNumberSubdevices(index_device, out i_number_of_subdevices);

		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
			        System.Console.Write("****    meQueryNumberSubdevices - Error: %d    ****\n", i_me_error);
        			
			        System.Console.Write("Press any key to continue\n\n");
        			
                    System.Console.ReadKey(true);
        			
			        continue;
		        }
        		
		        System.Console.Write("    Number of Subdevices: {0}\n\n", i_number_of_subdevices);

		        for(int index_subdevice = 0; index_subdevice < i_number_of_subdevices; index_subdevice++)
		        {
			        System.Console.Write(	"    Subdevice: {0}\n    ------------\n\n",
					                        index_subdevice			);

			        int i_subdevice_type;
			        int i_subdevice_subtype;

			        i_me_error = meIDS.meQuerySubdeviceType(index_device, index_subdevice, out i_subdevice_type, out i_subdevice_subtype);

			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
				        System.Console.Write("****    meQuerySubdeviceType - Error: %d    ****\n", i_me_error);
        				
			            System.Console.Write("Press any key to continue\n\n");
            			
                        System.Console.ReadKey(true);
        				
				        continue;
			        }

			        System.Console.Write("        Subdevice type:     0x{0:X}\n", i_subdevice_type);
                    System.Console.Write("        Subdevice subtype:  0x{0:X}\n", i_subdevice_subtype);
        			
			        int i_number_of_channels;
        			
			        i_me_error = meIDS.meQueryNumberChannels(index_device, index_subdevice, out i_number_of_channels);

			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
				        System.Console.Write("****    meQueryNumberChannels - Error: %d    ****\n", i_me_error);
        				
			            System.Console.Write("Press any key to continue\n\n");
            			
                        System.Console.ReadKey(true);
        				
				        continue;
			        }

			        System.Console.Write("        Number of channels: {0}\n\n", i_number_of_channels);
		        }

		        System.Console.Write("\n\n");
	        }

            return(i_number_of_devices);
        }
    }
}
