// Con_meIOStreamReadScan.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"


#include <stdio.h>
#include <conio.h>

#include <memory.h>

#include <windows.h>

extern "C"
{
#include "..\\medriver.h"
}

#include "..\\TestConsoleGeneric.h"

#ifdef WIN64

#pragma comment (lib, "..\\meIDSmain64.lib")

#else

#pragma comment (lib, "..\\meIDSmain.lib")

#endif

#define NUMBER_OF_CHANNELS      4

#define THRESHOLD				2

#define ACQUIRE_NUMBER_VALUES   NUMBER_OF_CHANNELS * 50


int _tmain(int argc, _TCHAR* argv[])
{
    // Call generic console function in parent directory to 
    // open system and list the available devices and subdevices.

    int i_number_of_devices = TestConsoleMEIDSProlog();

    if(i_number_of_devices <= 0)
    {
        return(-1);
    }

    // ---------------------------------------------------------------------
    
    int i_stream_read_device    = -1;
    int i_stream_read_subdevice = -1;

    // Search for an AI-Streaming subdevice in the system

    int i_me_error;

    for(int index_device = 0; index_device < i_number_of_devices; index_device++)
    {
        int i_vendor_id;
        int i_device_id;
        int i_serial_no;
        int i_bus_type;
        int i_bus_no;
        int i_dev_no;
        int i_func_no;
        int i_plugged;
        
        i_me_error = meQueryInfoDevice( index_device,               // Device index
                                        &i_vendor_id,               // Vendor ID returned here - not required               
                                        &i_device_id,               // Device ID returned here - not required
                                        &i_serial_no,               // Serial number returned here - not required
                                        &i_bus_type,                    // Bus type returned here - not required
                                        &i_bus_no,                  // Bus number returned here - not required
                                        &i_dev_no,                  // Bus device number returned here - not required
                                        &i_func_no,                 // Bus function number returned here - not required
                                        &i_plugged              );  // Plugged status returned hee - not required

        if(i_plugged != ME_PLUGGED_IN)
        {
            continue;
        }

        i_me_error = meQuerySubdeviceByType(index_device,                   // Device
                                            0,                              // Begin search at this sub-device 
                                            ME_TYPE_AI,                     // Type of sub-device to search for
                                            ME_SUBTYPE_STREAMING,           // Sub-type of sub-device to search for
                                            &i_stream_read_subdevice    );  // sub-device index returned here

        if(i_me_error == ME_ERRNO_SUCCESS)
        {
            i_stream_read_device = index_device;

            break;
        }
    }

    if(i_stream_read_device == -1)
    {
        printf("****    No AI-Streaming sub-device found in system    ****\n\n");
        
        printf("Press any key to terminate\n");
        
        _getch();
        
        meClose(0);
        
        return -1;
    }

    printf("Device: %d, Subdevice: %d is an AI-Streaming device\n\n", i_stream_read_device, i_stream_read_subdevice);
    
    meIOResetSubdevice(i_stream_read_device, i_stream_read_subdevice, ME_IO_RESET_SUBDEVICE_NO_FLAGS);

    double d_required_freq = 10.0;
    double d_achieved_freq = d_required_freq;

    int i_conv_ticks_low = 0;
    int i_conv_ticks_high = 0;

    i_me_error = meIOStreamFrequencyToTicks(i_stream_read_device,               // Device index
                                            i_stream_read_subdevice,            // Subdevice indexe, 
                                            ME_TIMER_CONV_START,                // Timer used 
                                            &d_achieved_freq,                   // Required time seconds, Achieved time returned here    
                                            &i_conv_ticks_low,                  // Lower 32 bits of the corresponding tick value returned here
                                            &i_conv_ticks_high,                 // Upper 32 bits of the corresponding tick value returned here
                                            ME_IO_FREQUENCY_TO_TICKS_NO_FLAGS); // No flags

    if(i_me_error != ME_ERRNO_SUCCESS)
    {
        printf("****    meIOStreamTimeToTicks - Error: %d    ****\n\n", i_me_error);

        printf("Press any key to terminate\n");
        
        _getch();
        
        meClose(0);
        
        return -1;
    }

    printf("meIOStreamFrequencyToTicks - Required Freq: %f Achieved Freq: %f Ticks low: %d Ticks high: %d\n\n", d_required_freq, d_achieved_freq, i_conv_ticks_low, i_conv_ticks_high);

    meIOStreamConfig_t arr_stream_config[NUMBER_OF_CHANNELS];

    for(int index_channel = 0; index_channel < NUMBER_OF_CHANNELS; index_channel++)
    {
        arr_stream_config[index_channel].iChannel = index_channel;

        arr_stream_config[index_channel].iStreamConfig = 0;
        arr_stream_config[index_channel].iRef = ME_REF_AI_GROUND;
        arr_stream_config[index_channel].iFlags = ME_IO_STREAM_CONFIG_TYPE_NO_FLAGS;
    }

    meIOStreamTrigger_t stream_trigger;

    memset( &stream_trigger, 0, sizeof(meIOStreamTrigger_t) );

    stream_trigger.iAcqStartTrigType = ME_TRIG_TYPE_SW;
    
    stream_trigger.iScanStartTrigType = ME_TRIG_TYPE_FOLLOW;

    stream_trigger.iConvStartTrigType = ME_TRIG_TYPE_TIMER;
    stream_trigger.iConvStartTicksLow = i_conv_ticks_low;
    stream_trigger.iConvStartTicksHigh = i_conv_ticks_high;
    stream_trigger.iScanStopTrigType = ME_TRIG_TYPE_NONE;

    stream_trigger.iAcqStopTrigType = ME_TRIG_TYPE_COUNT;
    stream_trigger.iAcqStopCount = ACQUIRE_NUMBER_VALUES;
    stream_trigger.iFlags = ME_IO_STREAM_TRIGGER_TYPE_NO_FLAGS;
    
    i_me_error = meIOStreamConfig(  i_stream_read_device,               // Device index
                                    i_stream_read_subdevice,            // Subdevice index, 
                                    &arr_stream_config[0],              // Pointer to an array of meIOStreamConfig_t structures
                                    NUMBER_OF_CHANNELS,                 // Number of elements in the stream config array above
                                    &stream_trigger,                    // Pointer to an meIOStreamTrigger_t structure
                                    THRESHOLD,                          // FIFO IRQ threshold
                                    ME_IO_STREAM_CONFIG_NO_FLAGS    );  // No flags

    if(i_me_error != ME_ERRNO_SUCCESS)
    {
        printf("****    meIOStreamConfig - Error: %d    ****\n\n", i_me_error);

        printf("Press any key to terminate\n");
        
        _getch();
        
        meClose(0);
        
        return -1;
    }

    double d_phys_min;

    double d_phys_max;

    int i_digital_max;
    
    i_me_error = meQueryRangeInfo(  i_stream_read_device,       // Device index
                                    i_stream_read_subdevice,    // Subdevice index, 
                                    0,                          // Range index
                                    NULL,                       // Unit returned here - not require
                                    &d_phys_min,                // Physical minimum returned here
                                    &d_phys_max,                // Physical maximum returned here
                                    &i_digital_max          );  // Digital maximum value returned here

    printf("d_phys_min: %lf d_phys_max: %lf i_digital_max: %d\n\n", d_phys_min, d_phys_max, i_digital_max);
               
    printf("The acquisition will automatically stop after %d values\n\n", ACQUIRE_NUMBER_VALUES);
        
    printf("Press any key to start the acquisition\n\n");

    _getch();
    
    meIOStreamStart_t arr_stream_start[1];

    arr_stream_start[0].iDevice = i_stream_read_device;
    arr_stream_start[0].iSubdevice = i_stream_read_subdevice;
    arr_stream_start[0].iStartMode = ME_START_MODE_BLOCKING;
    arr_stream_start[0].iTimeOut = 5;
    arr_stream_start[0].iFlags = ME_IO_STREAM_START_TYPE_NO_FLAGS;
    arr_stream_start[0].iErrno = 0;
    
    i_me_error = meIOStreamStart(   &arr_stream_start[0],           // Pointer to an array of meIOStreamStart_t structures 
                                    1,                              // Number of elements in the stream start array above
                                    ME_IO_STREAM_START_NO_FLAGS );  // No flags

    if(i_me_error != ME_ERRNO_SUCCESS)
    {
        printf("****    meIOStreamStart - Error: %d    ****\n\n", i_me_error);

        printf("Press any key to terminate\n");
        
        _getch();
        
        meClose(0);
        
        return -1;
    }

    printf("Acquisition started\n\n");

    int i_count;
    
    while(true)
    {
        int i_status;

        i_me_error = meIOStreamStatus(  i_stream_read_device,               // Device index
                                        i_stream_read_subdevice,            // Subdevice index, 
                                        ME_WAIT_NONE,                       // Return immediately
                                        &i_status,                          // Status returned here
                                        &i_count,                           // Count returned here
                                        ME_IO_STREAM_STATUS_NO_FLAGS    );  // Flags

        if(i_me_error != ME_ERRNO_SUCCESS)
        {
            printf("****    meIOStreamRead - Error: %d    ****\n\n", i_me_error);

            printf("Press any key to end the acquisition\n");
        
            _getch();

            meIOStreamStop_t arr_stream_stop[1];

            arr_stream_stop[0].iDevice = i_stream_read_device;
            arr_stream_stop[0].iSubdevice = i_stream_read_subdevice;
            arr_stream_stop[0].iStopMode = ME_STOP_MODE_IMMEDIATE;
            arr_stream_stop[0].iFlags = ME_IO_STREAM_STOP_TYPE_NO_FLAGS;
            arr_stream_stop[0].iErrno = 0;

            i_me_error = meIOStreamStop(&arr_stream_stop[0],            // Pointer to an array of meIOStreamStop_t structures  
                                        1,                              // Number of elements in the stream stop array above
                                        ME_IO_STREAM_STOP_NO_FLAGS);    // No flags

            if(i_me_error != ME_ERRNO_SUCCESS)
            {
                printf("****    meIOStreamStop - Error: %d    ****\n\n", i_me_error);

                printf("Press any key to terminate\n");
                
                _getch();
            
                meClose(0);
            
                return(-1);
            }

            break;
        }
    
        if(i_status == ME_STATUS_BUSY)
        {
            printf("Acquisition running - Count: %d\n", i_count);
        }
        else if(i_status == ME_STATUS_IDLE)
        {
            printf("Acquisition ended - Count: %d\n", i_count);

            break;
        }
        else
        {
            printf("Unknown status: 0x%08X - Count: %d\n", i_status, i_count);
        }

        Sleep(400);
    }

    // The acquisition should stop after exactly ACQUIRE_CHANNEL_LISTS channel lists 
    // have been acquired. This means after ACQUIRE_CHANNEL_LISTS * NUMBER_OF_CHANNELS 
    // values have been acquired. 
    //
    // We check this here and print an error message if it is not the case.


    if(i_count != ACQUIRE_NUMBER_VALUES)
    {
        printf("****    Number of values incorrect should be: %d, is: %d    ****\n\n", ACQUIRE_NUMBER_VALUES, i_count);

        printf("Press any key to continue\n");
    
        _getch();

        if(i_count > ACQUIRE_NUMBER_VALUES)
        {
            i_count = ACQUIRE_NUMBER_VALUES;
        }
    }

    int i_buffer[ACQUIRE_NUMBER_VALUES];

    i_me_error = meIOStreamRead(i_stream_read_device,           // Device index
                                i_stream_read_subdevice,        // Subdevice index, 
                                ME_READ_MODE_NONBLOCKING,       // Read mode 
                                &i_buffer[0],                   // Acquired data values returned in this buffer
                                &i_count,                       // Size of buffer in data values - Number of data values returned here
                                ME_IO_STREAM_READ_NO_FLAGS);    // No flags

    if(i_me_error != ME_ERRNO_SUCCESS)
    {
        printf("****    meIOStreamRead - Error: %d    ****\n\n", i_me_error);

        printf("Press any key to end the acquisition\n");
    
        _getch();

        return(-1);
    }

    for(int index_value = 0; index_value < i_count; index_value++)
    {
        double d_volt;
        
        i_me_error =  meUtilityDigitalToPhysical(   d_phys_min,                         // Min physical value
                                                    d_phys_max,                         // Max physical value
                                                    i_digital_max,                      // Maximum digital value
                                                    i_buffer[index_value],              // Digital value to convert
                                                    ME_MODULE_TYPE_MULTISIG_NONE,       // Module type - not used
                                                    0.0,                                // Reference value - not used
                                                    &d_volt                         );  // Converted physical value returned here

        printf("%5d - %5.2lf V    ", i_buffer[index_value], d_volt);

        if( ( (index_value + 1) % NUMBER_OF_CHANNELS ) == 0 )
        {
            printf("\n");
        }
    }

    printf("Acquisition ended\n\n");

    // ---------------------------------------------------------------------
    

    meClose(0);
    
    printf("Program completed successfully - Press any key to terminate\n\n");

    _getch();
    
    return 0;
}

